@@ -1,13 +1,16 @@
 """Implements Checkpoint functionality for easy dumping and restoration of any `NamedObject` in negmas."""
 
+from __future__ import annotations
+
 import shutil
+from os import PathLike
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Type, Union
+from typing import Any, Callable
 
 import numpy as np
 
-from negmas.common import NamedObject
-from negmas.helpers import load
+from negmas.helpers.inout import load
+from negmas.types import NamedObject
 
 
 class CheckpointMixin:
@@ -17,9 +20,9 @@ class CheckpointMixin:
         self,
         step_attrib: str = "current_step",
         every: int = 1,
-        folder: Optional[Union[str, Path]] = None,
-        filename: str = None,
-        info: Dict[str, Any] = None,
+        folder: PathLike | None = None,
+        filename: str | None = None,
+        info: dict[str, Any] | None = None,
         exist_ok: bool = True,
         single: bool = True,
     ):
@@ -50,7 +53,7 @@ class CheckpointMixin:
         self.__step_atrrib = step_attrib
         self.__checkpoint_filename = filename
 
-    def checkpoint_on_step_started(self) -> Optional[Path]:
+    def checkpoint_on_step_started(self) -> Path | None:
         """Should be called on every step to save checkpoints as needed.
 
         Returns:
@@ -74,7 +77,7 @@ class CheckpointMixin:
                 step_attribs=(self.__step_atrrib,),
             )
 
-    def checkpoint_final_step(self) -> Optional[Path]:
+    def checkpoint_final_step(self) -> Path | None:
         """Should be called at the end of the simulation to save the final state
 
         Remarks:
@@ -96,13 +99,23 @@ class CheckpointMixin:
 class CheckpointRunner:
     """Runs an object based on its checkpoints saved in an earlier run"""
 
+    def register_callback(self, callback: Callable[[NamedObject, int], None]) -> None:
+        """Registers a callback to be called whenever a new step is loaded
+
+        Args:
+            callback: A callable that takes the named object (after it is loaded and an integer specifying the step
+                      number and returns None.
+
+        """
+        self.__callbacks.append(callback)
+
     def __init__(
         self,
-        folder: Union[str, Path],
-        id: str = None,
-        callback: Callable[[NamedObject, int], None] = None,
+        folder: str | Path,
+        id: str | None = None,
+        callback: Callable[[NamedObject, int], None] | None = None,
         watch: bool = False,
-        object_type: Type[NamedObject] = NamedObject,
+        object_type: type[NamedObject] = NamedObject,
     ):
         self.__folder = Path(folder).absolute()
         if id is None:
@@ -117,7 +130,7 @@ class CheckpointRunner:
         )
         self.__sorted_steps = sorted(list(self.__files.keys()))
         self._step_index = -1
-        self.__object: NamedObject = None
+        self.__object: NamedObject | None = None
         self.__object_type = object_type
         self.__callbacks = []
         if callback is not None:
@@ -133,7 +146,43 @@ class CheckpointRunner:
             return -1
         return self.__sorted_steps[self._step_index]
 
-    def step(self) -> Optional[int]:
+    def goto(self, step: int, exact=False) -> int | None:
+        """Goes to the nearest step for the given one returning the exact step number.
+
+        Args:
+
+            step: The step we want to goto
+            exact: If True, must go to the exact step number, otherwise go to the nearest step stored in a checkpoint
+
+        Returns:
+
+            - None if the current step is the nearest to the given step. Otherwise the exact step we moved to
+
+        """
+        if step is None:
+            return None
+        step_index = np.searchsorted(self.__sorted_steps, step, side="left")
+        if step_index > 0 and self.__sorted_steps[step_index - 1] == step:
+            step_index -= 1
+        if not exact:
+            n = len(self.__sorted_steps)
+            if step_index > n:
+                step = n - 1
+            else:
+                step = self.__sorted_steps[step_index]
+
+        if self._step_index > -1 and step == self.__sorted_steps[self._step_index]:
+            return None
+        filename = self.__files.get(step)
+        if not filename:
+            raise ValueError(f"step {step} has no file")
+        self.__object = self.__object_type.from_checkpoint(filename, return_info=False)
+        self._step_index = step_index
+        for callback in self.__callbacks:
+            callback(self.__object, step)
+        return step
+
+    def step(self) -> int | None:
         """Go one step forward in the stored steps.
 
         Returns:
@@ -152,7 +201,7 @@ class CheckpointRunner:
             pass
 
     @property
-    def loaded_object(self) -> Optional[NamedObject]:
+    def loaded_object(self) -> NamedObject | None:
         """The object stored in the current checkpoint"""
         return self.__object
 
@@ -160,12 +209,12 @@ class CheckpointRunner:
         self,
         copy_past_checkpoints: bool = False,
         every: int = 1,
-        folder: Optional[Union[str, Path]] = None,
-        filename: str = None,
-        info: Dict[str, Any] = None,
+        folder: str | Path | None = None,
+        filename: str | None = None,
+        info: dict[str, Any] | None = None,
         exist_ok: bool = True,
         single: bool = True,
-    ) -> Optional[NamedObject]:
+    ) -> NamedObject | None:
         """
         Creates a copy of the internal object that can be run safely.
 
@@ -200,7 +249,9 @@ class CheckpointRunner:
                 "Cannot copy past checkpoints because no folder for new checkpoints is given"
             )
 
-        if folder is not None:
+        if folder is None:
+            folder = Path()
+        else:
             folder = Path(folder).absolute()
 
         if copy_past_checkpoints:
@@ -211,7 +262,7 @@ class CheckpointRunner:
         x = self.__object
         if isinstance(self.__object, CheckpointMixin):
             CheckpointMixin.checkpoint_init(
-                x,
+                x,  # type: ignore
                 every=every,
                 folder=folder,
                 filename=filename,
@@ -222,61 +273,17 @@ class CheckpointRunner:
         return x
 
     @property
-    def steps(self) -> List[int]:
+    def steps(self) -> list[int]:
         """A list of all stored steps"""
         return self.__sorted_steps
 
-    def register_callback(self, callback: Callable[[NamedObject, int], None]) -> None:
-        """Registers a callback to be called whenever a new step is loaded
-
-        Args:
-            callback: A callable that takes the named object (after it is loaded and an integer specifying the step
-                      number and returns None.
-
-        """
-        self.__callbacks.append(callback)
-
-    def goto(self, step: int, exact=False) -> Optional[step]:
-        """Goes to the nearest step for the given one returning the exact step number.
-
-        Args:
-
-            step: The step we want to goto
-            exact: If True, must go to the exact step number, otherwise go to the nearest step stored in a checkpoint
-
-        Returns:
-
-            - None if the current step is the nearest to the given step. Otherwise the exact step we moved to
-
-        """
-        if step is None:
-            return
-        step_index = np.searchsorted(self.__sorted_steps, step, side="left")
-        if step_index > 0 and self.__sorted_steps[step_index - 1] == step:
-            step_index -= 1
-        if not exact:
-            n = len(self.__sorted_steps)
-            if step_index > n:
-                step = n - 1
-            else:
-                step = self.__sorted_steps[step_index]
-
-        if self._step_index > -1 and step == self.__sorted_steps[self._step_index]:
-            return None
-        filename = self.__files.get(step)
-        self.__object = self.__object_type.from_checkpoint(filename, return_info=False)
-        self._step_index = step_index
-        for callback in self.__callbacks:
-            callback(self.__object, step)
-        return step
-
     def reset(self) -> None:
         """Goes before the first step"""
         self._step_index = -1
         self.__object = None
 
     @property
-    def next_step(self) -> Optional[int]:
+    def next_step(self) -> int | None:
         """Get the  next stored step number (None if it does not exist)"""
         nxt = self._step_index + 1
         if len(self.__sorted_steps) > nxt:
@@ -284,7 +291,7 @@ class CheckpointRunner:
         return None
 
     @property
-    def previous_step(self) -> Optional[int]:
+    def previous_step(self) -> int | None:
         """Get the  previous stored step number (None if it does not exist)"""
         if self._step_index < 0:
             return -1
@@ -294,11 +301,11 @@ class CheckpointRunner:
         return None
 
     @property
-    def last_step(self) -> Optional[int]:
+    def last_step(self) -> int:
         """Get the  last stored step number (None if it does not exist)"""
         return self.__sorted_steps[-1]
 
     @property
-    def first_step(self) -> Optional[int]:
+    def first_step(self) -> int:
         """Get the  first stored step number (None if it does not exist)"""
         return self.__sorted_steps[0]
