@@ -1,14 +1,16 @@
 """Implements a concurrent set of negotiations creating a chain of bilateral negotiations."""
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 from collections import defaultdict, namedtuple
 from dataclasses import dataclass
-from typing import Dict, List, Optional
 
-from ..common import AgentMechanismInterface
+from ..common import NegotiatorMechanismInterface
 from ..mechanisms import Mechanism, MechanismRoundResult, MechanismState
 from ..negotiators import Negotiator
-from ..outcomes import Outcome, ResponseType
-from ..utilities import UtilityFunction
+from ..outcomes import Outcome
+from ..preferences import Preferences
+from ..sao import ResponseType
 
 __all__ = [
     "ChainNegotiationsMechanism",
@@ -22,22 +24,22 @@ __all__ = [
 class Offer:
     """Defines an offer"""
 
-    outcome: "Outcome"
+    outcome: Outcome
     left: bool
     temp: bool
-    partner: str = None
+    partner: str | None = None
 
 
 Agreement = namedtuple("Agreement", ["outcome", "negotiators", "level"])
 """Defines an agreement for a multi-channel mechanism"""
 
 
-class ChainAMI(AgentMechanismInterface):
+class ChainAMI(NegotiatorMechanismInterface):
     def __init__(
         self,
         *args,
-        parent: "ChainNegotiationsMechanism",
-        negotiator: "ChainNegotiator",
+        parent: ChainNegotiationsMechanism,
+        negotiator: ChainNegotiator,
         level: int,
         **kwargs,
     ):
@@ -46,7 +48,7 @@ class ChainAMI(AgentMechanismInterface):
         self.__negotiator = negotiator
         self.__level = level
 
-    def confirm(self, parent: bool) -> None:
+    def confirm(self, parent: bool) -> bool:
         return self.__parent.on_confirm(self.__level, parent)
 
 
@@ -55,17 +57,18 @@ class ChainNegotiator(Negotiator, ABC):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._nmi: ChainAMI
         self.__level = -1
 
     def join(
         self,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        preferences: Preferences | None = None,
+        role: str = "negotiator",
     ) -> bool:
-        to_join = super().join(ami, state, ufun=ufun, role=role)
+        to_join = super().join(nmi, state, preferences=preferences, role=role)
         if to_join:
             self.__level = int(role)
         return to_join
@@ -80,7 +83,7 @@ class ChainNegotiator(Negotiator, ABC):
         Returns:
 
         """
-        self._ami.confirm(left)
+        return self._nmi.confirm(left)
 
     @abstractmethod
     def on_acceptance(self, state: MechanismState, offer: Offer) -> Offer:
@@ -109,7 +112,7 @@ class ChainNegotiator(Negotiator, ABC):
 
     @abstractmethod
     def respond(
-        self, state: MechanismState, outcome: "Outcome", from_left: bool, temp: bool
+        self, state: MechanismState, outcome: Outcome, from_left: bool, temp: bool
     ) -> ResponseType:
         """
         Called to respond to an offer
@@ -134,13 +137,13 @@ class MultiChainNegotiator(Negotiator, ABC):
 
     def join(
         self,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        preferences: Preferences | None = None,
+        role: str = "negotiator",
     ) -> bool:
-        to_join = super().join(ami, state, ufun=ufun, role=role)
+        to_join = super().join(nmi, state, preferences=preferences, role=role)
         if to_join:
             self.__level = int(role)
         return to_join
@@ -155,7 +158,7 @@ class MultiChainNegotiator(Negotiator, ABC):
         Returns:
 
         """
-        self._ami.confirm(left)
+        self._nmi.confirm(left)
 
     @abstractmethod
     def on_acceptance(self, state: MechanismState, offer: Offer) -> Offer:
@@ -186,7 +189,7 @@ class MultiChainNegotiator(Negotiator, ABC):
     def respond(
         self,
         state: MechanismState,
-        outcome: "Outcome",
+        outcome: Outcome,
         from_left: bool,
         temp: bool,
         source: str,
@@ -209,14 +212,16 @@ class MultiChainNegotiator(Negotiator, ABC):
 class ChainNegotiationsMechanism(Mechanism):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.__chain: List[Optional[ChainNegotiator]] = []
+        self.__chain: list[ChainNegotiator | None] = []
         self.__next_agent = 0
         self.__last_proposal: Offer = None
         self.__last_proposer_index: int = -1
-        self.__agreements: Dict[int, Outcome] = defaultdict(lambda: None)
-        self.__temp_agreements: Dict[int, Outcome] = defaultdict(lambda: None)
+        self.__agreements: dict[int, Outcome] = defaultdict(lambda: None)
+        self.__temp_agreements: dict[int, Outcome] = defaultdict(lambda: None)
 
-    def _get_ami(self, negotiator: Negotiator, role: str) -> AgentMechanismInterface:
+    def _get_ami(
+        self, negotiator: Negotiator, role: str
+    ) -> NegotiatorMechanismInterface:
         """
         Returns a chain AMI instead of the standard AMI.
 
@@ -229,15 +234,15 @@ class ChainNegotiationsMechanism(Mechanism):
         """
         return ChainAMI(
             id=self.id,
-            n_outcomes=self.ami.n_outcomes,
-            issues=self.ami.issues,
-            outcomes=self.ami.outcomes,
-            time_limit=self.ami.time_limit,
-            step_time_limit=self.ami.step_time_limit,
-            n_steps=self.ami.n_steps,
-            dynamic_entry=self.ami.dynamic_entry,
-            max_n_agents=self.ami.max_n_agents,
-            annotation=self.ami.annotation,
+            n_outcomes=self.nmi.n_outcomes,
+            issues=self.nmi.outcome_space,
+            outcomes=self.nmi.outcomes,
+            time_limit=self.nmi.time_limit,
+            step_time_limit=self.nmi.step_time_limit,
+            n_steps=self.nmi.n_steps,
+            dynamic_entry=self.nmi.dynamic_entry,
+            max_n_agents=self.nmi.max_n_agents,
+            annotation=self.nmi.annotation,
             parent=self,
             negotiator=negotiator,
             level=int(role) + 1,
@@ -245,18 +250,18 @@ class ChainNegotiationsMechanism(Mechanism):
 
     def add(
         self,
-        negotiator: "Negotiator",
+        negotiator: Negotiator,
         *,
-        ufun: Optional[UtilityFunction] = None,
-        role: Optional[str] = None,
+        preferences: Preferences | None = None,
+        role: str | None = None,
         **kwargs,
-    ) -> Optional[bool]:
+    ) -> bool | None:
         if role is None:
             raise ValueError(
                 "You cannot join this protocol without specifying the role. "
                 "Possible roles are integers >= -1 "
             )
-        added = super().add(negotiator, ufun=ufun, role=role, **kwargs)
+        added = super().add(negotiator, preferences=preferences, role=role, **kwargs)
         if not added:
             return added
         level = int(role) + 1
@@ -268,6 +273,12 @@ class ChainNegotiationsMechanism(Mechanism):
         self.__chain += [None] * (level - len(self.__chain) + 1)
         self.__chain[level] = negotiator
 
+    def _update_next(self) -> None:
+        if self.__last_proposal.left:
+            self.__next_agent = (self.__last_proposer_index - 1) % len(self.__chain)
+        else:
+            self.__next_agent = (self.__last_proposer_index + 1) % len(self.__chain)
+
     def round(self) -> MechanismRoundResult:
 
         # check that the chain is complete
@@ -353,13 +364,7 @@ class ChainNegotiationsMechanism(Mechanism):
         self._update_next()
         return MechanismRoundResult()
 
-    def _update_next(self) -> None:
-        if self.__last_proposal.left:
-            self.__next_agent = (self.__last_proposer_index - 1) % len(self.__chain)
-        else:
-            self.__next_agent = (self.__last_proposer_index + 1) % len(self.__chain)
-
-    def on_confirm(self, level: int, left: bool) -> None:
+    def on_confirm(self, level: int, left: bool) -> bool:
         """
         Called by negotiators to confirm their temporary accepted agreements
 
@@ -376,23 +381,26 @@ class ChainNegotiationsMechanism(Mechanism):
         if self.__agreements[level] is not None:
             raise ValueError(f"An agreement already exists at level {level}")
         self.__agreements[level] = self.__temp_agreements[level]
+        return True
 
 
 class MultiChainNegotiationsMechanism(Mechanism):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.__chain: List[List[MultiChainNegotiator]] = []
+        self.__chain: list[list[MultiChainNegotiator]] = []
         self.__next_agent_level = 0
         self.__next_agent_number = 0
         self.__last_proposal: Offer = None
         self.__last_proposer_level: int = -1
         self.__last_proposer_number: int = -1
-        self.__agreements: Dict[int, Agreement] = defaultdict(lambda: None)
-        self.__temp_agreements: Dict[int, Agreement] = defaultdict(lambda: None)
-        self.__level: Dict[str, int] = {}
-        self.__number: Dict[str, int] = {}
+        self.__agreements: dict[int, Agreement] = defaultdict(lambda: None)
+        self.__temp_agreements: dict[int, Agreement] = defaultdict(lambda: None)
+        self.__level: dict[str, int] = {}
+        self.__number: dict[str, int] = {}
 
-    def _get_ami(self, negotiator: Negotiator, role: str) -> AgentMechanismInterface:
+    def _get_ami(
+        self, negotiator: Negotiator, role: str
+    ) -> NegotiatorMechanismInterface:
         """
         Returns a chain AMI instead of the standard AMI.
 
@@ -405,15 +413,15 @@ class MultiChainNegotiationsMechanism(Mechanism):
         """
         return ChainAMI(
             id=self.id,
-            n_outcomes=self.ami.n_outcomes,
-            issues=self.ami.issues,
-            outcomes=self.ami.outcomes,
-            time_limit=self.ami.time_limit,
-            step_time_limit=self.ami.step_time_limit,
-            n_steps=self.ami.n_steps,
-            dynamic_entry=self.ami.dynamic_entry,
-            max_n_agents=self.ami.max_n_agents,
-            annotation=self.ami.annotation,
+            n_outcomes=self.nmi.n_outcomes,
+            issues=self.nmi.outcome_space,
+            outcomes=self.nmi.outcomes,
+            time_limit=self.nmi.time_limit,
+            step_time_limit=self.nmi.step_time_limit,
+            n_steps=self.nmi.n_steps,
+            dynamic_entry=self.nmi.dynamic_entry,
+            max_n_agents=self.nmi.max_n_agents,
+            annotation=self.nmi.annotation,
             parent=self,
             negotiator=negotiator,
             level=int(role) + 1,
@@ -421,18 +429,18 @@ class MultiChainNegotiationsMechanism(Mechanism):
 
     def add(
         self,
-        negotiator: "Negotiator",
+        negotiator: Negotiator,
         *,
-        ufun: Optional[UtilityFunction] = None,
-        role: Optional[str] = None,
+        preferences: Preferences | None = None,
+        role: str | None = None,
         **kwargs,
-    ) -> Optional[bool]:
+    ) -> bool | None:
         if role is None:
             raise ValueError(
                 "You cannot join this protocol without specifying the role. "
                 "Possible roles are integers >= -1 "
             )
-        added = super().add(negotiator, ufun=ufun, role=role, **kwargs)
+        added = super().add(negotiator, preferences=preferences, role=role, **kwargs)
         if not added:
             return added
         level = int(role) + 1
@@ -446,6 +454,17 @@ class MultiChainNegotiationsMechanism(Mechanism):
         self.__level[negotiator.id] = level
         self.__number[negotiator.id] = len(self.__chain[level]) - 1
 
+    def _update_next(self) -> None:
+        if self.__last_proposal.left:
+            self.__next_agent_level = (self.__last_proposer_level - 1) % len(
+                self.__chain
+            )
+        else:
+            self.__next_agent_level = (self.__last_proposer_level + 1) % len(
+                self.__chain
+            )
+        self.__next_agent_number = self.__number[self.__last_proposal.partner]
+
     def round(self) -> MechanismRoundResult:
 
         # check that the chain is complete
@@ -536,17 +555,6 @@ class MultiChainNegotiationsMechanism(Mechanism):
         self._update_next()
         return MechanismRoundResult()
 
-    def _update_next(self) -> None:
-        if self.__last_proposal.left:
-            self.__next_agent_level = (self.__last_proposer_level - 1) % len(
-                self.__chain
-            )
-        else:
-            self.__next_agent_level = (self.__last_proposer_level + 1) % len(
-                self.__chain
-            )
-        self.__next_agent_number = self.__number[self.__last_proposal.partner]
-
     def on_confirm(self, level: int, left: bool) -> None:
         """
         Called by negotiators to confirm their temporary accepted agreements
