@@ -1,37 +1,42 @@
+from __future__ import annotations
+
 import copy
 import time
 from abc import abstractmethod
-from typing import Callable, List, Optional, Tuple, Union
+from typing import Callable
 
 import numpy as np
 
-from ..common import AgentMechanismInterface, MechanismState
-from ..modeling import AdaptiveDiscreteAcceptanceModel
-from ..outcomes import Outcome, ResponseType
-from ..sao import AspirationNegotiator, SAONegotiator
-from ..utilities import (
-    IPUtilityFunction,
-    MappingUtilityFunction,
-    UtilityDistribution,
-    UtilityValue,
-)
+from ..common import MechanismState, NegotiatorMechanismInterface, Value
+from ..helpers.prob import ScipyDistribution
+from ..models.acceptance import AdaptiveDiscreteAcceptanceModel, DiscreteAcceptanceModel
+from ..outcomes import Outcome
+from ..preferences import IPUtilityFunction, MappingUtilityFunction, Preferences
+from ..sao import AspirationNegotiator, ResponseType, SAONegotiator
 from .common import _locs, _uppers
-from .expectors import MeanExpector
+from .expectors import Expector, MeanExpector
+from .strategy import EStrategy
+from .user import User
 
 __all__ = ["BaseElicitor"]
 
 
 class BaseElicitor(SAONegotiator):
+    def accuracy_limit(self, cost: float) -> float:
+        """The accuracy limit given the cost and `epsilon`."""
+        return 0.5 * max(self.epsilon, cost)
+
     def __init__(
         self,
-        user: "User",
+        user: User,
         *,
-        strategy: Optional["EStrategy"] = None,
+        strategy: EStrategy | None = None,
         base_negotiator: SAONegotiator = AspirationNegotiator(),
-        opponent_model_factory: Optional[
-            Callable[["AgentMechanismInterface"], "DiscreteAcceptanceModel"]
-        ] = lambda x: AdaptiveDiscreteAcceptanceModel.from_negotiation(ami=x),
-        expector_factory: Union["Expector", Callable[[], "Expector"]] = MeanExpector,
+        opponent_model_factory: None
+        | (
+            Callable[[NegotiatorMechanismInterface], DiscreteAcceptanceModel]
+        ) = lambda x: AdaptiveDiscreteAcceptanceModel.from_negotiation(nmi=x),
+        expector_factory: Expector | Callable[[], Expector] = MeanExpector,
         single_elicitation_per_round=False,
         continue_eliciting_past_reserved_val=False,
         epsilon=0.001,
@@ -98,13 +103,72 @@ class BaseElicitor(SAONegotiator):
         if strategy is not None:
             strategy.resolution = max(self.acc_limit, strategy.resolution)
 
+    def init_elicitation(
+        self,
+        preferences: None
+        | (IPUtilityFunction | ScipyDistribution | list[ScipyDistribution]),
+        **kwargs,
+    ) -> None:
+        """
+        Called once to initialize the elicitation process
+
+        Args:
+            preferences: The probabilistic utility function
+            **kwargs:
+
+        Remarks:
+            - If no `ufun` is given one will be created with 0-1 uniform distributions and
+              zero reserved value.
+            - If a single `Distribution` is given as `ufun`, it is repeated for all
+              outcomes (and the reserved value is set to zero).
+            - If a list of `Distribution` s is given, it must have the same length as
+              the list of outcomes of this negotiation and is used to set the `ufun`.
+            - The opponent model
+
+        """
+        nmi = self._nmi
+        if nmi is None:
+            raise ValueError(f"Unkown NMI")
+        self.elicitation_history = []
+        outcomes = list(nmi.discrete_outcomes())
+        self.indices = dict(zip(outcomes, range(len(outcomes))))
+        self.offerable_outcomes = []
+        self._elicitation_time = 0.0
+        if self.opponent_model_factory is None:
+            self.opponent_model = None
+        else:
+            self.opponent_model = self.opponent_model_factory(nmi)
+            self.base_negotiator.opponent_model = self.opponent_model
+        if preferences is None:
+            dists = [
+                ScipyDistribution(type="uniform", loc=0.0, scale=1.0) for _ in outcomes
+            ]
+            preferences = IPUtilityFunction(
+                outcomes=outcomes, distributions=dists, reserved_value=0.0
+            )
+        elif isinstance(preferences, ScipyDistribution):
+            preferences = [copy.copy(preferences) for _ in outcomes]
+            preferences = IPUtilityFunction(
+                outcomes=outcomes, distributions=preferences, reserved_value=0.0
+            )
+        elif (
+            isinstance(preferences, list)
+            and len(preferences) > 0
+            and isinstance(preferences[0], ScipyDistribution)
+        ):
+            preferences = IPUtilityFunction(
+                outcomes=outcomes, distributions=preferences, reserved_value=0.0
+            )
+        self.set_preferences(preferences)
+        self.initial_utility_priors = copy.copy(preferences)
+
     def join(
         self,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        preferences: Preferences | None = None,
+        role: str = "negotiator",
         **kwargs,
     ) -> bool:
         """
@@ -116,17 +180,17 @@ class BaseElicitor(SAONegotiator):
               result of the expector applied to the corresponding utility value.
             - The reserved value of the created ufun is set to -inf
         """
-        if ufun is None:
-            ufun = IPUtilityFunction(outcomes=ami.outcomes, reserved_value=0.0)
-        if not super().join(ami=ami, state=state, ufun=ufun, role=role):
+        if preferences is None:
+            preferences = IPUtilityFunction(outcomes=nmi.outcomes, reserved_value=0.0)
+        if not super().join(nmi=nmi, state=state, preferences=preferences, role=role):
             return False
-        self.expect = self.expector_factory(self._ami)
-        self.init_elicitation(ufun=ufun, **kwargs)
+        self.expect = self.expector_factory(self._nmi)
+        self.init_elicitation(preferences=preferences, **kwargs)
         self.base_negotiator.join(
-            ami,
+            nmi,
             state,
-            ufun=MappingUtilityFunction(
-                mapping=lambda x: self.expect(self.utility_function(x), state=state),
+            preferences=MappingUtilityFunction(
+                mapping=lambda x: self.expect(self.preferences(x), state=state),
                 reserved_value=float("-inf"),
             ),
         )
@@ -137,18 +201,18 @@ class BaseElicitor(SAONegotiator):
         base_negotiator."""
         self.base_negotiator.on_negotiation_start(state=state)
 
-    def utility_distributions(self) -> List[UtilityDistribution]:
+    def utility_distributions(self) -> list[ScipyDistribution]:
         """
-        Returns a `UtilityDistribution` for every outcome
+        Returns a `Distribution` for every outcome
         """
-        if self.utility_function is None:
-            return [None] * len(self._ami.outcomes)
-        if self.utility_function.base_type == "ip":
-            return list(self.utility_function.distributions.values())
+        if self.preferences is None:
+            return [None] * len(self._nmi.outcomes)
+        if self.preferences.base_type == "ip":
+            return list(self.preferences.distributions.values())
         else:
-            return [self.utility_function(o) for o in self._ami.outcomes]
+            return [self.preferences(o) for o in self._nmi.outcomes]
 
-    def user_ufun(self, outcome: Optional["Outcome"]) -> float:
+    def user_preferences(self, outcome: Outcome | None) -> float:
         """
         Finds the total utility obtained by the user for this outcome after
         discounting elicitation cost.
@@ -193,8 +257,20 @@ class BaseElicitor(SAONegotiator):
         """
         return min(_locs(self.utility_distributions()))
 
+    def on_opponent_model_updated(
+        self, outcomes: list[Outcome], old: list[float], new: list[float]
+    ) -> None:
+        """
+        Called whenever an opponents model is updated.
+
+        Args:
+            outcomes: A list of outcomes for which the acceptance probability are changed
+            old: The old acceptance probability
+            new: The new acceptance probability
+        """
+
     def on_partner_proposal(
-        self, state: MechanismState, partner_id: str, offer: "Outcome"
+        self, state: MechanismState, partner_id: str, offer: Outcome
     ):
         """
         Called when one of the partners propose (only if enable_callbacks is set
@@ -221,8 +297,8 @@ class BaseElicitor(SAONegotiator):
         self,
         state: MechanismState,
         partner_id: str,
-        outcome: "Outcome",
-        response: "ResponseType",
+        outcome: Outcome,
+        response: ResponseType,
     ):
         """
         Called when one of the partners respond  (only if enable_callbacks is set
@@ -252,73 +328,17 @@ class BaseElicitor(SAONegotiator):
             new_probs = [self.opponent_model.probability_of_acceptance(outcome)]
             self.on_opponent_model_updated([outcome], old=old_probs, new=new_probs)
 
-    def respond_(self, state: MechanismState, offer: "Outcome") -> ResponseType:
-        """
-        Called by the mechanism directly (through `counter` ) to respond to offers.
+    def before_eliciting(self) -> None:
+        """Called by apply just before continuously calling elicit_single"""
+
+    @abstractmethod
+    def elicit_single(self, state: MechanismState) -> None:
+        """Does a single elicitation act
 
         Args:
             state: mechanism state
-            offer: the offer to respond to
-
-        Remarks:
-            - Does the following steps:
-
-                1. Finds the the best offer using `best_offer` and uses the base negotiator
-                   to respond if that offer was `None`
-                2. Looks at `offerable_outcomes` and applies the elicitation strategy (one
-                   step) to the outcome if it was not offerable (or if there are no offerable
-                   outcomes defined).
-                3. Finds the utility of the offer using `utility_function` not taking into accout
-                   elicitation cost and uses the base negotiator if that fails (i.e. `utility_function`
-                   returns `None`).
-                4. Finds the expected utility of the offer using the `expect` () method which calls the
-                   expector passed during construction.
-                5. If the maximum attainable utility now (judging from the current estimation of
-                   the utility value of each outcome taking elicitation cost into account) is less
-                   than the reserved value, end the negotiation
-                6. If the utility of my best offer (returned from `best_offer`) is less than the offered
-                   utility, accept the offer
-                7. Otherwise, call bhe base negotiator to respond.
-
         """
-        my_offer, meu = self.best_offer(state=state)
-        if my_offer is None:
-            return self.base_negotiator.respond_(state=state, offer=offer)
-        if (
-            self.strategy
-            and self.offerable_outcomes is not None
-            and offer not in self.offerable_outcomes
-        ):
-            self.strategy.apply(user=self.user, outcome=offer)
-        offered_utility = self.utility_function(offer)
-        if offered_utility is None:
-            return self.base_negotiator.respond_(state=state, offer=offer)
-        offered_utility = self.expect(offered_utility, state=state)
-        if (
-            self.maximum_attainable_utility() - self.user.total_cost
-            < self.reserved_value
-        ):
-            return ResponseType.END_NEGOTIATION
-        if meu < offered_utility:
-            return ResponseType.ACCEPT_OFFER
-        else:
-            return self.base_negotiator.respond_(state=state, offer=offer)
-
-    def propose(self, state: MechanismState) -> "Outcome":
-        """
-        Called to propose an outcome
-
-        Args:
-            state: mechanism state
-
-        Remarks:
-
-            - if the negotiator `can_elicit`, it will `elicit`.
-            - always then calls the base negotiator to propose.
-        """
-        if self.can_elicit():
-            self.elicit(state=state)
-        return self.base_negotiator.propose(state=state)
+        raise NotImplementedError()
 
     def elicit(self, state: MechanismState) -> None:
         """
@@ -363,72 +383,21 @@ class BaseElicitor(SAONegotiator):
         self._elicitation_time += elapsed
         self.asking_time += elapsed
 
-    def accuracy_limit(self, cost: float) -> float:
-        """The accuracy limit given the cost and `epsilon`."""
-        return 0.5 * max(self.epsilon, cost)
-
-    def init_elicitation(
-        self,
-        ufun: Optional[
-            Union[
-                "IPUtilityFunction", "UtilityDistribution", List["UtilityDistribution"]
-            ]
-        ],
-        **kwargs,
-    ) -> None:
-        """
-        Called once to initialize the elicitation process
+    def utility_on_rejection(self, outcome: Outcome, state: MechanismState) -> Value:
+        """Estimated utility if this outcome rejected at this state.
 
         Args:
-            ufun: The probabilistic utility function
-            **kwargs:
+            outcome: The outcome tested
+            state: The mechanism state
 
         Remarks:
-            - If no `ufun` is given one will be created with 0-1 uniform distributions and
-              zero reserved value.
-            - If a single `UtilityDistribution` is given as `ufun`, it is repeated for all
-              outcomes (and the reserved value is set to zero).
-            - If a list of `UtilityDistribution` s is given, it must have the same length as
-              the list of outcomes of this negotiation and is used to set the `ufun`.
-            - The opponent model
-
+            - MUST be implemented by any Elicitor.
         """
-        ami = self._ami
-        self.elicitation_history = []
-        self.indices = dict(zip(ami.outcomes, range(ami.n_outcomes)))
-        self.offerable_outcomes = []
-        self._elicitation_time = 0.0
-        if self.opponent_model_factory is None:
-            self.opponent_model = None
-        else:
-            self.opponent_model = self.opponent_model_factory(ami)
-            self.base_negotiator.opponent_model = self.opponent_model
-        outcomes = ami.outcomes
-        if ufun is None:
-            dists = [
-                UtilityDistribution(dtype="uniform", loc=0.0, scale=1.0)
-                for _ in outcomes
-            ]
-            ufun = IPUtilityFunction(
-                outcomes=outcomes, distributions=dists, reserved_value=0.0
-            )
-        elif isinstance(ufun, UtilityDistribution):
-            ufun = [copy.copy(ufun) for _ in outcomes]
-            ufun = IPUtilityFunction(
-                outcomes=outcomes, distributions=ufun, reserved_value=0.0
-            )
-        elif (
-            isinstance(ufun, list)
-            and len(ufun) > 0
-            and isinstance(ufun[0], UtilityDistribution)
-        ):
-            ufun = IPUtilityFunction(
-                outcomes=outcomes, distributions=ufun, reserved_value=0.0
-            )
-        self.utility_function = ufun
-        self.initial_utility_priors = copy.copy(ufun)
+        raise NotImplementedError(
+            f"Must override utility_on_rejection in {self.__class__.__name__}"
+        )
 
-    def offering_utility(self, outcome, state) -> UtilityValue:
+    def offering_utility(self, outcome, state) -> Value:
         """
         returns expected utlity of offering `outcome` in `state`
 
@@ -443,40 +412,16 @@ class BaseElicitor(SAONegotiator):
             - returns $u(o) p(o) + ru(o) (1-p(o))$ where $p$ is the opponent model,
               $u$ is the utility function, and $r$ is the utility in case of rejections.
             - `state` is needed when calculating $r(o)$ by calling `utility_on_rejection`.
-            - Note that if $u$ or $r$ return a `UtilityDistribution`, this method will return
-              a `UtilityDistribution` not a real number.
+            - Note that if $u$ or $r$ return a `Distribution`, this method will return
+              a `Distribution` not a real number.
         """
         if self.opponent_model is None:
-            return self.utility_function(outcome)
-        u = self.utility_function(outcome)
+            return self.preferences(outcome)
+        u = self.preferences(outcome)
         p = self.opponent_model.probability_of_acceptance(outcome)
         return p * u + (1 - p) * self.utility_on_rejection(outcome, state=state)
 
-    def offering_utilities(self, state) -> np.ndarray:
-        """
-        Calculates the offering utility for all outcomes
-
-        Args:
-            state: Calculates the state at which the offering utilities are to be calculated
-
-        Returns:
-            An ndarray with the offering utility of every outcome (in order)
-
-        Remarks:
-            - This is just a faster version of calling `offering_utility` in a loop.
-
-        """
-        us = np.asarray(self.utility_distributions())
-        ps = np.asarray(self.opponent_model.acceptance_probabilities())
-        return ps * us + (1 - ps) * np.asarray(self.utilities_on_rejection(state=state))
-
-    def utility_on_acceptance(self, outcome: "Outcome") -> UtilityValue:
-        """
-        The utility of acceptance which is simply the utility function applied to `outcome`.
-        """
-        return self.utility_function(outcome)
-
-    def best_offer(self, state: "MechanismState") -> Tuple[Optional["Outcome"], float]:
+    def best_offer(self, state: MechanismState) -> tuple[Outcome | None, float]:
         """The outcome with maximum expected utility given the expector and its utility
 
         Args:
@@ -507,23 +452,104 @@ class BaseElicitor(SAONegotiator):
                 best, best_utility, bsf = outcome, utilitiy, expected_utility
         return best, self.expect(best_utility, state=state)
 
-    def utility_on_rejection(
-        self, outcome: "Outcome", state: MechanismState
-    ) -> UtilityValue:
-        """Estimated utility if this outcome rejected at this state.
+    def respond_(self, state: MechanismState, offer: Outcome) -> ResponseType:
+        """
+        Called by the mechanism directly (through `counter` ) to respond to offers.
 
         Args:
-            outcome: The outcome tested
-            state: The mechanism state
+            state: mechanism state
+            offer: the offer to respond to
 
         Remarks:
-            - MUST be implemented by any Elicitor.
-        """
-        raise NotImplementedError(
-            f"Must override utility_on_rejection in {self.__class__.__name__}"
-        )
+            - Does the following steps:
 
-    def utilities_on_rejection(self, state: MechanismState) -> List[UtilityValue]:
+                1. Finds the the best offer using `best_offer` and uses the base negotiator
+                   to respond if that offer was `None`
+                2. Looks at `offerable_outcomes` and applies the elicitation strategy (one
+                   step) to the outcome if it was not offerable (or if there are no offerable
+                   outcomes defined).
+                3. Finds the utility of the offer using `utility_function` not taking into accout
+                   elicitation cost and uses the base negotiator if that fails (i.e. `utility_function`
+                   returns `None`).
+                4. Finds the expected utility of the offer using the `expect` () method which calls the
+                   expector passed during construction.
+                5. If the maximum attainable utility now (judging from the current estimation of
+                   the utility value of each outcome taking elicitation cost into account) is less
+                   than the reserved value, end the negotiation
+                6. If the utility of my best offer (returned from `best_offer`) is less than the offered
+                   utility, accept the offer
+                7. Otherwise, call bhe base negotiator to respond.
+
+        """
+        my_offer, meu = self.best_offer(state=state)
+        if my_offer is None:
+            return self.base_negotiator.respond_(state=state, offer=offer)
+        if (
+            self.strategy
+            and self.offerable_outcomes is not None
+            and offer not in self.offerable_outcomes
+        ):
+            self.strategy.apply(user=self.user, outcome=offer)
+        offered_utility = self.preferences(offer)
+        if offered_utility is None:
+            return self.base_negotiator.respond_(state=state, offer=offer)
+        offered_utility = self.expect(offered_utility, state=state)
+        if (
+            self.maximum_attainable_utility() - self.user.total_cost
+            < self.reserved_value
+        ):
+            return ResponseType.END_NEGOTIATION
+        if meu < offered_utility:
+            return ResponseType.ACCEPT_OFFER
+        else:
+            return self.base_negotiator.respond_(state=state, offer=offer)
+
+    @abstractmethod
+    def can_elicit(self) -> bool:
+        """Returns whether we can do more elicitation"""
+        raise NotImplementedError()
+
+    def propose(self, state: MechanismState) -> Outcome:
+        """
+        Called to propose an outcome
+
+        Args:
+            state: mechanism state
+
+        Remarks:
+
+            - if the negotiator `can_elicit`, it will `elicit`.
+            - always then calls the base negotiator to propose.
+        """
+        if self.can_elicit():
+            self.elicit(state=state)
+        return self.base_negotiator.propose(state=state)
+
+    def offering_utilities(self, state) -> np.ndarray:
+        """
+        Calculates the offering utility for all outcomes
+
+        Args:
+            state: Calculates the state at which the offering utilities are to be calculated
+
+        Returns:
+            An ndarray with the offering utility of every outcome (in order)
+
+        Remarks:
+            - This is just a faster version of calling `offering_utility` in a loop.
+
+        """
+        us = np.asarray(self.utility_distributions())
+        ps = np.asarray(self.opponent_model.acceptance_probabilities())
+        return ps * us + (1 - ps) * np.asarray(self.preferences(state=state))
+
+    def utility_on_acceptance(self, outcome: Outcome) -> Value:
+        """
+        The utility of acceptance which is simply the utility function applied to `outcome`.
+        """
+        return self.preferences(outcome)
+
+    def utilities_on_rejection(self, state: MechanismState) -> list[Value]:
         """Finds the utility of rejection for all outputs.
 
         Remarks:
@@ -532,41 +558,12 @@ class BaseElicitor(SAONegotiator):
         """
         return [
             self.utility_on_rejection(outcome=outcome, state=state)
-            for outcome in self._ami.outcomes
+            for outcome in self._nmi.outcomes
         ]
 
-    def on_opponent_model_updated(
-        self, outcomes: List[Outcome], old: List[float], new: List[float]
-    ) -> None:
-        """
-        Called whenever an opponents model is updated.
-
-        Args:
-            outcomes: A list of outcomes for which the acceptance probability are changed
-            old: The old acceptance probability
-            new: The new acceptance probability
-        """
+    def __getattr__(self, item):
+        return getattr(self.base_negotiator, item)
         # TODO extend this to take the partner_id as a parameter to handle multiparty negotiation
 
     def __str__(self):
         return f"{self.name}"
-
-    def before_eliciting(self) -> None:
-        """Called by apply just before continuously calling elicit_single"""
-
-    @abstractmethod
-    def can_elicit(self) -> bool:
-        """Returns whether we can do more elicitation"""
-        raise NotImplementedError()
-
-    @abstractmethod
-    def elicit_single(self, state: MechanismState) -> None:
-        """Does a single elicitation act
-
-        Args:
-            state: mechanism state
-        """
-        raise NotImplementedError()
-
-    def __getattr__(self, item):
-        return getattr(self.base_negotiator, item)
