@@ -1,9 +1,10 @@
-from abc import ABC, abstractmethod
-from typing import Optional, Union
+from __future__ import annotations
 
-from ..common import AgentMechanismInterface, MechanismState
-from ..negotiators import AspirationMixin
-from ..utilities import UtilityValue
+from abc import ABC, abstractmethod
+from typing import Literal
+
+from ..common import MechanismState, NegotiatorMechanismInterface, Value
+from ..negotiators.helpers import PolyAspiration
 
 
 class Expector(ABC):
@@ -14,8 +15,8 @@ class Expector(ABC):
     reduction method that receives a utility value and return a real number.
     """
 
-    def __init__(self, ami: Optional[AgentMechanismInterface] = None):
-        self.ami = ami
+    def __init__(self, nmi: NegotiatorMechanismInterface | None = None):
+        self.nmi = nmi
 
     @abstractmethod
     def is_dependent_on_negotiation_info(self) -> bool:
@@ -23,7 +24,7 @@ class Expector(ABC):
         ...
 
     @abstractmethod
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         ...
 
 
@@ -32,22 +33,22 @@ class StaticExpector(Expector):
         return False
 
     @abstractmethod
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         ...
 
 
 class MeanExpector(StaticExpector):
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         return u if isinstance(u, float) else float(u)
 
 
 class MaxExpector(StaticExpector):
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         return u if isinstance(u, float) else u.loc + u.scale
 
 
 class MinExpector(StaticExpector):
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         return u if isinstance(u, float) else u.loc
 
 
@@ -55,9 +56,9 @@ class BalancedExpector(Expector):
     def is_dependent_on_negotiation_info(self) -> bool:
         return True
 
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         if state is None:
-            state = self.ami.state
+            state = self.nmi.state
         if isinstance(u, float):
             return u
         else:
@@ -66,28 +67,29 @@ class BalancedExpector(Expector):
             )
 
 
-class AspiringExpector(Expector, AspirationMixin):
+class AspiringExpector(Expector):
     def __init__(
         self,
-        ami: Optional[AgentMechanismInterface] = None,
+        nmi: NegotiatorMechanismInterface | None = None,
         max_aspiration=1.0,
-        aspiration_type: Union[str, int, float] = "linear",
-    ) -> bool:
-        Expector.__init__(self, ami=ami)
-        self.aspiration_init(
-            max_aspiration=max_aspiration,
-            aspiration_type=aspiration_type,
-            above_reserved_value=False,
-        )
+        aspiration_type: (
+            Literal["linear"] | Literal["conceder"] | Literal["boulware"] | float
+        ) = "linear",
+    ):
+        Expector.__init__(self, nmi=nmi)
+        self.__asp = PolyAspiration(max_aspiration, aspiration_type)
+
+    def utility_at(self, x):
+        return self.__asp.utility_at(x)
 
     def is_dependent_on_negotiation_info(self) -> bool:
         return True
 
-    def __call__(self, u: UtilityValue, state: MechanismState = None) -> float:
+    def __call__(self, u: Value, state: MechanismState = None) -> float:
         if state is None:
-            state = self.ami.state
+            state = self.nmi.state
         if isinstance(u, float):
             return u
         else:
-            alpha = self.aspiration(state.relative_time)
+            alpha = self.__asp.utility_at(state.relative_time)
             return alpha * u.loc + (1.0 - alpha) * (u.loc + u.scale)
