@@ -1,8 +1,8 @@
-from typing import List, Optional, Tuple, Union
+from __future__ import annotations
 
-from ..common import AgentMechanismInterface
-from ..outcomes import Outcome, outcome_as_tuple
-from ..utilities import IPUtilityFunction, UtilityDistribution, UtilityValue
+from ..common import NegotiatorMechanismInterface, Value
+from ..helpers.prob import ScipyDistribution
+from ..preferences import IPUtilityFunction
 from .common import _loc, _upper
 from .queries import Answer, Query, RangeConstraint
 
@@ -54,60 +54,9 @@ class EStrategy:
             "dpingpong{f}",
         ]
 
-    def apply(
-        self, user: "User", outcome: "Outcome"
-    ) -> Tuple[Optional[UtilityValue], Optional["QResponse"]]:
-        """Do the elicitation and incur the cost.
-
-        Remarks:
-
-            - This function returns a uniform distribution whenever it returns a distribution
-            - Can return `None` which indicates that elicitation failed
-            - If it could find an exact value, it will return a `float` not a `UtilityDistribution`
-
-        """
-
-        lower, upper, _ = self.lower, self.upper, self.outcomes
-        index = self.indices[outcome_as_tuple(outcome)]
-        lower, upper = lower[index], upper[index]
-        epsilon = self.resolution
-
-        if abs(upper - lower) < epsilon:
-            return (upper + lower) / 2, None
-
-        if self.stop_at_cost and abs(upper - lower) < 2 * user.cost:
-            return (upper + lower) / 2, None
-
-        reply = None
-        query = self.next_query(outcome=outcome)
-        if query is not None:
-            reply = user.ask(query)
-            if reply is None or reply.answer is None:
-                return (
-                    UtilityDistribution(
-                        dtype="uniform", loc=lower, scale=upper - lower
-                    ),
-                    None,
-                )
-            lower_new, upper_new = (
-                reply.answer.constraint.range[0],
-                reply.answer.constraint.range[1],
-            )
-            if abs(upper_new - lower_new) >= abs(upper - lower):
-                upper_new = lower_new = (upper_new + lower_new) / 2
-            self.lower[index], self.upper[index] = lower_new, upper_new
-            lower, upper = lower_new, upper_new
-        if self.strategy == "exact":
-            u = user.ufun(outcome)
-        elif abs(upper - lower) < epsilon or query is None:
-            u = (upper + lower) / 2
-        else:
-            u = UtilityDistribution(dtype="uniform", loc=lower, scale=upper - lower)
-        return u, reply
-
-    def next_query(self, outcome: "Outcome") -> Optional[Query]:
+    def next_query(self, outcome: Outcome) -> Query | None:
         lower, upper, outcomes = self.lower, self.upper, self.outcomes
-        index = self.indices[outcome_as_tuple(outcome)]
+        index = self.indices[outcome]
         lower, upper = lower[index], upper[index]
 
         if abs(upper - lower) < self.resolution:
@@ -256,20 +205,69 @@ class EStrategy:
 
         return query
 
-    def utility_estimate(self, outcome: "Outcome") -> UtilityValue:
+    def apply(
+        self, user: User, outcome: Outcome
+    ) -> tuple[Value | None, QResponse | None]:
+        """Do the elicitation and incur the cost.
+
+        Remarks:
+
+            - This function returns a uniform distribution whenever it returns a distribution
+            - Can return `None` which indicates that elicitation failed
+            - If it could find an exact value, it will return a `float` not a `Distribution`
+
+        """
+
+        lower, upper, _ = self.lower, self.upper, self.outcomes
+        index = self.indices[outcome]
+        lower, upper = lower[index], upper[index]
+        epsilon = self.resolution
+
+        if abs(upper - lower) < epsilon:
+            return (upper + lower) / 2, None
+
+        if self.stop_at_cost and abs(upper - lower) < 2 * user.cost:
+            return (upper + lower) / 2, None
+
+        reply = None
+        query = self.next_query(outcome=outcome)
+        if query is not None:
+            reply = user.ask(query)
+            if reply is None or reply.answer is None:
+                return (
+                    ScipyDistribution(type="uniform", loc=lower, scale=upper - lower),
+                    None,
+                )
+            lower_new, upper_new = (
+                reply.answer.constraint.range[0],
+                reply.answer.constraint.range[1],
+            )
+            if abs(upper_new - lower_new) >= abs(upper - lower):
+                upper_new = lower_new = (upper_new + lower_new) / 2
+            self.lower[index], self.upper[index] = lower_new, upper_new
+            lower, upper = lower_new, upper_new
+        if self.strategy == "exact":
+            u = user.ufun(outcome)
+        elif abs(upper - lower) < epsilon or query is None:
+            u = (upper + lower) / 2
+        else:
+            u = ScipyDistribution(type="uniform", loc=lower, scale=upper - lower)
+        return u, reply
+
+    def utility_estimate(self, outcome: Outcome) -> Value:
         """Gets a probability distribution of the Negotiator for this outcome without elicitation. Costs nothing"""
-        indx = self.indices[outcome_as_tuple(outcome)]
+        indx = self.indices[outcome]
         scale = self.upper[indx] - self.lower[indx]
         if scale < self.resolution:
             return self.lower[indx]
-        return UtilityDistribution(dtype="uniform", loc=self.lower[indx], scale=scale)
+        return ScipyDistribution(type="uniform", loc=self.lower[indx], scale=scale)
 
     def until(
         self,
-        outcome: "Outcome",
-        user: "User",
-        dist: Union[List[UtilityValue], UtilityValue],
-    ) -> UtilityValue:
+        outcome: Outcome,
+        user: User,
+        dist: list[Value] | Value,
+    ) -> Value:
         if isinstance(dist, list):
             targets = [
                 (_ - self.resolution, _ + self.resolution)
@@ -301,18 +299,16 @@ class EStrategy:
         return u
 
     def on_enter(
-        self, ami: AgentMechanismInterface, ufun: IPUtilityFunction = None
+        self, nmi: NegotiatorMechanismInterface, preferences: IPUtilityFunction = None
     ) -> None:
-        self.lower = [0.0] * ami.n_outcomes
-        self.upper = [1.0] * ami.n_outcomes
-        self.indices = dict(
-            zip((outcome_as_tuple(_) for _ in ami.outcomes), range(ami.n_outcomes))
-        )
-        if ufun is not None:
-            distributions = list(ufun.distributions.values())
+        self.lower = [0.0] * nmi.n_outcomes
+        self.upper = [1.0] * nmi.n_outcomes
+        self.indices = dict(zip(nmi.outcomes, range(nmi.n_outcomes)))
+        if preferences is not None:
+            distributions = list(preferences.distributions.values())
             for i, dist in enumerate(distributions):
                 self.lower[i] = _loc(dist)
                 self.upper[i] = _upper(dist)
-        self.outcomes = ami.outcomes
+        self.outcomes = nmi.outcomes
         self._total_cost = 0.0
         self._elicited_queries = []
