@@ -33,14 +33,16 @@ Bridge Control Operations
 
 
 """
+from __future__ import annotations
+
 import os
 import pathlib
 import socket
 import subprocess
 import time
 import traceback
-import warnings
-from typing import Any, Dict, Optional
+from pathlib import Path
+from typing import Any
 
 import psutil
 from py4j.java_gateway import (
@@ -51,6 +53,8 @@ from py4j.java_gateway import (
 )
 from py4j.protocol import Py4JNetworkError
 
+import negmas.warnings as warnings
+
 from ..config import CONFIG_KEY_GENIUS_BRIDGE_JAR, NEGMAS_CONFIG
 from ..helpers import TimeoutCaller, TimeoutError, unique_name
 from .common import DEFAULT_JAVA_PORT, get_free_tcp_port
@@ -70,8 +74,58 @@ def _kill_process(proc_pid):
     process.kill()
 
 
+def genius_bridge_is_running(port: int = DEFAULT_JAVA_PORT) -> bool:
+    """
+    Checks whether a Genius Bridge is running. A genius bridge allows you to use `GeniusNegotiator` objects.
+
+    Remarks:
+
+        You can start a Genius Bridge in at least three ways:
+
+        - execute the python function `init_genius_bridge()` in this module
+        - run "negmas genius" on the terminal
+        - execute `GeniusBridge.start()`
+
+    """
+    s = socket.socket()
+    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+    try:
+        s.connect(("127.0.0.1", port))
+        s.close()
+        # we know someone is listening. Now we check that it is us
+        gateway = JavaGateway(
+            gateway_parameters=GatewayParameters(port=port, auto_close=True),
+            callback_server_parameters=CallbackServerParameters(
+                port=0, daemonize=True, daemonize_connections=True
+            ),
+        )
+        gateway.jvm.System.currentTimeMillis()  # type: ignore
+        return True
+    except ConnectionRefusedError:
+        # try:
+        #     s.shutdown(2)
+        # except:
+        #     pass
+        s.close()
+        return False
+    except IndexError:
+        # try:
+        #     s.shutdown(2)
+        # except:
+        #     pass
+        s.close()
+        return False
+    except Py4JNetworkError:
+        # try:
+        #     s.shutdown(2)
+        # except:
+        #     pass
+        s.close()
+        return False
+
+
 def init_genius_bridge(
-    path: str = None,
+    path: Path | str | None = None,
     port: int = DEFAULT_JAVA_PORT,
     debug: bool = False,
     timeout: float = 0,
@@ -106,10 +160,11 @@ def init_genius_bridge(
             'to ~/negmas/config.json under the key "genius_bridge_jar".\n\nFor example, if you downloaded the jar'
             " to /path/to/your/jar then edit ~/negmas/config.json to read something like\n\n"
             '{\n\t"genius_bridge_jar": "/path/to/your/jar",\n\t.... rest of the config\n}\n\n'
-            "You can find the jar at http://www.yasserm.com/scml/genius-8.0.4-bridge.jar"
+            "You can find the jar at http://www.yasserm.com/scml/genius-8.0.4-bridge.jar",
+            warnings.NegmasBridgePathWarning,
         )
         return False
-    path = pathlib.Path(path).expanduser().absolute()
+    path = Path(path).expanduser().absolute()
     if debug:
         params = " --debug"
     else:
@@ -130,9 +185,9 @@ def init_genius_bridge(
             port=0, daemonize=True, daemonize_connections=True
         ),
     )
-    python_port = gateway.get_callback_server().get_listening_port()
-    gateway.java_gateway_server.resetCallbackClient(
-        gateway.java_gateway_server.getCallbackClient().getAddress(), python_port
+    python_port = gateway.get_callback_server().get_listening_port()  # type: ignore
+    gateway.java_gateway_server.resetCallbackClient(  # type: ignore
+        gateway.java_gateway_server.getCallbackClient().getAddress(), python_port  # type: ignore
     )
     return True
 
@@ -141,72 +196,23 @@ def genius_bridge_is_installed() -> bool:
     """
     Checks if geniusbridge is available in the default path location
     """
-    return (pathlib.Path.home() / "negmas" / "files" / "geniusbridge.jar").exists()
-
-
-def genius_bridge_is_running(port: int = DEFAULT_JAVA_PORT) -> bool:
-    """
-    Checks whether a Genius Bridge is running. A genius bridge allows you to use `GeniusNegotiator` objects.
-
-    Remarks:
-
-        You can start a Genius Bridge in at least three ways:
-
-        - execute the python function `init_genius_bridge()` in this module
-        - run "negmas genius" on the terminal
-        - execute `GeniusBridge.start()`
-
-    """
-    s = socket.socket()
-    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-    try:
-        s.connect(("127.0.0.1", port))
-        s.close()
-        # we know someone is listening. Now we check that it is us
-        gateway = JavaGateway(
-            gateway_parameters=GatewayParameters(port=port, auto_close=True),
-            callback_server_parameters=CallbackServerParameters(
-                port=0, daemonize=True, daemonize_connections=True
-            ),
-        )
-        gateway.jvm.System.currentTimeMillis()
-        return True
-    except ConnectionRefusedError:
-        # try:
-        #     s.shutdown(2)
-        # except:
-        #     pass
-        s.close()
-        return False
-    except IndexError:
-        # try:
-        #     s.shutdown(2)
-        # except:
-        #     pass
-        s.close()
-        return False
-    except Py4JNetworkError:
-        # try:
-        #     s.shutdown(2)
-        # except:
-        #     pass
-        s.close()
-        return False
+    return (Path.home() / "negmas" / "files" / "geniusbridge.jar").exists()
 
 
 class GeniusBridge:
-    gateways: Dict[int, "GeniusBridge"] = dict()
-    """Gateways to different genius bridges"""
-    java_processes: Dict[int, Any] = dict()
-    """handles to the java processes running the bridges"""
-    python_ports: Dict[int, int] = dict()
-    """The port used by python's Gateway to connect to the bridge"""
+    gateways: dict[int, GeniusBridge] = dict()
+    java_processes: dict[int, Any] = dict()
+    python_ports: dict[int, int] = dict()
 
     def __init__(
         self,
     ):
         raise RuntimeError(f"Cannot create objects of type GeniusBridge.")
 
+    """Gateways to different genius bridges"""
+    """handles to the java processes running the bridges"""
+    """The port used by python's Gateway to connect to the bridge"""
+
     @classmethod
     def is_running(cls, port: int) -> bool:
         """Returns true if a geniusbridge.jar is running on the given port"""
@@ -217,16 +223,80 @@ class GeniusBridge:
         """Returns true if a geniusbridge.jar is available"""
         return genius_bridge_is_installed()
 
+    @classmethod
+    def gateway(cls, port=DEFAULT_JAVA_PORT, force=False):
+        """
+        Finds and returns a gateway for a genius bridge on the given port
+
+        Args:
+            port: The port used by the Jave genius bridge.
+            force: If true, a new gateway is created even if one exists in
+                   the list of gateways available in `GeniusBridge`.gateways.
+        Returns:
+            The gateway if found otherwise an exception will be thrown
+
+        Remarks:
+            - this method does NOT start a bridge. It only connects to a
+              running bridge.
+        """
+        assert port > 0
+        gateway = cls.gateways.get(port, None) if not force else None
+        if gateway is not None:
+            return gateway
+        try:
+            gateway = JavaGateway(
+                gateway_parameters=GatewayParameters(port=port, auto_close=True),
+                callback_server_parameters=CallbackServerParameters(
+                    port=0, daemonize=True, daemonize_connections=True
+                ),
+            )
+            python_port = gateway.get_callback_server().get_listening_port()  # type: ignore
+            gateway.java_gateway_server.resetCallbackClient(  # type: ignore
+                gateway.java_gateway_server.getCallbackClient().getAddress(),  # type: ignore
+                python_port,
+            )
+        except:
+            if gateway is not None:
+                gateway.shutdown()
+                gateway.shutdown_callback_server()
+            return None
+        cls.python_ports[port] = python_port
+        cls.gateways[port] = gateway  # type: ignore
+        return gateway
+
+    @classmethod
+    def wait_until_listening(
+        cls,
+        port: int = DEFAULT_JAVA_PORT,
+        timeout: float = 0.5,
+    ) -> bool:
+        """
+        waits until a genius bridge is  listening to the given port
+
+        Args:
+            port: The port to test
+            timeout: Maximum time to wait before returning (in seconds)
+
+        Returns:
+            True if the genius bridge is running any more (success).
+        """
+        if genius_bridge_is_running(port):
+            return True
+        time.sleep(timeout)
+        return TimeoutCaller.run(
+            lambda: genius_bridge_is_running(port), timeout=timeout
+        )
+
     @classmethod
     def start(
         cls,
         port: int = DEFAULT_JAVA_PORT,
-        path: str = None,
+        path: str | None = None,
         debug: bool = False,
         timeout: float = 0,
         force_timeout: bool = True,
         save_logs: bool = False,
-        log_path: Optional[os.PathLike] = None,
+        log_path: os.PathLike | None = None,
         die_on_exit: bool = False,
         use_shell: bool = False,
     ) -> int:
@@ -270,13 +340,14 @@ class GeniusBridge:
                 'to ~/negmas/config.json under the key "genius_bridge_jar".\n\nFor example, if you downloaded the jar'
                 " to /path/to/your/jar then edit ~/negmas/config.json to read something like\n\n"
                 '{\n\t"genius_bridge_jar": "/path/to/your/jar",\n\t.... rest of the config\n}\n\n'
-                "You can find the jar at http://www.yasserm.com/scml/geniusbridge.jar"
+                "You can find the jar at http://www.yasserm.com/scml/geniusbridge.jar",
+                warnings.NegmasBridgePathWarning,
             )
             return 0
-        path = pathlib.Path(path).expanduser().absolute()
+        path = Path(path).expanduser().absolute()  # type: ignore
         if log_path is None or not log_path:
             log_path = (
-                pathlib.Path.home()
+                Path.home()
                 / "negmas"
                 / "geniusbridge"
                 / "logs"
@@ -284,7 +355,7 @@ class GeniusBridge:
             ).absolute()
             log_path.parent.mkdir(parents=True, exist_ok=True)
         else:
-            log_path = pathlib.Path(log_path).absolute()
+            log_path = Path(log_path).absolute()
             log_path.parent.mkdir(parents=True, exist_ok=True)
         # if die_on_exit:
         #     if debug:
@@ -328,55 +399,14 @@ class GeniusBridge:
             cls.java_processes[port] = subprocess.Popen(
                 ["java", "-jar", str(path)] + params + [f"{port}"],
                 shell=use_shell,
-                cwd=path.parent,
+                cwd=path.parent,  # type: ignore
             )
         except (OSError, TimeoutError, RuntimeError, ValueError) as e:
-            warnings.warn(str(e))
+            warnings.warn(str(e), warnings.NegmasBridgeProcessWarning)
             return 0
         cls.wait_until_listening(port, timeout=0.1)
         return port if cls.gateway(port, force=True) is not None else 0
 
-    @classmethod
-    def gateway(cls, port=DEFAULT_JAVA_PORT, force=False):
-        """
-        Finds and returns a gateway for a genius bridge on the given port
-
-        Args:
-            port: The port used by the Jave genius bridge.
-            force: If true, a new gateway is created even if one exists in
-                   the list of gateways available in `GeniusBridge`.gateways.
-        Returns:
-            The gateway if found otherwise an exception will be thrown
-
-        Remarks:
-            - this method does NOT start a bridge. It only connects to a
-              running bridge.
-        """
-        assert port > 0
-        gateway = cls.gateways.get(port, None) if not force else None
-        if gateway is not None:
-            return gateway
-        try:
-            gateway = JavaGateway(
-                gateway_parameters=GatewayParameters(port=port, auto_close=True),
-                callback_server_parameters=CallbackServerParameters(
-                    port=0, daemonize=True, daemonize_connections=True
-                ),
-            )
-            python_port = gateway.get_callback_server().get_listening_port()
-            gateway.java_gateway_server.resetCallbackClient(
-                gateway.java_gateway_server.getCallbackClient().getAddress(),
-                python_port,
-            )
-        except:
-            if gateway is not None:
-                gateway.shutdown()
-                gateway.shutdown_callback_server()
-            return None
-        cls.python_ports[port] = python_port
-        cls.gateways[port] = gateway
-        return gateway
-
     @classmethod
     def _close_gateway(cls, port):
         """
@@ -387,8 +417,8 @@ class GeniusBridge:
             cls.java_processes.pop(port, None)
             cls.python_ports.pop(port, None)
 
-        gateway.shutdown()
-        gateway.shutdown_callback_server()
+        gateway.shutdown()  # type: ignore
+        gateway.shutdown_callback_server()  # type: ignore
         cls.gateways.pop(port, None)
         cls.java_processes.pop(port, None)
         cls.python_ports.pop(port, None)
@@ -411,9 +441,31 @@ class GeniusBridge:
         """
         Closes all open gateways.
         """
-        for p in cls.gateways.keys():
+        keys = list(cls.gateways.keys())
+        for p in keys:
             cls._close_gateway(p)
 
+    @classmethod
+    def wait_until_not_listening(
+        cls,
+        port: int = DEFAULT_JAVA_PORT,
+        timeout: float = 0.5,
+    ) -> bool:
+        """
+        waits until the genius bridge is not listening to the given port
+
+        Args:
+            port: The port to test
+            max_sleep: Maximum time to wait before returning (in seconds)
+
+        Returns:
+            True if the genius bridge is NOT running any more (success).
+        """
+        if not genius_bridge_is_running(port):
+            return False
+        time.sleep(timeout)
+        return not genius_bridge_is_running(port)
+
     @classmethod
     def shutdown(
         cls,
@@ -447,6 +499,55 @@ class GeniusBridge:
         cls._close_gateway(port)
         return True
 
+    @classmethod
+    def kill(
+        cls,
+        port: int = DEFAULT_JAVA_PORT,
+        wait: bool = True,
+    ) -> bool:
+        """Kills the java bridge connected to this port by asking it to exit"""
+        assert port > 0
+        try:
+            gateway = cls.gateway(port, force=True)
+        except:
+            return False
+        if gateway is None:
+            return False
+        try:
+            gateway.entry_point.kill()
+        except:
+            pass
+        if wait:
+            cls.wait_until_not_listening(port)
+        cls._close_gateway(port)
+        return True
+
+    @classmethod
+    def kill_forced(
+        cls,
+        port: int = DEFAULT_JAVA_PORT,
+        wait: bool = True,
+    ) -> bool:
+        """Kills the java bridge connected to this port forcibly.
+
+        Remarks:
+            - The java bridge process must have been started by this process.
+        """
+        assert port > 0
+        p = cls.java_processes.pop(port, None)
+        if p is None:
+            warnings.warn(
+                f"Attempting to force-kill a genius brdige we did not start "
+                "at port {port}",
+                warnings.NegmasBridgeProcessWarning,
+            )
+            return False
+        _kill_process(p.pid)
+        if wait:
+            cls.wait_until_not_listening(port)
+        cls._close_gateway(port)
+        return True
+
     @classmethod
     def stop(cls, port: int = DEFAULT_JAVA_PORT) -> bool:
         """Stops a running bridge
@@ -502,50 +603,6 @@ class GeniusBridge:
         cls.wait_until_listening(port, 1)
         return genius_bridge_is_running(port)
 
-    @classmethod
-    def wait_until_listening(
-        cls,
-        port: int = DEFAULT_JAVA_PORT,
-        timeout: float = 0.5,
-    ) -> bool:
-        """
-        waits until a genius bridge is  listening to the given port
-
-        Args:
-            port: The port to test
-            timeout: Maximum time to wait before returning (in seconds)
-
-        Returns:
-            True if the genius bridge is running any more (success).
-        """
-        if genius_bridge_is_running(port):
-            return True
-        time.sleep(timeout)
-        return TimeoutCaller.run(
-            lambda: genius_bridge_is_running(port), timeout=timeout
-        )
-
-    @classmethod
-    def wait_until_not_listening(
-        cls,
-        port: int = DEFAULT_JAVA_PORT,
-        timeout: float = 0.5,
-    ) -> bool:
-        """
-        waits until the genius bridge is not listening to the given port
-
-        Args:
-            port: The port to test
-            max_sleep: Maximum time to wait before returning (in seconds)
-
-        Returns:
-            True if the genius bridge is NOT running any more (success).
-        """
-        if not genius_bridge_is_running(port):
-            return False
-        time.sleep(timeout)
-        return not genius_bridge_is_running(port)
-
     @classmethod
     def kill_threads(
         cls,
@@ -559,7 +616,8 @@ class GeniusBridge:
         except Exception as e:
             warnings.warn(
                 f"Failed to kill threads at port {port} with error: {str(e)}\n"
-                f"{traceback.format_exc()}"
+                f"{traceback.format_exc()}",
+                warnings.NegmasBridgeProcessWarning,
             )
             return False
         if gateway is None:
@@ -570,54 +628,6 @@ class GeniusBridge:
             pass
         return True
 
-    @classmethod
-    def kill(
-        cls,
-        port: int = DEFAULT_JAVA_PORT,
-        wait: bool = True,
-    ) -> bool:
-        """Kills the java bridge connected to this port by asking it to exit"""
-        assert port > 0
-        try:
-            gateway = cls.gateway(port, force=True)
-        except:
-            return False
-        if gateway is None:
-            return False
-        try:
-            gateway.entry_point.kill()
-        except:
-            pass
-        if wait:
-            cls.wait_until_not_listening(port)
-        cls._close_gateway(port)
-        return True
-
-    @classmethod
-    def kill_forced(
-        cls,
-        port: int = DEFAULT_JAVA_PORT,
-        wait: bool = True,
-    ) -> bool:
-        """Kills the java bridge connected to this port forcibly.
-
-        Remarks:
-            - The java bridge process must have been started by this process.
-        """
-        assert port > 0
-        p = cls.java_processes.pop(port, None)
-        if p is None:
-            warnings.warn(
-                f"Attempting to force-kill a genius brdige we did not start "
-                "at port {port}"
-            )
-            return False
-        _kill_process(p.pid)
-        if wait:
-            cls.wait_until_not_listening(port)
-        cls._close_gateway(port)
-        return True
-
     @classmethod
     def clean(cls, port=DEFAULT_JAVA_PORT) -> bool:
         """
@@ -629,7 +639,8 @@ class GeniusBridge:
         except Exception as e:
             warnings.warn(
                 f"Failed to kill threads at port {port} with error: {str(e)}\n"
-                f"{traceback.format_exc()}"
+                f"{traceback.format_exc()}",
+                warnings.NegmasBridgeProcessWarning,
             )
             return False
         if gateway is None:
@@ -666,9 +677,10 @@ class GeniusBridge:
         except Exception as e:
             warnings.warn(
                 f"Failed to kill threads at port {port} with error: {str(e)}\n"
-                f"{traceback.format_exc()}"
+                f"{traceback.format_exc()}",
+                warnings.NegmasBridgeProcessWarning,
             )
             raise RuntimeError(e)
         if gateway is None:
-            return None
+            raise RuntimeError(f"Got None as the gateway!!")
         return gateway.entry_point
