@@ -1,8 +1,9 @@
 """
 Keeps information about ANAC competitions and Genius Agents
 """
+from __future__ import annotations
+
 import itertools
-from typing import List, Optional, Set, Tuple
 
 __all__ = [
     "GENIUS_INFO",
@@ -262,7 +263,7 @@ GENIUS_INFO = {
         "multilateral": True,
         "bilateral": False,
         "reservation": None,
-        "discounting": None,
+        "discounting": True,
         "uncertainty": False,
         "elicitation": False,
     },
@@ -291,163 +292,6 @@ GENIUS_INFO = {
 }
 
 
-AGENT_BASED_NEGOTIATORS = [
-    "agents.ABMPAgent",
-    "agents.ABMPAgent2",
-    "agents.BayesianAgent",
-    "agents.BayesianAgentForAuction",
-    "agents.DecUtilAgent",
-    "agents.FuzzyAgent",
-    "agents.OptimalBidder",
-    "agents.OptimalBidderU",
-    "agents.QOAgent",
-    "agents.RandomIncreasingUtilAgent",
-    "agents.SimilarityAgent",
-    "agents.anac.y2010.AgentFSEGA.AgentFSEGA",
-    "agents.anac.y2010.AgentK.Agent_K",
-    "agents.anac.y2010.AgentSmith.AgentSmith",
-    "agents.anac.y2010.Nozomi.Nozomi",
-    "agents.anac.y2010.Southampton.SouthamptonAgent",
-    "agents.anac.y2010.Yushu.Yushu",
-    "agents.anac.y2011.AgentK2.Agent_K2",
-    "agents.anac.y2011.BramAgent.BRAMAgent",
-    "agents.anac.y2011.Gahboninho.Gahboninho",
-    "agents.anac.y2011.HardHeaded.KLH",
-    "agents.anac.y2011.IAMhaggler2011.IAMhaggler2011",
-    "agents.anac.y2011.Nice_Tit_for_Tat.NiceTitForTat",
-    "agents.anac.y2011.TheNegotiator.TheNegotiator",
-    "agents.anac.y2011.ValueModelAgent.ValueModelAgent",
-    "agents.anac.y2012.AgentLG.AgentLG",
-    "agents.anac.y2012.TheNegotiatorReloaded.TheNegotiatorReloaded",
-    "agents.anac.y2012.AgentMR.AgentMR",
-    "agents.anac.y2012.BRAMAgent2.BRAMAgent2",
-    "agents.anac.y2012.CUHKAgent.CUHKAgent",
-    "agents.anac.y2012.MetaAgent.MetaAgent",
-    "agents.anac.y2012.OMACagent.OMACagent",
-    "agents.anac.y2013.AgentKF.AgentKF",
-    "agents.anac.y2013.GAgent.AgentI",
-    "agents.anac.y2013.MetaAgent.MetaAgent2013",
-    "agents.anac.y2013.SlavaAgent.SlavaAgent",
-    "agents.anac.y2013.TMFAgent.TMFAgent",
-    "agents.anac.y2013.TheFawkes.TheFawkes",
-    "agents.anac.y2014.AgentM.AgentM",
-    "agents.anac.y2014.AgentQuest.AgentQuest",
-    "agents.anac.y2014.AgentTD.AgentTD",
-    "agents.anac.y2014.AgentTRP.AgentTRP",
-    "agents.anac.y2014.AgentWhale.WhaleAgent",
-    "agents.anac.y2014.AgentYK.AgentYK",
-    "agents.anac.y2014.ArisawaYaki.ArisawaYaki",
-    "agents.anac.y2014.Aster.Aster",
-    "agents.anac.y2014.Atlas.Atlas",
-    "agents.anac.y2014.BraveCat.BraveCat",
-    "agents.anac.y2014.DoNA.DoNA",
-    "agents.anac.y2014.E2Agent.AnacSampleAgent",
-    "agents.anac.y2014.Flinch.Flinch",
-    "agents.anac.y2014.Gangster.Gangster",
-    "agents.anac.y2014.KGAgent.KGAgent",
-    "agents.anac.y2014.SimpaticoAgent.Simpatico",
-    "agents.anac.y2014.Sobut.Sobut",
-    "agents.anac.y2017.geneking.GeneKing",
-]
-"""Genius agents based on the Agent base-class. These are the oldest agents"""
-
-PARTY_BASED_NEGOTIATORS = [
-    "agents.ai2014.group1.Group1",
-    "agents.ai2014.group10.Group10",
-    "agents.ai2014.group11.Group11",
-    "agents.ai2014.group12.Group12",
-    "agents.ai2014.group2.Group2",
-    "agents.ai2014.group3.Group3",
-    "agents.ai2014.group4.Group4",
-    "agents.ai2014.group5.Group5",
-    "agents.ai2014.group6.Group6",
-    "agents.ai2014.group7.Group7",
-    "agents.ai2014.group9.Group9",
-    "agents.anac.y2015.AgentNeo.Groupn",
-    "agents.anac.y2015.AgentX.AgentX",
-    "agents.anac.y2015.AresParty.AresParty",
-    "agents.anac.y2015.Atlas3.Atlas3",
-    "agents.anac.y2015.DrageKnight.DrageKnight",
-    "agents.anac.y2015.JonnyBlack.JonnyBlack",
-    "agents.anac.y2015.Mercury.Mercury",
-    "agents.anac.y2015.ParsAgent.ParsAgent",
-    "agents.anac.y2015.Phoenix.PhoenixParty",
-    "agents.anac.y2015.RandomDance.RandomDance",
-    "agents.anac.y2015.SENGOKU.SENGOKU",
-    "agents.anac.y2015.TUDMixedStrategyAgent.TUDMixedStrategyAgent",
-    "agents.anac.y2015.agentBuyogV2.AgentBuyogMain",
-    "agents.anac.y2015.agenth.AgentH",
-    "agents.anac.y2015.cuhkagent2015.CUHKAgent2015",
-    "agents.anac.y2015.fairy.kawaii",
-    "agents.anac.y2015.group2.Group2",
-    "agents.anac.y2015.meanBot.MeanBot",
-    "agents.anac.y2015.pokerface.PokerFace",
-    "agents.anac.y2016.agentsmith.AgentSmith2016",
-    "agents.anac.y2016.atlas3.Atlas32016",
-    "agents.anac.y2016.caduceus.Caduceus",
-    "agents.anac.y2016.clockworkagent.ClockworkAgent",
-    "agents.anac.y2016.farma.Farma",
-    "agents.anac.y2016.grandma.GrandmaAgent",
-    "agents.anac.y2016.myagent.MyAgent",
-    "agents.anac.y2016.ngent.Ngent",
-    "agents.anac.y2016.pars2.ParsAgent2",
-    "agents.anac.y2016.parscat.ParsCat",
-    "agents.anac.y2016.syagent.SYAgent",
-    "agents.anac.y2016.terra.Terra",
-    "agents.anac.y2016.yxagent.YXAgent",
-    "agents.anac.y2017.agentf.AgentF",
-    "agents.anac.y2017.caduceusdc16.CaduceusDC16",
-    "agents.anac.y2017.farma.Farma17",
-    "agents.anac.y2017.ponpokoagent.PonPokoAgent",
-    "agents.anac.y2017.tangxun.taxibox",
-    "agents.anac.y2017.tucagent.TucAgent",
-]
-"""Genius agents based on the Party base-class. These are the newest agents"""
-
-TEST_FAILING_NEGOTIATORS = [
-    "agents.anac.y2017.rubick.Rubick",  # failed all tests
-    "agents.anac.y2017.caduceusdc16.CaduceusDC16",  # failed all tests
-    "agents.OptimalBidderSimple",  # failes most tests but not all
-    "agents.FuzzyAgent",  # fails most test but not all
-    "agents.SimilarityAgent",  # failed some but not all tests
-    "agents.ABMPAgent2",  # failed some but not all tests
-    "agents.anac.y2013.MetaAgent.MetaAgent2013",  # failed a few tests
-    "agents.BayesianAgentForAuction",
-    "agents.DecUtilAgent",
-    "agents.OptimalBidder",
-    "agents.OptimalBidderU",
-    "agents.QOAgent",
-    "agents.RandomIncreasingUtilAgent",
-    "agents.anac.y2014.AgentTD.AgentTD",
-    "agents.anac.y2017.geneking.GeneKing",
-    "agents.anac.y2018.beta_one.Group2",
-    "agents.anac.y2018.agreeableagent2018.AgreeableAgent2018",
-    "agents.anac.y2016.agenthp2.AgentHP2_main",
-    "agents.anac.y2014.KGAgent.KGAgent",
-    "agents.anac.y2018.meng_wan.Agent36",
-    "agents.anac.y2014.E2Agent.AnacSampleAgent",
-    "agents.anac.y2010.Southampton.SouthamptonAgent",
-]
-"""Agetns taht fail simple tests making them less robust over the bridge"""
-
-TESTED_NEGOTIATORS = list(
-    set(
-        ["agents.anac.y2015.AgentX.AgentX"]
-        + list(
-            itertools.chain(
-                *(
-                    list(_[1] for _ in itertools.chain(*v["winners"]))
-                    for year, v in GENIUS_INFO.items()
-                    if v["multilateral"] and not v["learning"]
-                )
-            )
-        )
-    )
-    - set(TEST_FAILING_NEGOTIATORS)
-)
-"""Some of the most tested negotaitors"""
-
-
 ALL_GENIUS_INVALID_NEGOTIATORS = [
     "agents.UIAgent",
     "agents.UIAgentExtended",
@@ -669,6 +513,190 @@ ALL_GENIUS_FEEDBACK_MEDIATOR_NEGOTIATORS = [
 ]
 """All Genius negotiators for the Feedback Mediator Protocol"""
 
+AGENT_BASED_NEGOTIATORS = [
+    "agents.ABMPAgent",
+    "agents.ABMPAgent2",
+    "agents.BayesianAgent",
+    "agents.BayesianAgentForAuction",
+    "agents.DecUtilAgent",
+    "agents.FuzzyAgent",
+    "agents.OptimalBidder",
+    "agents.OptimalBidderU",
+    "agents.QOAgent",
+    "agents.RandomIncreasingUtilAgent",
+    "agents.SimilarityAgent",
+    "agents.anac.y2010.AgentFSEGA.AgentFSEGA",
+    "agents.anac.y2010.AgentK.Agent_K",
+    "agents.anac.y2010.AgentSmith.AgentSmith",
+    "agents.anac.y2010.Nozomi.Nozomi",
+    "agents.anac.y2010.Southampton.SouthamptonAgent",
+    "agents.anac.y2010.Yushu.Yushu",
+    "agents.anac.y2011.AgentK2.Agent_K2",
+    "agents.anac.y2011.BramAgent.BRAMAgent",
+    "agents.anac.y2011.Gahboninho.Gahboninho",
+    "agents.anac.y2011.HardHeaded.KLH",
+    "agents.anac.y2011.IAMhaggler2011.IAMhaggler2011",
+    "agents.anac.y2011.Nice_Tit_for_Tat.NiceTitForTat",
+    "agents.anac.y2011.TheNegotiator.TheNegotiator",
+    "agents.anac.y2011.ValueModelAgent.ValueModelAgent",
+    "agents.anac.y2012.AgentLG.AgentLG",
+    "agents.anac.y2012.TheNegotiatorReloaded.TheNegotiatorReloaded",
+    "agents.anac.y2012.AgentMR.AgentMR",
+    "agents.anac.y2012.BRAMAgent2.BRAMAgent2",
+    "agents.anac.y2012.CUHKAgent.CUHKAgent",
+    "agents.anac.y2012.MetaAgent.MetaAgent",
+    "agents.anac.y2012.OMACagent.OMACagent",
+    "agents.anac.y2013.AgentKF.AgentKF",
+    "agents.anac.y2013.GAgent.AgentI",
+    "agents.anac.y2013.MetaAgent.MetaAgent2013",
+    "agents.anac.y2013.SlavaAgent.SlavaAgent",
+    "agents.anac.y2013.TMFAgent.TMFAgent",
+    "agents.anac.y2013.TheFawkes.TheFawkes",
+    "agents.anac.y2014.AgentM.AgentM",
+    "agents.anac.y2014.AgentQuest.AgentQuest",
+    "agents.anac.y2014.AgentTD.AgentTD",
+    "agents.anac.y2014.AgentTRP.AgentTRP",
+    "agents.anac.y2014.AgentWhale.WhaleAgent",
+    "agents.anac.y2014.AgentYK.AgentYK",
+    "agents.anac.y2014.ArisawaYaki.ArisawaYaki",
+    "agents.anac.y2014.Aster.Aster",
+    "agents.anac.y2014.Atlas.Atlas",
+    "agents.anac.y2014.BraveCat.BraveCat",
+    "agents.anac.y2014.DoNA.DoNA",
+    "agents.anac.y2014.E2Agent.AnacSampleAgent",
+    "agents.anac.y2014.Flinch.Flinch",
+    "agents.anac.y2014.Gangster.Gangster",
+    "agents.anac.y2014.KGAgent.KGAgent",
+    "agents.anac.y2014.SimpaticoAgent.Simpatico",
+    "agents.anac.y2014.Sobut.Sobut",
+    "agents.anac.y2017.geneking.GeneKing",
+]
+"""Genius agents based on the Agent base-class. These are the oldest agents"""
+
+PARTY_BASED_NEGOTIATORS = [
+    "agents.ai2014.group1.Group1",
+    "agents.ai2014.group10.Group10",
+    "agents.ai2014.group11.Group11",
+    "agents.ai2014.group12.Group12",
+    "agents.ai2014.group2.Group2",
+    "agents.ai2014.group3.Group3",
+    "agents.ai2014.group4.Group4",
+    "agents.ai2014.group5.Group5",
+    "agents.ai2014.group6.Group6",
+    "agents.ai2014.group7.Group7",
+    "agents.ai2014.group9.Group9",
+    "agents.anac.y2015.AgentNeo.Groupn",
+    "agents.anac.y2015.AgentX.AgentX",
+    "agents.anac.y2015.AresParty.AresParty",
+    "agents.anac.y2015.Atlas3.Atlas3",
+    "agents.anac.y2015.DrageKnight.DrageKnight",
+    "agents.anac.y2015.JonnyBlack.JonnyBlack",
+    "agents.anac.y2015.Mercury.Mercury",
+    "agents.anac.y2015.ParsAgent.ParsAgent",
+    "agents.anac.y2015.Phoenix.PhoenixParty",
+    "agents.anac.y2015.RandomDance.RandomDance",
+    "agents.anac.y2015.SENGOKU.SENGOKU",
+    "agents.anac.y2015.TUDMixedStrategyAgent.TUDMixedStrategyAgent",
+    "agents.anac.y2015.agentBuyogV2.AgentBuyogMain",
+    "agents.anac.y2015.agenth.AgentH",
+    "agents.anac.y2015.cuhkagent2015.CUHKAgent2015",
+    "agents.anac.y2015.fairy.kawaii",
+    "agents.anac.y2015.group2.Group2",
+    "agents.anac.y2015.meanBot.MeanBot",
+    "agents.anac.y2015.pokerface.PokerFace",
+    "agents.anac.y2016.agentsmith.AgentSmith2016",
+    "agents.anac.y2016.atlas3.Atlas32016",
+    "agents.anac.y2016.caduceus.Caduceus",
+    "agents.anac.y2016.clockworkagent.ClockworkAgent",
+    "agents.anac.y2016.farma.Farma",
+    "agents.anac.y2016.grandma.GrandmaAgent",
+    "agents.anac.y2016.myagent.MyAgent",
+    "agents.anac.y2016.ngent.Ngent",
+    "agents.anac.y2016.pars2.ParsAgent2",
+    "agents.anac.y2016.parscat.ParsCat",
+    "agents.anac.y2016.syagent.SYAgent",
+    "agents.anac.y2016.terra.Terra",
+    "agents.anac.y2016.yxagent.YXAgent",
+    "agents.anac.y2017.agentf.AgentF",
+    "agents.anac.y2017.caduceusdc16.CaduceusDC16",
+    "agents.anac.y2017.farma.Farma17",
+    "agents.anac.y2017.ponpokoagent.PonPokoAgent",
+    "agents.anac.y2017.tangxun.taxibox",
+    "agents.anac.y2017.tucagent.TucAgent",
+]
+"""Genius agents based on the Party base-class. These are the newest agents"""
+
+TEST_FAILING_NEGOTIATORS = [
+    "agents.ABMPAgent2",  # failed some but not all tests
+    "agents.BayesianAgentForAuction",
+    "agents.DecUtilAgent",
+    "agents.FuzzyAgent",  # fails most test but not all
+    "agents.OptimalBidder",
+    "agents.OptimalBidderSimple",  # failes most tests but not all
+    "agents.OptimalBidderU",
+    "agents.QOAgent",
+    "agents.RandomIncreasingUtilAgent",
+    "agents.SimilarityAgent",  # failed some but not all tests
+    "agents.anac.y2010.Southampton.SouthamptonAgent",
+    "agents.anac.y2013.MetaAgent.MetaAgent2013",  # failed a few tests
+    "agents.anac.y2014.AgentTD.AgentTD",
+    "agents.anac.y2014.ArisawaYaki.ArisawaYaki",
+    "agents.anac.y2014.Aster.Aster",
+    "agents.anac.y2014.Atlas.Atlas",
+    "agents.anac.y2014.E2Agent.AnacSampleAgent",
+    "agents.anac.y2014.KGAgent.KGAgent",
+    "agents.anac.y2014.TUDelftGroup2.Group2Agent",
+    "agents.anac.y2014.simpleagent.SimpleAgent",
+    "agents.anac.y2015.AgentHP.AgentHP",
+    "agents.anac.y2015.AgentW.AgentW",
+    "agents.anac.y2016.agenthp2.AgentHP2_main",
+    "agents.anac.y2016.agentlight.SgentLight",
+    "agents.anac.y2016.maxoops.MaxOops",
+    "agents.anac.y2016.ngent.Ngent",
+    "agents.anac.y2017.agentkn.AgentKN",
+    "agents.anac.y2017.caduceusdc16.CaduceusDC16",  # failed all tests
+    "agents.anac.y2017.geneking.GeneKing",
+    "agents.anac.y2017.mamenchis.Mamenchis",
+    "agents.anac.y2017.parsagent3.ShahAgent",
+    "agents.anac.y2017.rubick.Rubick",  # failed all tests
+    "parties.in4010.q12015.group1.Group1",
+    "parties.in4010.q12015.group10.Group10",
+    "parties.in4010.q12015.group13.Group13",
+    "parties.in4010.q12015.group18.Group18",
+    "parties.in4010.q12015.group21.Group21",
+    "parties.in4010.q12015.group7.Group7",
+    "agents.anac.y2015.cuhkagent2015.CUHKAgent2015",
+    "agents.anac.y2017.simpleagent.SimpleAgent",
+    "agents.ai2014.group4.Group4",
+    "parties.in4010.q12015.group6.Group6",
+    "agents.anac.y2016.agentlight.AgentLight",
+    "parties.in4010.q12015.group3.Group3",
+    "parties.in4010.q12015.group19.Group19",
+    "agents.anac.y2015.Phoenix.PhoenixParty",
+    "agents.anac.y2015.Mercury.Mercury",
+    "agents.anac.y2015.xianfa.XianFaAgent",
+    "agents.anac.y2016.grandma.GrandmaAgent",
+    "agents.anac.y2015.pnegotiator.PNegotiator",
+] + [_ for _ in ALL_GENIUS_NEGOTIATORS if "y2018" in _]
+
+"""Agetns taht fail simple tests making them less robust over the bridge"""
+
+TESTED_NEGOTIATORS = list(
+    set(
+        ["agents.anac.y2015.AgentX.AgentX"]
+        + list(
+            itertools.chain(
+                *(
+                    list(_[1] for _ in itertools.chain(*v["winners"]))
+                    for year, v in GENIUS_INFO.items()
+                    if v["multilateral"] and not v["learning"]
+                )
+            )
+        )
+    )
+    - set(TEST_FAILING_NEGOTIATORS)
+)
+"""Some of the most tested negotaitors"""
 
 ALL_NEGOTIATORS = list(
     set(PARTY_BASED_NEGOTIATORS + AGENT_BASED_NEGOTIATORS + ALL_GENIUS_NEGOTIATORS)
@@ -691,7 +719,7 @@ def get_name(java_class: str) -> str:
     return java_class.split(".")[-1]
 
 
-def get_java_class(name) -> Optional[str]:
+def get_java_class(name) -> str | None:
     """Returns the java class for the agent with this name if known otherwise it returns None"""
     for class_name in ALL_NEGOTIATORS:
         if name in class_name:
@@ -701,18 +729,18 @@ def get_java_class(name) -> Optional[str]:
 
 def get_anac_agents(
     *,
-    year: int = None,
-    linear: bool = None,
-    learning: bool = None,
-    multilateral: bool = None,
-    bilateral: bool = None,
-    reservation: bool = None,
-    discounting: bool = None,
-    uncertainty: bool = None,
-    elicitation: bool = None,
+    year: int | None = None,
+    linear: bool | None = None,
+    learning: bool | None = None,
+    multilateral: bool | None = None,
+    bilateral: bool | None = None,
+    reservation: bool | None = None,
+    discounting: bool | None = None,
+    uncertainty: bool | None = None,
+    elicitation: bool | None = None,
     winners_only: bool = False,
     finalists_only: bool = False,
-) -> List[Tuple[str, str]]:
+) -> list[tuple[str, str]]:
     """
     Get Genius agents matching some given criteria
 
@@ -727,19 +755,23 @@ def get_anac_agents(
           with caution.
     """
 
-    def get_agents(year, d) -> List[Tuple[str, str]]:
-        lst = tuple(
-            d.get("winners", [[]])
-            if winners_only
-            else d.get("finalists", [])
-            if finalists_only
-            else [(_.split(".")[-1], _) for _ in ALL_NEGOTIATORS if str(year) in _]
-        )
-        if winners_only or finalists_only:
-            return set(itertools.chain(*lst))
+    def get_agents(year, d) -> set[tuple[str, str]]:
+
+        if winners_only:
+            lst = d.get("winners", [[]])
+            lst = list(itertools.chain(*lst))
+        elif finalists_only:
+            lst = d.get("finalists", [])
+            lst = list(itertools.chain(*lst))
+        else:
+            lst = [_ for _ in ALL_NEGOTIATORS if str(year) in _]
+            lst = [(_.split(".")[-1], _) for _ in lst]
+        # lst = tuple(lst)
+        # if winners_only or finalists_only:
+        #     return set(itertools.chain(*lst))
         return set(lst)
 
-    agents: Set[Tuple[str, str]] = set()
+    agents: set[tuple[str, str]] = set()
     if year is None:
         for y in GENIUS_INFO.keys():
             agents = agents.union(
