@@ -3,22 +3,25 @@ Genius Negotiator
 An agent used to connect to GENIUS agents (ver 8.0.4) and allow them to join negotiation mechanisms
 
 """
+from __future__ import annotations
+
 import math
 import os
 import pathlib
 import random
 import tempfile
-import warnings
-from typing import List, Optional, Tuple, Union
 
-from ..common import AgentMechanismInterface, MechanismState
+from negmas import warnings
+from negmas.outcomes.base_issue import Issue
+
+from ..common import MechanismState, NegotiatorMechanismInterface
 from ..config import CONFIG_KEY_GENIUS_BRIDGE_JAR, NEGMAS_CONFIG
 from ..inout import get_domain_issues
 from ..negotiators import Controller
-from ..outcomes import Issue, ResponseType
-from ..sao.common import SAOResponse
+from ..outcomes import CartesianOutcomeSpace, Outcome, issues_to_xml_str
+from ..preferences import UtilityFunction, make_discounted_ufun
+from ..sao.common import ResponseType, SAOResponse, SAOState
 from ..sao.negotiators import SAONegotiator
-from ..utilities import UtilityFunction, make_discounted_ufun, normalize
 from .bridge import GeniusBridge
 from .common import (
     DEFAULT_GENIUS_NEGOTIATOR_TIMEOUT,
@@ -36,23 +39,18 @@ FAILED, OK, TIMEOUT = "__FAILED__", "__OK__", "__TIMEOUT__"
 
 
 class GeniusNegotiator(SAONegotiator):
-    """Encapsulates a Genius Negotiator
+    """
+    Encapsulates a Genius Negotiator
 
     Args:
-        assume_normalized: Assume that the utility function is already normalized (or do not need to be normalized)
-        ufun: The ufun of the negotiator [optional]
+        preferences: The ufun of the negotiator [optional]
         name: Negotiator name [optional]
-        rational_proposal: If true, the negotiator will not offer anything less than their reserved-value
         parent: Parent `Controller`
         owner: The agent that owns the negotiator (if any)
         java_class_name: The java class name of the Geinus underlying agent
         domain_file_name: Optional domain file name (containing the negotiation issues or agenda)
         utility_file_name: Optional ufun file name (xml) from which a ufun will be loaded for the agent
-        keep_issue_names: When reading domainand utility files keep issue names
-        keep_value_names: When reading domainand utility files keep value names
         can_propose: The negotiator can propose
-        normalize_utility: Normalize the ufun [0-1] if it is not already normalized and not assumed-normalized.
-        normalize_max_only: Normalize the max to 1.0 but do not normalize the min.
         auto_load_java: Load the genius bridge if needed
         port: The port to load the genius bridge to (or use if it is already loaded)
         genius_bridge_path: The path to the genius bridge
@@ -65,30 +63,22 @@ class GeniusNegotiator(SAONegotiator):
 
     def __init__(
         self,
-        assume_normalized=True,
-        ufun: Optional[UtilityFunction] = None,
-        name: str = None,
-        rational_proposal=False,
-        parent: Controller = None,
-        owner: "Agent" = None,  # type: ignore
-        java_class_name: str = None,
-        domain_file_name: Union[str, pathlib.Path] = None,
-        utility_file_name: Union[str, pathlib.Path] = None,
-        keep_issue_names: bool = True,
-        keep_value_names: bool = True,
+        preferences: UtilityFunction | None = None,
+        name: str | None = None,
+        parent: Controller | None = None,
+        owner: Agent = None,  # type: ignore
+        java_class_name: str | None = None,
+        domain_file_name: str | pathlib.Path | None = None,
+        utility_file_name: str | pathlib.Path | None = None,
         can_propose=True,
-        normalize_utility: bool = False,
-        normalize_max_only: bool = False,
         auto_load_java: bool = True,
         port: int = DEFAULT_JAVA_PORT,
-        genius_bridge_path: str = None,
-        strict: bool = None,
+        genius_bridge_path: str | None = None,
+        strict: bool | None = None,
     ):
         super().__init__(
             name=name,
-            assume_normalized=assume_normalized,
-            ufun=None,
-            rational_proposal=rational_proposal,
+            preferences=None,
             parent=parent,
             owner=owner,
         )
@@ -112,38 +102,38 @@ class GeniusNegotiator(SAONegotiator):
             else GeniusNegotiator.random_negotiator_name()
         )
         self._port = port
-        self._normalize_utility = normalize_utility
-        self._normalize_max_only = normalize_max_only
         self.domain_file_name = str(domain_file_name) if domain_file_name else None
         self.utility_file_name = str(utility_file_name) if utility_file_name else None
         self._my_last_offer = None
-        self.keep_issue_names = keep_issue_names
-        self._utility_function, self.discount = None, None
+        self._preferences, self.discount = None, None
         self.issue_names = self.issues = self.issue_index = None
         self.auto_load_java = auto_load_java
         if domain_file_name is not None:
             # we keep original issues details so that we can create appropriate answers to Java
             self.issues = get_domain_issues(
-                domain_file_name=domain_file_name,  # type: ignore
-                keep_issue_names=True,
-                keep_value_names=True,
+                domain_file_name=domain_file_name,
             )
-            self.issue_names = [_.name for _ in self.issues]  # type: ignore
+            if not self.issues:
+                raise ValueError(f"Cannot read domain file {domain_file_name}")
+            self.issue_names = [_.name for _ in self.issues]
             self.issue_index = dict(zip(self.issue_names, range(len(self.issue_names))))
         self.discount = None
         if utility_file_name is not None:
-            self._utility_function, self.discount = UtilityFunction.from_genius(  # type: ignore
+            self._preferences, self.discount = UtilityFunction.from_genius(
                 utility_file_name,
-                keep_issue_names=keep_issue_names,
-                keep_value_names=keep_value_names,
+                issues=self.issues,
             )
         # if ufun is not None:
-        #     self._utility_function = ufun
-        self.base_utility = self._utility_function
-        self.__ufun_received = ufun
-        self._temp_domain_file = self._temp_ufun_file = False
+        #     self._preferences = ufun
+        self.base_utility = self._preferences
+        self.__preferences_received = preferences
+        self._temp_domain_file = self._temp_preferences_file = False
         self.connected = False
 
+    @property
+    def is_connected(self):
+        return self.connected and self.java is not None
+
     @property
     def port(self):
         # if a port was not specified then we just set any random empty port to be used
@@ -156,14 +146,14 @@ class GeniusNegotiator(SAONegotiator):
         self._port = port
 
     @classmethod
-    def robust_negotiators(cls) -> List[str]:
+    def robust_negotiators(cls) -> list[str]:
         """
         Returns a list of genius agents that were tested and seem to be robustly working with negmas
         """
         return TESTED_NEGOTIATORS
 
     @classmethod
-    def negotiators(cls, agent_based=True, party_based=True) -> List[str]:
+    def negotiators(cls, agent_based=True, party_based=True) -> list[str]:
         """
         Returns a list of all available agents in genius 8.4.0
 
@@ -196,14 +186,12 @@ class GeniusNegotiator(SAONegotiator):
         agent_based=True,
         party_based=True,
         port: int = DEFAULT_JAVA_PORT,
-        domain_file_name: str = None,
-        utility_file_name: str = None,
-        keep_issue_names: bool = True,
-        keep_value_names: bool = True,
+        domain_file_name: str | None = None,
+        utility_file_name: str | None = None,
         auto_load_java: bool = False,
         can_propose=True,
-        name: str = None,
-    ) -> "GeniusNegotiator":
+        name: str | None = None,
+    ) -> GeniusNegotiator:
         """
         Returns an agent with a random class name
 
@@ -211,8 +199,6 @@ class GeniusNegotiator(SAONegotiator):
             name: negotiator name
             can_propose: Can this negotiator propose?
             auto_load_java: load the JVM if needed
-            keep_value_names: Keep value names if values are strings
-            keep_issue_names: Use dictionaries instead of tuples for representing outcomes
             utility_file_name: Name of the utility xml file
             domain_file_name: Name of the domain XML file
             port: port number to use if the JVM is to be started
@@ -231,20 +217,16 @@ class GeniusNegotiator(SAONegotiator):
             port=port,
             domain_file_name=domain_file_name,
             utility_file_name=utility_file_name,
-            keep_issue_names=keep_issue_names,
-            keep_value_names=keep_value_names,
             auto_load_java=auto_load_java,
             can_propose=can_propose,
             name=name,
         )
 
-    @property
-    def is_connected(self):
-        return self.connected and self.java is not None
-
     def _create(self):
         """Creates the corresponding java agent"""
-        aid = self.java.create_agent(self.java_class_name)
+        if self.java is None:
+            raise ValueError(f"Cannot create Genius Agent (no java instance)")
+        aid = self.java.create_agent(self.java_class_name)  # type: ignore
         if aid == FAILED:
             raise ValueError(f"Cannot initialized {self.java_class_name}")
         return aid
@@ -274,24 +256,29 @@ class GeniusNegotiator(SAONegotiator):
     def java_name(self):
         if not self.java:
             return None
-        return self.java.get_name(self.java_uuid)
+        return self.java.get_name(self.java_uuid)  # type: ignore
 
     def join(
         self,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        preferences: UtilityFunction | None = None,
+        ufun: UtilityFunction | None = None,
+        role: str = "negotiator",
     ) -> bool:
-        if ufun is None:
-            ufun = self.__ufun_received
-        result = super().join(ami=ami, state=state, ufun=ufun, role=role)
-        if not result:
+        if ufun:
+            preferences = ufun
+        if not preferences:
+            preferences = self.__preferences_received
+        result = super().join(
+            nmi=nmi, state=state, preferences=preferences, ufun=None, role=role
+        )
+        if not result or self.ufun is None:
             return False
         # only connect to the JVM running genius-bridge if you are going to join a negotiation.
         if not self.is_connected:
-            mechanism_port = ami.params.get("genius_port", 0)
+            mechanism_port: int = nmi.params.get("genius_port", 0)
             if mechanism_port > 0:
                 self.port = mechanism_port
             self.connected = self._connect(
@@ -304,41 +291,54 @@ class GeniusNegotiator(SAONegotiator):
 
         self.java_uuid = self._create()
 
-        if self._normalize_utility:
-            self._utility_function = normalize(
-                self.ufun,  # type: ignore
-                outcomes=ami.discrete_outcomes,  # type: ignore
-                max_only=self._normalize_max_only,
+        if not isinstance(nmi.cartesian_outcome_space, CartesianOutcomeSpace):
+            raise ValueError(
+                f"Genius negotiators cannot be used with an out come space of type {nmi.cartesian_outcome_space.__class__.__name__}. Must use `CartesianOutcomeSpace`"
             )
-        self.issue_names = [_.name for _ in ami.issues]
-        self.issues = ami.issues
+        self.issues = nmi.cartesian_outcome_space.issues
+        self.issue_names = [_.name for _ in self.issues]
         self.issue_index = dict(zip(self.issue_names, range(len(self.issue_names))))
-        self.keep_issue_names = self.keep_value_names = ami.outcome_type == dict
-        if ami.issues is not None and self.domain_file_name is None:
+        if nmi.cartesian_outcome_space is not None and self.domain_file_name is None:
             domain_file = tempfile.NamedTemporaryFile("w", suffix=".xml", delete=False)
             self.domain_file_name = domain_file.name
-            domain_file.write(Issue.to_xml_str(ami.issues))
+            domain_file.write(issues_to_xml_str(nmi.cartesian_outcome_space.issues))
             domain_file.close()
             self._temp_domain_file = True
-        if ufun is not None and self.utility_file_name is None:
+        if preferences is not None and self.utility_file_name is None:
             utility_file = tempfile.NamedTemporaryFile("w", suffix=".xml", delete=False)
             self.utility_file_name = utility_file.name
             utility_file.write(
                 UtilityFunction.to_xml_str(
-                    ufun, issues=ami.issues, discount_factor=self.discount
+                    preferences,
+                    issues=nmi.cartesian_outcome_space.issues,
+                    discount_factor=self.discount,
                 )
             )
             utility_file.close()
-            self._temp_ufun_file = True
+            self._temp_preferences_file = True
         return result
 
+    def _outcome2str(self, outcome):
+        output = ""
+        if not self.issue_names:
+            nmi = self.nmi
+            if not nmi:
+                raise ValueError("Cannot send outcome to java without an NMI")
+            self.issue_names = [_.name for _ in nmi.cartesian_outcome_space.issues]
+        outcome_dict = dict(zip(self.issue_names, outcome))
+        for i, v in outcome_dict.items():
+            # todo check that the order here will be correct!!
+            output += f"{i}{INTERNAL_SEP}{v}{ENTRY_SEP}"
+        output = output[: -len(ENTRY_SEP)]
+        return output
+
     def destroy_java_counterpart(self, state=None) -> None:
         if self.__started and not self.__destroyed:
             if self.java is not None:
-                self.__frozen_relative_time = self.java.get_relative_time(
+                self.__frozen_relative_time = self.java.get_relative_time(  # type: ignore
                     self.java_uuid
                 )
-                result = self.java.on_negotiation_end(
+                result = self.java.on_negotiation_end(  # type: ignore
                     self.java_uuid,
                     None
                     if state is None
@@ -347,7 +347,7 @@ class GeniusNegotiator(SAONegotiator):
                     else None,
                 )
                 if result in (OK, TIMEOUT):
-                    results = self.java.destroy_agent(self.java_uuid)
+                    result = self.java.destroy_agent(self.java_uuid)  # type: ignore
                     if result in (FAILED, TIMEOUT) and self._strict:
                         raise ValueError(
                             f"{self._me()} ended the negotiation but failed to destroy the agent. A possible memory leak"
@@ -355,12 +355,12 @@ class GeniusNegotiator(SAONegotiator):
                 elif self._strict:
                     raise ValueError(f"{self._me()} failed to end the negotiation!!")
             self.__destroyed = True
-        if self._temp_ufun_file:
+        if self._temp_preferences_file:
             try:
                 os.unlink(self.utility_file_name)  # type: ignore
             except (FileNotFoundError, PermissionError):
                 pass
-            self._temp_ufun_file = False
+            self._temp_preferences_file = False
 
         if self._temp_domain_file:
             try:
@@ -377,29 +377,45 @@ class GeniusNegotiator(SAONegotiator):
     def on_negotiation_start(self, state: MechanismState) -> None:
         """Called when the info starts. Connects to the JVM."""
         super().on_negotiation_start(state=state)
-        if self._strict is None:
-            self._strict = self.ami.n_steps is not None and self.ami.n_steps != float(
+        if self._strict is None and self.nmi is not None:
+            self._strict = self.nmi.n_steps is not None and self.nmi.n_steps != float(
                 "inf"
             )
-        if self._utility_function is not None and self.utility_file_name is None:
+        info = self.nmi
+        if info is None:
+            raise ValueError("Cannot start a negotiation without a NMI")
+        if self._preferences is not None and self.utility_file_name is None:
+            domain_file = tempfile.NamedTemporaryFile("w", suffix=".xml", delete=False)
+            self.domain_file_name = domain_file.name
+            domain_file.write(issues_to_xml_str(self._preferences.outcome_space.issues))  # type: ignore
+            domain_file.close()
+            self._temp_domain_file = True
             utility_file = tempfile.NamedTemporaryFile("w", suffix=".xml", delete=False)
             self.utility_file_name = utility_file.name
+            if not isinstance(self.ufun, UtilityFunction):
+                raise ValueError(
+                    f"Genius Negotiator must have `UtilityFunctions` but you passed {self.ufun.__class__.__name__}"
+                )
             utility_file.write(
                 UtilityFunction.to_xml_str(
-                    self._utility_function,
-                    issues=self.ami.issues,
+                    self.ufun,
+                    issues=info.cartesian_outcome_space.issues,
                     discount_factor=self.discount,
                 )
             )
             utility_file.close()
-            self._temp_ufun_file = True
-        info = self._ami
-        if self.discount is not None and self.discount != 1.0:
-            self._utility_function = make_discounted_ufun(
-                self._utility_function,
-                ami=info,
-                discount_per_round=self.discount,
-                power_per_round=1.0,
+            self._temp_preferences_file = True
+        if self.discount is not None and self.discount != 1.0 and self._preferences:
+            if not isinstance(self.ufun, UtilityFunction):
+                raise ValueError(
+                    f"Genius Negotiator must have `UtilityFunctions` but you passed {self.ufun.__class__.__name__}"
+                )
+            self.set_preferences(
+                make_discounted_ufun(
+                    self.ufun,
+                    discount_per_round=self.discount,
+                    power_per_round=1.0,
+                )
             )
         n_steps = -1 if info.n_steps is None else int(info.n_steps)  # number of steps
         n_seconds = (
@@ -416,21 +432,36 @@ class GeniusNegotiator(SAONegotiator):
             if info.time_limit and not math.isinf(info.time_limit)
             else DEFAULT_GENIUS_NEGOTIATOR_TIMEOUT
         )
-        if timeout is None or math.isinf(timeout) or timeout <= 0:
+        if timeout is None or math.isinf(timeout) or timeout < 0:
             timeout = DEFAULT_GENIUS_NEGOTIATOR_TIMEOUT
 
+        if info.mechanism:
+            timeout = min(timeout, info.mechanism._hidden_time_limit)
+
         if n_steps * n_seconds > 0:
-            # n_seconds take precedence
-            if self._strict:
-                raise ValueError(
-                    f"{self._me()}: Both n_steps ({n_steps}) and n_seconds ({n_seconds}) are given. Not allowed in strict execution"
+            if n_steps < 0:
+                if self._strict:
+                    raise ValueError(
+                        f"{self._me()}: Neither n_steps ({n_steps}) nor n_seconds ({n_seconds}) are given. Not allowed in strict mode"
+                    )
+                warnings.warn(
+                    f"{self._me()}: Neither n_steps ({n_steps}) nor n_seconds ({n_seconds}) are given. This may lead to an infinite negotiation",
+                    warnings.NegmasInfiniteNegotiationWarning,
                 )
-            warnings.warn(
-                f"{self._me()}: Both n_steps ({n_steps}) and n_seconds ({n_seconds}) are given. n_steps will be ignored"
-            )
-            n_steps = -1
+            else:
+                # n_steps take precedence
+                if self._strict:
+                    raise ValueError(
+                        f"{self._me()}: Both n_steps ({n_steps}) and n_seconds ({n_seconds}) are given. Not allowed in strict execution"
+                    )
+                warnings.warn(
+                    f"{self._me()}: Both n_steps ({n_steps}) and n_seconds ({n_seconds}) are given. time_limit will be ignored",
+                    warnings.NegmasStepAndTimeLimitWarning,
+                )
+                n_seconds, timeout = -1, -1
+        timeout = int(timeout)
         try:
-            result = self.java.on_negotiation_start(
+            result = self.java.on_negotiation_start(  # type: ignore
                 self.java_uuid,  # java_uuid
                 info.n_negotiators,  # number of agents
                 n_steps,
@@ -438,26 +469,28 @@ class GeniusNegotiator(SAONegotiator):
                 n_seconds > 0,
                 self.domain_file_name,  # domain file
                 self.utility_file_name,  # Negotiator file
-                int(timeout),
+                timeout,
                 self._strict,
             )
             self.__started = result == OK
         except Exception as e:
-            raise ValueError(f"{self._me()}: Cannot start negotiation: {str(e)}")
+            raise ValueError(
+                f"{self._me()}: Cannot start negotiation: {str(e)}\nDomain file: {self.domain_file_name}\nUFun file: {self.domain_file_name}\n{'strict' if self._strict else 'non-strict'} {n_steps=} {timeout=}"
+            )
 
     def cancel(self, reason=None) -> None:
         try:
-            self.java.cancel(self.java_uuid)
+            self.java.cancel(self.java_uuid)  # type: ignore
         except:
             pass
 
     @property
-    def relative_time(self) -> Optional[float]:
-        if self.ami is None or not self.ami.state.started:
+    def relative_time(self) -> float | None:
+        if self.nmi is None or not self.nmi.state.started:
             return 0
-        if self.ami is not None and self.ami.state.ended:
+        if self.nmi is not None and self.nmi.state.ended:
             return self.__frozen_relative_time
-        t = self.java.get_relative_time(self.java_uuid)
+        t = self.java.get_relative_time(self.java_uuid)  # type: ignore
         if t < 0:
             if self._strict:
                 raise ValueError(
@@ -471,46 +504,7 @@ class GeniusNegotiator(SAONegotiator):
             f"Agent {self.name} (jid: {self.java_uuid}) of type {self.java_class_name}"
         )
 
-    def counter(self, state: MechanismState, offer: Optional["Outcome"]):
-        if offer is None and self._my_last_offer is not None and self._strict:
-            raise ValueError(f"{self._me()} got counter with a None offer.")
-        if offer is not None:
-            received = self.java.receive_message(
-                self.java_uuid,
-                state.current_proposer,
-                "Offer",
-                self._outcome2str(offer),
-                state.step,
-            )
-            if self._strict and received == FAILED:
-                raise ValueError(
-                    f"{self._me()} failed to receive message in step {state.step}"
-                )
-        response, outcome = self.parse(
-            self.java.choose_action(self.java_uuid, state.step)
-        )
-        if self._strict and (
-            response
-            not in (
-                ResponseType.REJECT_OFFER,
-                ResponseType.ACCEPT_OFFER,
-                ResponseType.END_NEGOTIATION,
-            )
-            or (response == ResponseType.REJECT_OFFER and outcome is None)
-        ):
-            raise ValueError(
-                f"{self._me()} returned a None counter offer in step {state.step}"
-            )
-
-        self._my_last_offer = outcome
-        return SAOResponse(response, outcome)
-
-    def propose(self, state):
-        raise ValueError(
-            f"{self._me()}: propose should never be called directly on GeniusNegotiator"
-        )
-
-    def parse(self, action: str) -> Tuple[Optional[ResponseType], Optional["Outcome"]]:
+    def parse(self, action: str) -> tuple[ResponseType, Outcome | None]:
         """
         Parses an action into a ResponseType and an Outcome (if one is included)
         Args:
@@ -519,64 +513,52 @@ class GeniusNegotiator(SAONegotiator):
         Returns:
 
         """
-        response, outcome = None, None
+        outcome = None
         if len(action) < 1:
             if self._strict:
                 raise ValueError(f"{self._me()} received no actions while parsing")
             return ResponseType.REJECT_OFFER, None
         _, typ_, bid_str = action.split(FIELD_SEP)
+        nmi = self.nmi
+        if not nmi:
+            raise ValueError("Cannot parse without an NMI")
         if typ_ == FAILED:
             raise ValueError(
                 f"{self._me()} sent an action that cannot be parsed ({action})"
             )
         elif typ_ == TIMEOUT:
-            if self.ami.state.relative_time < 1.0:
+            if nmi.state.relative_time < 1.0:
                 raise ValueError(
-                    f"{self._me()} indicated that it timedout at relative time ({self.ami.state.relative_time})"
+                    f"{self._me()} indicated that it timedout at relative time ({nmi.state.relative_time})"
                 )
 
-        issues = self._ami.issues
+        issues = nmi.cartesian_outcome_space.issues
+
+        def map_value(issue: Issue, val: str):
+            if not issue.value_type:
+                return val
+            if issubclass(issue.value_type, tuple):
+                return eval(val)
+            return issue.value_type(val)  # type: ignore (It will always be a constructable type)
 
         if typ_ in ("Offer",) and (bid_str is not None and len(bid_str) > 0):
             try:
-                if self._ami.outcome_type == tuple:
-                    values = {
-                        _[0]: _[1]
-                        for _ in [
-                            _.split(INTERNAL_SEP) for _ in bid_str.split(ENTRY_SEP)
-                        ]
-                    }
-                    outcome = []
-                    for issue in issues:
-                        outcome.append(issue.value_type(values[issue.name]))
-                    outcome = tuple(outcome)
-                else:
-                    outcome = {
-                        _[0]: _[1]
-                        for _ in [
-                            _.split(INTERNAL_SEP) for _ in bid_str.split(ENTRY_SEP)
-                        ]
-                    }
-                    issue_map = {i.name: i for i in issues}
-                    for k, v in outcome.items():
-                        outcome[k] = issue_map[k].value_type(v)
-                    if self._ami.outcome_type != dict:
-                        outcome = self._ami.outcome_type(
-                            **{
-                                _[0]: _[1]
-                                for _ in [
-                                    _.split(INTERNAL_SEP)
-                                    for _ in bid_str.split(ENTRY_SEP)
-                                ]
-                            }
-                        )
+                values = {
+                    _[0]: _[1]
+                    for _ in [_.split(INTERNAL_SEP) for _ in bid_str.split(ENTRY_SEP)]
+                }
+                outcome = []
+                for issue in issues:
+                    outcome.append(map_value(issue, values[issue.name]))
+                outcome = tuple(outcome)
             except Exception as e:
                 if self._strict:
                     raise ValueError(
                         f"{self._me()} failed to parse {bid_str} of action {action} with exception {str(e)}"
                     )
                 warnings.warn(
-                    f"{self._me()} failed in parsing bid string: {bid_str} of action {action} with exception {str(e)}"
+                    f"{self._me()} failed in parsing bid string: {bid_str} of action {action} with exception {str(e)}",
+                    warnings.NegmasBrdigeParsingWarning,
                 )
 
         if typ_ in (TIMEOUT, "Offer"):
@@ -594,19 +576,49 @@ class GeniusNegotiator(SAONegotiator):
             raise ValueError(
                 f"{self._me()}: Unknown response: {typ_} in action {action}"
             )
-        return response, outcome
+        return response, tuple(outcome) if outcome else None
 
-    def _outcome2str(self, outcome):
-        output = ""
-        if not isinstance(outcome, dict):
-            outcome_dict = dict(zip(self.issue_names, outcome))
-        else:
-            outcome_dict = outcome
-        for i, v in outcome_dict.items():
-            # todo check that the order here will be correct!!
-            output += f"{i}{INTERNAL_SEP}{v}{ENTRY_SEP}"
-        output = output[: -len(ENTRY_SEP)]
-        return output
+    def counter(self, state: SAOState, offer: Outcome | None):
+        if offer is None and self._my_last_offer is not None and self._strict:
+            raise ValueError(f"{self._me()} got counter with a None offer.")
+        if offer is not None:
+            received = self.java.receive_message(  # type: ignore
+                self.java_uuid,
+                state.current_proposer,
+                "Offer",
+                self._outcome2str(offer),
+                state.step,
+            )
+            if self._strict and received == FAILED:
+                raise ValueError(
+                    f"{self._me()} failed to receive message in step {state.step}"
+                )
+        response, outcome = self.parse(
+            self.java.choose_action(self.java_uuid, state.step)  # type: ignore
+        )
+        if response is None or (
+            self._strict
+            and (
+                response
+                not in (
+                    ResponseType.REJECT_OFFER,
+                    ResponseType.ACCEPT_OFFER,
+                    ResponseType.END_NEGOTIATION,
+                )
+                or (response == ResponseType.REJECT_OFFER and outcome is None)
+            )
+        ):
+            raise ValueError(
+                f"{self._me()} returned a None counter offer in step {state.step}"
+            )
+
+        self._my_last_offer = outcome
+        return SAOResponse(response, outcome)
+
+    def propose(self, state):
+        raise ValueError(
+            f"{self._me()}: propose should never be called directly on GeniusNegotiator"
+        )
 
     def __str__(self):
         name = super().__str__().split("/")
