@@ -1,21 +1,26 @@
-"""Implements single text negotiation mechanisms"""
+"""
+Implements single text negotiation mechanisms
+"""
+from __future__ import annotations
+
 import time
 from copy import deepcopy
-from dataclasses import dataclass, field
-from typing import List, Optional, Tuple
+
+from attr import define, field
 
 from .mechanisms import Mechanism, MechanismRoundResult, MechanismState
+from .outcomes import Outcome
 
 __all__ = [
     "VetoMTMechanism",
 ]
 
 
-@dataclass
+@define
 class MTState(MechanismState):
     """Defines extra values to keep in the mechanism state. This is accessible to all negotiators"""
 
-    current_offers: List[Optional["Outcome"]] = field(default_factory=list)
+    current_offers: list[Outcome | None] = field(factory=list)
 
 
 class VetoMTMechanism(Mechanism):
@@ -43,21 +48,23 @@ class VetoMTMechanism(Mechanism):
         *args,
         epsilon: float = 1e-6,
         n_texts: int = 10,
-        initial_outcomes=None,
-        initial_responses: Tuple[Tuple[bool]] = None,
-        **kwargs
+        initial_outcomes: list[Outcome | None] | None = None,
+        initial_responses: tuple[tuple[bool]] | None = None,
+        **kwargs,
     ):
         kwargs["state_factory"] = MTState
         super().__init__(*args, **kwargs)
+        self._current_state: MTState
+        state = self._current_state
 
         self.add_requirements(
             {"compare-binary": True}
         )  # assert that all agents must have compare-binary capability
-        self.current_offers = (
+        state.current_offers = (
             initial_outcomes if initial_outcomes is not None else [None] * n_texts
         )
         """The current offer"""
-        self.initial_outcomes = deepcopy(self.current_offers.copy)
+        self.initial_outcomes = deepcopy(state.current_offers)
         """The initial offer"""
         self.last_responses = (
             [list(_) for _ in initial_responses]
@@ -69,12 +76,7 @@ class VetoMTMechanism(Mechanism):
         """The initial set of responses. See the remarks of this class to understand its role."""
         self.epsilon = epsilon
 
-    def extra_state(self):
-        return dict(
-            current_offers=deepcopy(self.current_offers),
-        )
-
-    def next_outcome(self, outcome: Optional["Outcome"]) -> Optional["Outcome"]:
+    def next_outcome(self, outcome: Outcome | None) -> Outcome | None:
         """Generate the next outcome given some outcome.
 
         Args:
@@ -88,7 +90,8 @@ class VetoMTMechanism(Mechanism):
 
     def round(self) -> MechanismRoundResult:
         """Single round of the protocol"""
-        for i, current_offer in enumerate(self.current_offers):
+        state: SAOState = self._current_state  # type: ignore
+        for i, current_offer in enumerate(state.current_offers):
             new_offer = self.next_outcome(current_offer)
             responses = []
 
@@ -98,7 +101,7 @@ class VetoMTMechanism(Mechanism):
                     neg.is_better(new_offer, current_offer, epsilon=self.epsilon)
                     is not False
                 )
-                if time.perf_counter() - strt > self.ami.step_time_limit:
+                if time.perf_counter() - strt > self.nmi.step_time_limit:
                     return MechanismRoundResult(
                         broken=False, timedout=True, agreement=None
                     )
@@ -106,12 +109,12 @@ class VetoMTMechanism(Mechanism):
             self.last_responses = responses
 
             if all(responses):
-                self.current_offers[i] = new_offer
+                state.current_offers[i] = new_offer
 
         return MechanismRoundResult(broken=False, timedout=False, agreement=None)
 
     def on_negotiation_end(self) -> None:
         """Used to pass the final offer for agreement between all negotiators"""
-        self._agreement = self.current_offers
+        self._agreement = self._current_state.current_offers
 
         super().on_negotiation_end()
