@@ -2,12 +2,14 @@
 """
 Implements Stacked Alternating Offers (SAO) mechanism and basic negotiators.
 """
+from __future__ import annotations
 
 from .common import *
 from .components import *
 from .controllers import *
 from .mechanism import *
 from .negotiators import *
+from .plots import *
 
 __all__ = (
     common.__all__
@@ -15,4 +17,5 @@ __all__ = (
     + mechanism.__all__
     + negotiators.__all__
     + controllers.__all__
+    + plots.__all__
 )
