@@ -1,19 +1,25 @@
 """
 Implements controllers for the SAO mechanism.
 """
+from __future__ import annotations
+
 import itertools
 import random
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from functools import lru_cache
-from typing import Dict, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Literal, TypeVar
 
-from ..common import AgentMechanismInterface, MechanismState
-from ..negotiators import AspirationMixin, Controller
-from ..outcomes import Outcome, ResponseType, outcome_is_valid
-from ..utilities import utility_range
-from .common import SAOResponse, SAOState
-from .negotiators import AspirationNegotiator, PassThroughSAONegotiator, SAONegotiator
+from negmas.preferences.protocols import UFun
+
+from ..common import MechanismState, NegotiatorMechanismInterface
+from ..negotiators import Controller
+from ..negotiators.helpers import PolyAspiration
+from ..outcomes import Outcome, outcome_is_valid
+from .common import ResponseType, SAOResponse, SAOState
+from .negotiators import AspirationNegotiator, ControlledSAONegotiator, SAONegotiator
+
+if TYPE_CHECKING:
+    from negmas.preferences import Preferences
 
 __all__ = [
     "SAOController",
@@ -26,6 +32,8 @@ __all__ = [
     "SAOMetaNegotiatorController",
 ]
 
+ControlledNegotiatorType = TypeVar("ControlledNegotiatorType", bound=SAONegotiator)
+
 
 class SAOController(Controller):
     """
@@ -33,44 +41,49 @@ class SAOController(Controller):
 
     Args:
          default_negotiator_type: Default type to use when creating negotiators using this controller. The default type is
-                                  `PassThroughSAONegotiator` which passes *full control* to the controller.
+                                  `ControlledSAONegotiator` which passes *full control* to the controller.
          default_negotiator_params: Default paramters to pass to the default controller.
          auto_kill: Automatically kill the negotiator once its negotiation session is ended.
          name: Controller name
-         ufun: The ufun of the controller.
+         preferences: The preferences of the controller.
+         ufun: The ufun of the controller (overrides `preferences`).
     """
 
     def __init__(
         self,
-        default_negotiator_type=PassThroughSAONegotiator,
+        default_negotiator_type=ControlledSAONegotiator,
         default_negotiator_params=None,
-        auto_kill=False,
+        auto_kill=True,
         name=None,
+        preferences=None,
         ufun=None,
     ):
+        if ufun is not None:
+            preferences = ufun
         super().__init__(
             default_negotiator_type=default_negotiator_type,
             default_negotiator_params=default_negotiator_params,
             auto_kill=auto_kill,
             name=name,
+            preferences=preferences,
             ufun=ufun,
         )
 
     def before_join(
         self,
         negotiator_id: str,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        preferences: Preferences | None = None,
+        role: str = "negotiator",
     ) -> bool:
         """
         Called by children negotiators to get permission to join negotiations
 
         Args:
             negotiator_id: The negotiator ID
-            ami  (AgentMechanismInterface): The negotiation.
+            nmi  (NegotiatorMechanismInterface): The negotiation.
             state (MechanismState): The current state of the negotiation
             ufun (UtilityFunction): The ufun function to use before any discounting.
             role (str): role of the agent.
@@ -82,14 +95,17 @@ class SAOController(Controller):
         """
         return True
 
-    def after_join(
+    def create_negotiator(
         self,
-        negotiator_id: str,
-        ami: AgentMechanismInterface,
-        state: MechanismState,
-        *,
-        ufun: Optional["UtilityFunction"] = None,
-        role: str = "agent",
+        negotiator_type: str | ControlledNegotiatorType | None = None,
+        name: str | None = None,
+        cntxt: Any = None,
+        **kwargs,
+    ) -> ControlledNegotiatorType:
+        return super().create_negotiator(negotiator_type, name, cntxt, **kwargs)  # type: ignore I know that the return type is an SAONegotiator
+
+    def after_join(
+        self, negotiator_id, *args, ufun=None, preferences=None, **kwargs
     ) -> None:
         """
         Called by children negotiators after joining a negotiation to inform
@@ -97,13 +113,13 @@ class SAOController(Controller):
 
         Args:
             negotiator_id: The negotiator ID
-            ami  (AgentMechanismInterface): The negotiation.
+            nmi  (NegotiatorMechanismInterface): The negotiation.
             state (MechanismState): The current state of the negotiation
             ufun (UtilityFunction): The ufun function to use before any discounting.
             role (str): role of the agent.
         """
 
-    def propose(self, negotiator_id: str, state: MechanismState) -> Optional["Outcome"]:
+    def propose(self, negotiator_id: str, state: MechanismState) -> Outcome | None:
 
         negotiator, _ = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
@@ -111,8 +127,8 @@ class SAOController(Controller):
         return self.call(negotiator, "propose", state=state)
 
     def respond(
-        self, negotiator_id: str, state: MechanismState, offer: "Outcome"
-    ) -> "ResponseType":
+        self, negotiator_id: str, state: MechanismState, offer: Outcome
+    ) -> ResponseType:
         negotiator, cntxt = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
             raise ValueError(f"Unknown negotiator {negotiator_id}")
@@ -139,18 +155,18 @@ class SAORandomController(SAOController):
         super().__init__(*args, **kwargs)
         self._p_acceptance = p_acceptance
 
-    def propose(self, negotiator_id: str, state: MechanismState) -> Optional["Outcome"]:
+    def propose(self, negotiator_id: str, state: MechanismState) -> Outcome | None:
 
         negotiator, cntxt = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
             raise ValueError(f"Unknown negotiator {negotiator_id}")
-        if negotiator.ami is None:
+        if negotiator.nmi is None:
             return None
-        return negotiator.ami.random_outcomes(1)[0]
+        return negotiator.nmi.random_outcomes(1)[0]
 
     def respond(
-        self, negotiator_id: str, state: MechanismState, offer: "Outcome"
-    ) -> "ResponseType":
+        self, negotiator_id: str, state: MechanismState, offer: Outcome
+    ) -> ResponseType:
         negotiator, cntxt = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
             raise ValueError(f"Unknown negotiator {negotiator_id}")
@@ -177,24 +193,62 @@ class SAOSyncController(SAOController):
 
     """
 
-    def __init__(self, *args, global_ufun=False, **kwargs):
+    def __init__(self, *args, global_ufun=None, **kwargs):
         super().__init__(*args, **kwargs)
-        self.__offers: Dict[str, "Outcome"] = {}
-        """Keeps the last offer received for each negotiation"""
-        self.__responses: Dict[str, ResponseType] = {}
-        """Keeps the next response type for each negotiation"""
-        self.__proposals: Dict[str, "Outcome"] = {}
-        """Keeps the next proposal for each negotiation"""
-        self.__offer_states: Dict[str, "SAOState"] = {}
-        """Keeps the last state received for each negotiation"""
-        self.__n_waits: Dict[str, int] = defaultdict(int)
-        """The number of contiguous waits sent for every partner"""
         self.__global_ufun = global_ufun
+        self.reset()
 
-    def propose(self, negotiator_id: str, state: MechanismState) -> Optional["Outcome"]:
+    def reset(self):
+        self.__offers: dict[str, Outcome] = dict()
+        self.__responses: dict[str, ResponseType] = dict()
+        self.__proposals: dict[str, Outcome | None] = dict()
+        self.__offer_states: dict[str, SAOState] = dict()
+        self.__n_waits: dict[str, int] = defaultdict(int)
+        self.__first_proposals_collected = False
+        super().reset()
+
+    def first_offer(self, negotiator_id: str) -> Outcome | None:
+        """
+        Finds the first offer for this given negotiator. By default it will be the best offer
+
+        Args:
+            negotiator_id: The ID of the negotiator
+
+        Returns:
+            The first offer to use.
+
+        Remarks:
+            Default behavior is to use the ufun defined for the controller if any then try the ufun
+            defined for the negotiator. If neither exists, the first offer will be None.
+        """
+        negotiator, _ = self.negotiators.get(negotiator_id, (None, None))
+        if negotiator is None or negotiator.nmi is None:
+            return None
+        # if the controller has a ufun, use it otherwise use the negotiator's
+        try:
+            ufun = self.ufun
+        except ValueError:
+            ufun = negotiator.ufun
+        if isinstance(ufun, UFun):
+            _, best = ufun.extreme_outcomes()
+        else:
+            best = None
+        return best
+
+    def first_proposals(self) -> dict[str, Outcome | None]:
+        """Gets a set of proposals to use for initializing the negotiation."""
+        return dict(
+            zip(
+                self.negotiators.keys(),
+                (self.first_offer(_) for _ in self.negotiators.keys()),
+            )
+        )
+
+    def propose(self, negotiator_id: str, state: MechanismState) -> Outcome | None:
         # if there are no proposals yet, get first proposals
         if not self.__proposals:
             self.__proposals = self.first_proposals()
+            self.__first_proposals_collected = True
         # get the saved proposal if it exists and return it
         if negotiator_id in self.__proposals.keys():
             # if some proposal was there, delete it to force the controller to get a new one
@@ -203,6 +257,7 @@ class SAOSyncController(SAOController):
             # if there was no proposal, get one. Note that `None` is a valid proposal
             if self.__global_ufun:
                 self.__proposals = self.first_proposals()
+                self.__first_proposals_collected = True
                 proposal = self.__proposals.pop(negotiator_id)
             else:
                 proposal = self.first_offer(negotiator_id)
@@ -214,9 +269,26 @@ class SAOSyncController(SAOController):
         self.__offer_states.pop(negotiator_id, None)
         return proposal
 
+    @abstractmethod
+    def counter_all(
+        self, offers: dict[str, Outcome], states: dict[str, SAOState]
+    ) -> dict[str, SAOResponse]:
+        """Calculate a response to all offers from all negotiators (negotiator ID is the key).
+
+        Args:
+            offers: Maps negotiator IDs to offers
+            states: Maps negotiator IDs to offers AT the time the offers were made.
+
+        Remarks:
+            - The response type CANNOT be WAIT.
+            - If the system determines that a loop is formed, the agent may receive this call for a subset of
+              negotiations not all of them.
+
+        """
+
     def respond(
-        self, negotiator_id: str, state: MechanismState, offer: "Outcome"
-    ) -> "ResponseType":
+        self, negotiator_id: str, state: SAOState, offer: Outcome
+    ) -> ResponseType:
         # get the saved response to this negotiator if any
         response = self.__responses.pop(negotiator_id, ResponseType.WAIT)
 
@@ -240,80 +312,29 @@ class SAOSyncController(SAOController):
             self.__n_waits[negotiator_id] += 1
             return ResponseType.WAIT
 
-        # we arrive here if we already have all the offers to counter
+        # we arrive here if we already have all the offers to counter. WE may though not have proposed yet
+        if not self.__first_proposals_collected:
+            self.__proposals = self.first_proposals()
+            self.__first_proposals_collected = True
         responses = self.counter_all(offers=self.__offers, states=self.__offer_states)
-        for nid in responses.keys():
-            saved_response = responses.get(nid, None)
+        for neg in responses.keys():
+            saved_response = responses.get(neg, None)
             if saved_response is None:
-                self.__responses[nid] = ResponseType.REJECT_OFFER
-                self.__proposals[nid] = None
+                self.__responses[neg] = ResponseType.REJECT_OFFER
+                self.__proposals[neg] = None
                 continue
             # register the responses for next time for all other negotiators
-            self.__responses[nid] = saved_response.response
+            self.__responses[neg] = saved_response.response
             # register the proposals to be sent to all agents including this one
-            self.__proposals[nid] = saved_response.outcome
+            self.__proposals[neg] = saved_response.outcome
             # register that we are not waiting anymore on any of the offers we received
-            self.__n_waits[nid] = 0
+            self.__n_waits[neg] = 0
         self.__offers = dict()
         self.__offer_states = dict()
+        if negotiator_id not in self.__responses:
+            return ResponseType.REJECT_OFFER
         return self.__responses.pop(negotiator_id)
 
-    def first_proposals(self) -> Dict[str, "Outcome"]:
-        """Gets a set of proposals to use for initializing the negotiation."""
-        return dict(
-            zip(
-                self.negotiators.keys(),
-                (self.first_offer(_) for _ in self.negotiators.keys()),
-            )
-        )
-
-    @abstractmethod
-    def counter_all(
-        self, offers: Dict[str, "Outcome"], states: Dict[str, SAOState]
-    ) -> Dict[str, SAOResponse]:
-        """Calculate a response to all offers from all negotiators (negotiator ID is the key).
-
-        Args:
-            offers: Maps negotiator IDs to offers
-            states: Maps negotiator IDs to offers AT the time the offers were made.
-
-        Remarks:
-            - The response type CANNOT be WAIT.
-            - If the system determines that a loop is formed, the agent may receive this call for a subset of
-              negotiations not all of them.
-
-        """
-
-    def first_offer(self, negotiator_id: str) -> Optional["Outcome"]:
-        """
-        Finds the first offer for this given negotiator. By default it will be the best offer
-
-        Args:
-            negotiator_id: The ID of the negotiator
-
-        Returns:
-            The first offer to use.
-
-        Remarks:
-            Default behavior is to use the ufun defined for the controller if any then try the ufun
-            defined for the negotiator. If neither exists, the first offer will be None.
-        """
-        negotiator, _ = self.negotiators.get(negotiator_id, (None, None))
-        if negotiator is None or negotiator.ami is None:
-            return None
-        # if the controller has a ufun, use it otherwise use the negotiator's
-        ufun = self.ufun if self.ufun is not None else negotiator.ufun
-        if ufun is not None:
-            _, _, _, best = utility_range(
-                ufun,
-                issues=negotiator.ami.issues,
-                return_outcomes=True,
-                ami=negotiator.ami,
-            )
-        else:
-            best = None
-        return best
-
     def on_negotiation_end(self, negotiator_id: str, state: MechanismState) -> None:
         if negotiator_id in self.__offers.keys():
             del self.__offers[negotiator_id]
@@ -325,7 +346,10 @@ class SAOSyncController(SAOController):
             del self.__proposals[negotiator_id]
         if negotiator_id in self.__n_waits.keys():
             del self.__n_waits[negotiator_id]
-        return super().on_negotiation_end(negotiator_id, state)
+        results = super().on_negotiation_end(negotiator_id, state)
+        if not self.negotiators:
+            self.reset()
+        return results
 
 
 class SAORandomSyncController(SAOSyncController):
@@ -351,7 +375,7 @@ class SAORandomSyncController(SAOSyncController):
         self.p_acceptance = p_acceptance
         self.p_rejection = p_rejection
         self.p_ending = p_ending
-        self.wheel: List[Tuple[float, ResponseType]] = [(0.0, ResponseType.NO_RESPONSE)]
+        self.wheel: list[tuple[float, ResponseType]] = [(0.0, ResponseType.NO_RESPONSE)]
         if self.p_acceptance > 0.0:
             self.wheel.append(
                 (self.wheel[-1][0] + self.p_acceptance, ResponseType.ACCEPT_OFFER)
@@ -369,7 +393,7 @@ class SAORandomSyncController(SAOSyncController):
 
         self.wheel = self.wheel[1:]
 
-    def make_response(self) -> "ResponseType":
+    def make_response(self) -> ResponseType:
         r = random.random()
         for w in self.wheel:
             if w[0] >= r:
@@ -382,7 +406,7 @@ class SAORandomSyncController(SAOSyncController):
             response = self.make_response()
             if response == ResponseType.REJECT_OFFER:
                 result[negotiator] = SAOResponse(
-                    response, self.negotiators[negotiator][0].ami.random_outcomes(1)[0]
+                    response, self.negotiators[negotiator][0].nmi.random_outcomes(1)[0]
                 )
             else:
                 result[negotiator] = SAOResponse(response, None)
@@ -392,7 +416,7 @@ class SAORandomSyncController(SAOSyncController):
         return dict(
             zip(
                 self.negotiators.keys(),
-                [n[0].ami.random_outcomes(1)[0] for n in self.negotiators.values()],
+                [n[0].nmi.random_outcomes(1)[0] for n in self.negotiators.values()],
             )
         )
 
@@ -443,9 +467,125 @@ class SAOSingleAgreementController(SAOSyncController, ABC):
         if state.agreement is not None:
             self.__end_all = True
 
+    def best_outcome(
+        self, negotiator: str, state: SAOState | None = None
+    ) -> Outcome | None:
+        """
+        The best outcome for the negotiation `negotiator` engages in given the
+        `state` .
+
+        Args:
+            negotiator: The negotiator for which the best outcome is to be found
+            state: If given, the state of the negotiation. If None, should
+                   return the absolute best outcome
+
+        Return:
+            The outcome with maximum utility.
+
+        Remarks:
+
+            - The default implementation, just returns the best outcome for
+              this negotiation without considering the `state` or returns None
+              if it is not possible to find this best outcome.
+            - If the negotiator defines a ufun, it is used otherwise the ufun
+              defined for the controller if used (if any)
+        """
+        neg = self.negotiators[negotiator][0]
+        if neg is None:
+            return None
+        if neg.nmi is None:
+            return None
+        ufun = neg.ufun
+        if ufun is None and hasattr(self, "ufun"):
+            ufun = self.ufun
+        if ufun is None:
+            return None
+        _, top_outcome = ufun.extreme_outcomes(outcome_space=neg.nmi.outcome_space)
+        return top_outcome
+
+    @abstractmethod
+    def is_better(
+        self, a: Outcome | None, b: Outcome | None, negotiator: str, state: SAOState
+    ):
+        """Compares two outcomes of the same negotiation
+
+        Args:
+            a: Outcome
+            b: Outcome
+            negotiator: The negotiator for which the comparison is to be made
+            state: Current state of the negotiation
+
+        Returns:
+            True if utility(a) > utility(b)
+        """
+
+    def make_offer(
+        self,
+        negotiator: str,
+        state: SAOState,
+        best_offer: Outcome | None,
+        best_from: str | None,
+    ) -> Outcome | None:
+        """Generate an offer for the given partner
+
+        Args:
+            negotiator: The ID of the negotiator for who an offer is to be made.
+            state: The mechanism state of this partner
+            best_offer: The best offer received in this round. None means that
+                        no known best offers.
+            best_from: The ID of the negotiator that received the best offer
+
+        Returns:
+            The outcome to be offered to `negotiator` (None means no-offer)
+
+        Remarks:
+
+            Default behavior is to offer everyone `best_offer` if available
+            otherwise, it returns no offers (None). The `best_from` negotiator
+            will be sending the result of `best_outcome`.
+
+        """
+        current_best = self.best_outcome(negotiator, state)
+        if negotiator == best_from:
+            return current_best
+        nmi = self.negotiators[negotiator][0].nmi
+        if nmi is None:
+            return None
+        if best_offer is not None and outcome_is_valid(best_offer, nmi.issues):
+            if self.is_better(best_offer, current_best, negotiator, state):
+                return best_offer
+            return current_best
+
+    @abstractmethod
+    def is_acceptable(self, offer: Outcome, source: str, state: SAOState) -> bool:
+        """Should decide if the given offer is acceptable
+
+        Args:
+            offer: The offer being tested
+            source: The ID of the negotiator that received this offer
+            state: The state of the negotiation handled by that negotiator
+
+        Remarks:
+            - If True is returned, this offer will be accepted and all other
+              negotiations will be ended.
+        """
+
+    @abstractmethod
+    def best_offer(self, offers: dict[str, Outcome]) -> str | None:
+        """
+        Return the ID of the negotiator with the best offer
+
+        Args:
+            offers: A mapping from negotiator ID to the offer it received
+
+        Returns:
+            The ID of the negotiator with best offer. Ties should be broken.
+            Return None only if there is no way to calculate the best offer.
+        """
+
     def counter_all(
-        self, offers: Dict[str, "Outcome"], states: Dict[str, SAOState]
-    ) -> Dict[str, SAOResponse]:
+        self, offers: dict[str, Outcome], states: dict[str, SAOState]
+    ) -> dict[str, SAOResponse]:
         """
         Counters all responses
 
@@ -551,7 +691,7 @@ class SAOSingleAgreementController(SAOSyncController, ABC):
             )
         )
 
-    def first_proposals(self) -> Dict[str, "Outcome"]:
+    def first_proposals(self) -> dict[str, Outcome | None]:
         if not self.strict:
             return super().first_proposals()
 
@@ -565,8 +705,8 @@ class SAOSingleAgreementController(SAOSyncController, ABC):
         )
 
     def response_to_best_offer(
-        self, negotiator: str, state: SAOState, offer: "Outcome"
-    ) -> Optional["Outcome"]:
+        self, negotiator: str, state: SAOState, offer: Outcome
+    ) -> Outcome | None:
         """
         Return a response to the partner from which the best current offer was
         received
@@ -582,125 +722,13 @@ class SAOSingleAgreementController(SAOSyncController, ABC):
         """
         return self._best_outcomes[negotiator][0]
 
-    def after_join(self, negotiator_id, ami, state, *, ufun=None, role="agent"):
-        super().after_join(negotiator_id, ami, state, ufun=ufun, role=role)
-        self._best_outcomes[negotiator_id] = self.best_outcome(negotiator_id)
-
-    def best_outcome(
-        self, negotiator: str, state: Optional[SAOState] = None
-    ) -> Optional["Outcome"]:
-        """
-        The best outcome for the negotiation `negotiator` engages in given the
-        `state` .
-
-        Args:
-            negotiator: The negotiator for which the best outcome is to be found
-            state: If given, the state of the negotiation. If None, should
-                   return the absolute best outcome
-
-        Return:
-            The outcome with maximum utility.
-
-        Remarks:
-
-            - The default implementation, just returns the best outcome for
-              this negotiation without considering the `state` or returns None
-              if it is not possible to find this best outcome.
-            - If the negotiator defines a ufun, it is used otherwise the ufun
-              defined for the controller if used (if any)
-        """
-        neg = self.negotiators[negotiator][0]
-        if neg is None:
-            return None
-        if neg.ami is None:
-            return None
-        ufun = neg.ufun
-        if ufun is None and hasattr(self, "ufun"):
-            ufun = self.ufun
-        if ufun is None:
-            return None
-        _, _, _, top_outcome = ufun.utility_range(
-            issues=neg.ami.issues, return_outcomes=True, ami=neg.ami
+    def after_join(
+        self, negotiator_id, nmi, state, *, preferences=None, role="negotiator"
+    ):
+        super().after_join(
+            negotiator_id, nmi, state, preferences=preferences, role=role
         )
-        return top_outcome
-
-    def make_offer(
-        self,
-        negotiator: str,
-        state: SAOState,
-        best_offer: Optional["Outcome"],
-        best_from: Optional[str],
-    ) -> Optional["Outcome"]:
-        """Generate an offer for the given partner
-
-        Args:
-            negotiator: The ID of the negotiator for who an offer is to be made.
-            state: The mechanism state of this partner
-            best_offer: The best offer received in this round. None means that
-                        no known best offers.
-            best_from: The ID of the negotiator that received the best offer
-
-        Returns:
-            The outcome to be offered to `negotiator` (None means no-offer)
-
-        Remarks:
-
-            Default behavior is to offer everyone `best_offer` if available
-            otherwise, it returns no offers (None). The `best_from` negotiator
-            will be sending the result of `best_outcome`.
-
-        """
-        current_best = self.best_outcome(negotiator, state)
-        if negotiator == best_from:
-            return current_best
-        ami = self.negotiators[negotiator][0].ami
-        if ami is None:
-            return None
-        if best_offer is not None and outcome_is_valid(best_offer, ami.issues):
-            if self.is_better(best_offer, current_best, negotiator, state):
-                return best_offer
-            return current_best
-
-    @abstractmethod
-    def is_acceptable(self, offer: "Outcome", source: str, state: SAOState) -> bool:
-        """Should decide if the given offer is acceptable
-
-        Args:
-            offer: The offer being tested
-            source: The ID of the negotiator that received this offer
-            state: The state of the negotiation handled by that negotiator
-
-        Remarks:
-            - If True is returned, this offer will be accepted and all other
-              negotiations will be ended.
-        """
-
-    @abstractmethod
-    def best_offer(self, offers: Dict[str, "Outcome"]) -> Optional[str]:
-        """
-        Return the ID of the negotiator with the best offer
-
-        Args:
-            offers: A mapping from negotiator ID to the offer it received
-
-        Returns:
-            The ID of the negotiator with best offer. Ties should be broken.
-            Return None only if there is no way to calculate the best offer.
-        """
-
-    @abstractmethod
-    def is_better(self, a: "Outcome", b: "Outcome", negotiator: str, state: SAOState):
-        """Compares two outcomes of the same negotiation
-
-        Args:
-            a: "Outcome"
-            b: "Outcome"
-            negotiator: The negotiator for which the comparison is to be made
-            state: Current state of the negotiation
-
-        Returns:
-            True if utility(a) > utility(b)
-        """
+        self._best_outcomes[negotiator_id] = self.best_outcome(negotiator_id)
 
 
 class SAOMetaNegotiatorController(SAOController):
@@ -725,7 +753,7 @@ class SAOMetaNegotiatorController(SAOController):
 
     """
 
-    def __init__(self, *args, meta_negotiator: SAONegotiator = None, **kwargs):
+    def __init__(self, *args, meta_negotiator: SAONegotiator | None = None, **kwargs):
         super().__init__(*args, **kwargs)
         if meta_negotiator is None:
             meta_negotiator = AspirationNegotiator(
@@ -733,22 +761,22 @@ class SAOMetaNegotiatorController(SAOController):
             )
         self.meta_negotiator = meta_negotiator
 
-    def propose(self, negotiator_id: str, state: MechanismState) -> Optional["Outcome"]:
+    def propose(self, negotiator_id: str, state: SAOState) -> Outcome | None:
         """Uses the meta negotiator to propose"""
-        negotiator, cntxt = self._negotiators.get(negotiator_id, (None, None))
+        negotiator, _ = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
             raise ValueError(f"Unknown negotiator {negotiator_id}")
-        self.meta_negotiator._ami = negotiator.ami
+        self.meta_negotiator._nmi = negotiator.nmi
         return self.meta_negotiator.propose(state)
 
     def respond(
-        self, negotiator_id: str, state: MechanismState, offer: "Outcome"
-    ) -> "ResponseType":
+        self, negotiator_id: str, state: SAOState, offer: Outcome
+    ) -> ResponseType:
         """Uses the meta negotiator to respond"""
         negotiator, cntxt = self._negotiators.get(negotiator_id, (None, None))
         if negotiator is None:
             raise ValueError(f"Unknown negotiator {negotiator_id}")
-        self.meta_negotiator._ami = negotiator.ami
+        self.meta_negotiator._nmi = negotiator.nmi
         return self.meta_negotiator.respond(state=state, offer=offer)
 
 
@@ -761,29 +789,27 @@ class SAOSingleAgreementRandomController(SAOSingleAgreementController):
 
     """
 
-    def is_acceptable(self, offer: "Outcome", source: str, state: SAOState) -> bool:
-        return random.random() > self.p_acceptance
-
-    def best_offer(self, offers: Dict[str, "Outcome"]) -> Optional[str]:
-        return random.sample(list(offers.keys()), 1)[0]
-
-    def is_better(self, a: "Outcome", b: "Outcome", negotiator: str, state: SAOState):
-        return random.random() > 0.5
-
     def __init__(self, *args, p_acceptance=0.1, **kwargs):
         super().__init__(*args, **kwargs)
         self.p_acceptance = p_acceptance
 
+    def is_acceptable(self, offer: Outcome, source: str, state: SAOState) -> bool:
+        return random.random() > self.p_acceptance
 
-class SAOSingleAgreementAspirationController(
-    SAOSingleAgreementController, AspirationMixin
-):
+    def best_offer(self, offers: dict[str, Outcome]) -> str | None:
+        return random.sample(list(offers.keys()), 1)[0]
+
+    def is_better(self, a: Outcome, b: Outcome, negotiator: str, state: SAOState):
+        return random.random() > 0.5
+
+
+class SAOSingleAgreementAspirationController(SAOSingleAgreementController):
     """
     A `SAOSingleAgreementController` that uses aspiration level to decide what
     to accept and what to propose.
 
     Args:
-        ufun: The utility function to use for ALL negotiations
+        preferences: The utility function to use for ALL negotiations
         max_aspiration: The maximum aspiration level to start with
         aspiration_type: The aspiration type/ exponent
 
@@ -804,47 +830,50 @@ class SAOSingleAgreementAspirationController(
         self,
         *args,
         max_aspiration: float = 1.0,
-        aspiration_type: Union[str, int, float] = "boulware",
+        aspiration_type: Literal["boulware", "linear", "conceder"]
+        | int
+        | float = "boulware",
         **kwargs,
     ):
         super().__init__(*args, **kwargs)
-        AspirationMixin.aspiration_init(self, max_aspiration, aspiration_type)
+        self.__asp = PolyAspiration(max_aspiration, aspiration_type)
 
-    def after_join(self, negotiator_id, ami, state, *, ufun=None, role="agent"):
-        if self.ufun is not None:
-            if (
-                self.ufun.outcome_type is not None
-                and self.ufun.outcome_type != ami.outcome_type
-            ):
-                raise ValueError(
-                    f"Controller's ufun is of type {self.ufun.outcome_type} while the mechanism ufun is of type"
-                    f"{ami.outcome_type}"
-                )
-            self.ufun.outcome_type = ami.outcome_type
+    def utility_at(self, x):
+        return self.__asp.utility_at(x)
 
-    def is_acceptable(self, offer: "Outcome", source: str, state: SAOState):
-        return self.ufun(offer) >= self.aspiration(state.relative_time)
+    def is_acceptable(self, offer: Outcome, source: str, state: SAOState):
+        if not self.ufun:
+            return False
+        return self.ufun(offer) >= self.utility_at(state.relative_time)
 
-    def is_better(self, a: "Outcome", b: "Outcome", negotiator: str, state: SAOState):
-        return self.ufun(a) > self.ufun(b)
+    def is_better(self, a: Outcome, b: Outcome, negotiator: str, state: SAOState):
+        if not self.ufun:
+            raise ValueError("No ufun is defined")
+        return self.ufun.is_better(a, b)
 
     def best_offer(self, offers):
-        best_val, best_negotiator = float("-inf"), None
+        if not self.ufun:
+            raise ValueError("No ufun is defined")
+        best_negotiator, best_offer = None, None
         for k, offer in offers.items():
-            curr_val = self.ufun(offer)
-            if curr_val >= best_val:
-                best_val, best_negotiator = curr_val, k
+            if best_negotiator is None:
+                best_negotiator, best_offer = k, offer
+                continue
+            if self.ufun.is_better(offer, best_offer):
+                best_negotiator, best_offer = k, offer
         return best_negotiator
 
     def best_outcome(self, negotiator, state=None):
-        outcome = self.ufun.outcome_with_utility(
+        if not self.ufun:
+            raise ValueError("No ufun is defined")
+        outcome = self.ufun.sample_outcome_with_utility(
             rng=(
-                self.aspiration(state.relative_time)
+                self.utility_at(state.relative_time)
                 if state is not None
-                else self.ufun(super().best_outcome(negotiator, None)),
-                None,
+                else float(self.ufun(super().best_outcome(negotiator, None))),
+                float("inf"),
             ),
-            issues=self.negotiators[negotiator][0].ami.issues,
+            issues=self.negotiators[negotiator][0].nmi.issues,
         )
         if outcome is None:
             return self._best_outcomes.get(negotiator, None)
