@@ -1,26 +1,18 @@
 """
 Defines a world for running negotiations directly
 """
+from __future__ import annotations
+
 import logging
 from collections import defaultdict
 from dataclasses import dataclass
-from typing import (
-    Any,
-    Callable,
-    Collection,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Set,
-    Type,
-    Union,
-)
+from typing import Any, Callable, Collection, Iterable
 
-from negmas import AgentMechanismInterface, MechanismState
+from negmas import MechanismState, NegotiatorMechanismInterface
 from negmas.helpers import get_class, get_full_type_name, instantiate
 from negmas.negotiators import Negotiator
 from negmas.outcomes import Issue
+from negmas.preferences import Preferences
 from negmas.serialization import deserialize, serialize, to_flat_dict
 from negmas.situated import (
     Action,
@@ -32,7 +24,6 @@ from negmas.situated import (
     RenegotiationRequest,
     World,
 )
-from negmas.utilities import UtilityFunction, utility_range
 
 __all__ = [
     "NegWorld",
@@ -41,6 +32,115 @@ __all__ = [
 ]
 
 
+class NegAgent(Agent):
+    """Wraps a negotiator for evaluaton"""
+
+    def __init__(
+        self,
+        *args,
+        negotiator_type: str | type[Negotiator],
+        negotiator_params: dict[str, Any] | None = None,
+        **kwargs,
+    ):
+        super().__init__(*args, **kwargs)
+        self._negotiator_params = negotiator_params if negotiator_params else dict()
+        self._negotiator_type = get_class(negotiator_type)
+
+    @property
+    def short_type_name(self):
+        """Returns a short name of the type of this entity"""
+        return self._negotiator_type.__name__.replace("Negotiator", "").replace(
+            "Agent", ""
+        )
+
+    @property
+    def type_name(self):
+        """Returns a short name of the type of this entity"""
+        return get_full_type_name(self._negotiator_type)
+
+    @classmethod
+    def _type_name(cls):
+        return cls.__module__ + "." + cls.__name__
+
+    def make_negotiator(self, preferences: Preferences | None = None):
+        """Makes a negotiator of the appropriate type passing it an optional ufun"""
+        return instantiate(
+            self._negotiator_type, preferences=preferences, **self._negotiator_params
+        )
+
+    def _respond_to_negotiation_request(
+        self,
+        initiator: str,
+        partners: list[str],
+        issues: list[Issue],
+        annotation: dict[str, Any],
+        mechanism: NegotiatorMechanismInterface,
+        role: str | None,
+        req_id: str | None,
+    ) -> Negotiator | None:
+        """Responds to any negotiation request by creating a negotiator"""
+        return self.make_negotiator(self.awi.get_preferences(partners.index(self.id)))
+
+    def step(self):
+        """Called by the simulator at every simulation step"""
+
+    def init(self):
+        """Called to initialize the agent **after** the world is initialized. the AWI is accessible at this point."""
+
+    def on_neg_request_rejected(self, req_id: str, by: list[str] | None):
+        """Called when a requested negotiation is rejected"""
+
+    def on_neg_request_accepted(
+        self, req_id: str, mechanism: NegotiatorMechanismInterface
+    ):
+        """Called when a requested negotiation is accepted"""
+
+    def on_negotiation_failure(
+        self,
+        partners: list[str],
+        annotation: dict[str, Any],
+        mechanism: NegotiatorMechanismInterface,
+        state: MechanismState,
+    ) -> None:
+        """Called whenever a negotiation ends without agreement"""
+
+    def on_negotiation_success(
+        self, contract: Contract, mechanism: NegotiatorMechanismInterface
+    ) -> None:
+        """Called whenever a negotiation ends with agreement"""
+
+    def on_contract_signed(self, contract: Contract) -> None:
+        """Called whenever a contract is signed by all partners"""
+
+    def on_contract_cancelled(self, contract: Contract, rejectors: list[str]) -> None:
+        """Called whenever at least a partner did not sign the contract"""
+
+    def respond_to_renegotiation_request(
+        self, contract: Contract, breaches: list[Breach], agenda: RenegotiationRequest
+    ) -> Negotiator | None:
+        """Called to respond to a renegotiation request"""
+
+    def on_contract_executed(self, contract: Contract) -> None:
+        """
+        Called after successful contract execution for which the agent is one of the partners.
+        """
+
+    def on_contract_breached(
+        self, contract: Contract, breaches: list[Breach], resolution: Contract | None
+    ) -> None:
+        """
+        Called after complete processing of a contract that involved a breach.
+        """
+
+    def set_renegotiation_agenda(
+        self, contract: Contract, breaches: list[Breach]
+    ) -> RenegotiationRequest | None:
+        """
+        Received by partners in ascending order of their total breach levels in order to set the
+        renegotiation agenda when contract execution fails
+        """
+
+
 def _unwrap_negotiators(types, params):
     """
     Removes the agent wrapping the negotiator for each type
@@ -103,19 +203,19 @@ class NegDomain:
 
     name: str
     """Domain name"""
-    issues: List[Issue]
+    issues: tuple[Issue, ...]
     """The issue space as a list of issues"""
-    ufuns: List[UtilityFunction]
+    ufuns: tuple[Preferences, ...]
     """The utility functions used by all negotiators in the domain"""
-    partner_types: List[Union[str, Negotiator]]
+    partner_types: tuple[str | Negotiator, ...]
     """The types of all partners (other than the agent being evaluated). Its length must be one less than `ufuns`"""
     index: int = 0
     """The index of the negotiator being evaluated in the list of negotiators passed to the mechanism"""
-    partner_params: Optional[List[Optional[Dict[str, Any]]]] = None
+    partner_params: tuple[dict[str, Any] | None, ...] | None = None
     """Any paramters used to construct partners (must be the same length as `partner_types`)"""
-    roles: Optional[List[str]] = None
+    roles: tuple[str, ...] | None = None
     """Roles of all negotiators (includng the negotiator being evaluated) in order"""
-    annotation: Optional[Dict[str, Any]] = None
+    annotation: dict[str, Any] | None = None
     """Any extra annotation to add to the mechanism."""
 
     def to_dict(self):
@@ -123,7 +223,7 @@ class NegDomain:
             name=self.name,
             issues=[i.to_dict() for i in self.issues],
             ufuns=[serialize(u) for u in self.ufuns],
-            partner_types=[get_full_type_name(_) for _ in self.partner_types],
+            partner_types=[get_full_type_name(_) for _ in self.partner_types],  # type: ignore
             index=self.index,
             partner_params=serialize(self.partner_params),
             roles=self.roles,
@@ -134,131 +234,24 @@ class NegDomain:
     def from_dict(cls, d):
         return cls(
             name=d["name"],
-            issues=[Issue.from_dict(_) for _ in d["issues"]],
-            ufuns=[deserialize(u) for u in d["ufuns"]],
-            partner_types=[get_class(_) for _ in d["partner_types"]],
+            issues=tuple(Issue.from_dict(_) for _ in d["issues"]),
+            ufuns=tuple(deserialize(u) for u in d["ufuns"]),  # type: ignore The type should be correct in the dict
+            partner_types=tuple(get_class(_) for _ in d["partner_types"]),
             index=d["index"],
-            partner_params=deserialize(d["partner_params"]),
+            partner_params=deserialize(d["partner_params"]),  # type: ignore The type should be correct in the dict
             roles=d["roles"],
-            annotation=deserialize(d["annotation"]),
+            annotation=deserialize(d["annotation"]),  # type: ignore The type should be correct in the dict
         )
 
 
-class NegAgent(Agent):
-    """Wraps a negotiator for evaluaton"""
-
-    def __init__(
-        self,
-        *args,
-        negotiator_type: Union[str, Type[Negotiator]],
-        negotiator_params: Optional[Dict[str, Any]] = None,
-        **kwargs,
-    ):
-        super().__init__(*args, **kwargs)
-        self._negotiator_params = negotiator_params if negotiator_params else dict()
-        self._negotiator_type = get_class(negotiator_type)
-
-    @property
-    def short_type_name(self):
-        """Returns a short name of the type of this entity"""
-        return self._negotiator_type.__name__.replace("Negotiator", "").replace(
-            "Agent", ""
-        )
-
-    @property
-    def type_name(self):
-        """Returns a short name of the type of this entity"""
-        return get_full_type_name(self._negotiator_type)
-
-    @classmethod
-    def _type_name(cls):
-        return cls.__module__ + "." + cls.__name__
-
-    def make_negotiator(self, ufun: Optional[UtilityFunction] = None):
-        """Makes a negotiator of the appropriate type passing it an optional ufun"""
-        return instantiate(self._negotiator_type, ufun=ufun, **self._negotiator_params)
-
-    def _respond_to_negotiation_request(
-        self,
-        initiator: str,
-        partners: List[str],
-        issues: List[Issue],
-        annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
-        role: Optional[str],
-        req_id: Optional[str],
-    ) -> Optional[Negotiator]:
-        """Responds to any negotiation request by creating a negotiator"""
-        return self.make_negotiator(self.awi.get_ufun(partners.index(self.id)))
-
-    def step(self):
-        """Called by the simulator at every simulation step"""
-
-    def init(self):
-        """Called to initialize the agent **after** the world is initialized. the AWI is accessible at this point."""
-
-    def on_neg_request_rejected(self, req_id: str, by: Optional[List[str]]):
-        """Called when a requested negotiation is rejected"""
-
-    def on_neg_request_accepted(self, req_id: str, mechanism: AgentMechanismInterface):
-        """Called when a requested negotiation is accepted"""
-
-    def on_negotiation_failure(
-        self,
-        partners: List[str],
-        annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
-        state: MechanismState,
-    ) -> None:
-        """Called whenever a negotiation ends without agreement"""
-
-    def on_negotiation_success(
-        self, contract: Contract, mechanism: AgentMechanismInterface
-    ) -> None:
-        """Called whenever a negotiation ends with agreement"""
-
-    def on_contract_signed(self, contract: Contract) -> None:
-        """Called whenever a contract is signed by all partners"""
-
-    def on_contract_cancelled(self, contract: Contract, rejectors: List[str]) -> None:
-        """Called whenever at least a partner did not sign the contract"""
-
-    def respond_to_renegotiation_request(
-        self, contract: Contract, breaches: List[Breach], agenda: RenegotiationRequest
-    ) -> Optional[Negotiator]:
-        """Called to respond to a renegotiation request"""
-
-    def on_contract_executed(self, contract: Contract) -> None:
-        """
-        Called after successful contract execution for which the agent is one of the partners.
-        """
-
-    def on_contract_breached(
-        self, contract: Contract, breaches: List[Breach], resolution: Optional[Contract]
-    ) -> None:
-        """
-        Called after complete processing of a contract that involved a breach.
-        """
-
-    def set_renegotiation_agenda(
-        self, contract: Contract, breaches: List[Breach]
-    ) -> Optional[RenegotiationRequest]:
-        """
-        Received by partners in ascending order of their total breach levels in order to set the
-        renegotiation agenda when contract execution fails
-        """
-
-
 class _NegPartner(NegAgent):
     """A `NegAgent` representing a partner that is not being evaluated"""
 
-    pass
-
 
 class _NegAWI(AgentWorldInterface):
     """The AWI for the `NegWorld`"""
 
-    def get_ufun(self, uid: int):
+    def get_preferences(self, uid: int):
         """Get the agent's ufun"""
         return self._world._domain.ufuns[uid]
 
@@ -283,8 +276,8 @@ class NegWorld(NoContractExecutionMixin, World):
         self,
         *args,
         domain: NegDomain,
-        types: List[Union[Negotiator, NegAgent]],
-        params: Optional[List[Optional[Dict[str, Any]]]] = None,
+        types: list[Negotiator | NegAgent],
+        params: list[dict[str, Any] | None] | None = None,
         agent_names_reveal_type: bool = True,
         compact: bool = False,
         no_logs: bool = False,
@@ -318,19 +311,22 @@ class NegWorld(NoContractExecutionMixin, World):
         if domain.annotation is None:
             domain.annotation = dict()
         self._domain = domain
-        self._received_utility: Dict[str, List[float]] = defaultdict(list)
-        self._partner_utility: Dict[str, List[float]] = defaultdict(list)
-        self._success: Dict[str, bool] = defaultdict(bool)
-        self._received_advantage: Dict[str, List[float]] = defaultdict(list)
-        self._partner_advantage: Dict[str, List[float]] = defaultdict(list)
-        self._competitors: Dict[str, NegAgent] = dict()
-        self._partners: Dict[str, NegAgent] = dict()
+        self._received_utility: dict[str, list[float]] = defaultdict(list)
+        self._partner_utility: dict[str, list[float]] = defaultdict(list)
+        self._success: dict[str, bool] = defaultdict(bool)
+        self._received_advantage: dict[str, list[float]] = defaultdict(list)
+        self._n_agreements_per_cometitor: dict[str, list[int]] = defaultdict(list)
+        self._partner_advantage: dict[str, list[float]] = defaultdict(list)
+        self._competitors: dict[str, NegAgent] = dict()
+        self._partners: dict[str, NegAgent] = dict()
         self._n_negs_per_copmetitor = defaultdict(int)
         self._normalize_scores = normalize_scores
-        self._ufun_ranges = None
+        self._preferences_ranges = None
+        # if not all(isinstance(_, HasRange) for _ in domain.ufuns):
+        #     raise ValueError(f"Not all ufuns has a range!!!")
         if self._normalize_scores:
-            self._ufun_ranges = [
-                utility_range(u, issues=domain.issues) for u in domain.ufuns
+            self._preferences_ranges = [
+                u.minmax() for u in domain.ufuns  # type: ignore We already check just above
             ]
         partner_types = domain.partner_types
         partner_params = domain.partner_params
@@ -372,7 +368,7 @@ class NegWorld(NoContractExecutionMixin, World):
     def simulation_step(self, stage):
         def unormalize(u, indx):
             if self._normalize_scores:
-                mn, mx = self._ufun_ranges[indx]
+                mn, mx = self._preferences_ranges[indx]
                 if mx > mn:
                     u = (u - mn) / (mx - mn)
                 else:
@@ -406,9 +402,10 @@ class NegWorld(NoContractExecutionMixin, World):
             )
             r = unormalize(float(ufun.reserved_value), self._domain.index)
             self._received_utility[aid].append(u)
+            self._n_agreements_per_cometitor[aid].append(int(mechanism is not None))
             self._received_advantage[aid].append(u - r)
             pufuns = [
-                (partners.index(pid), p.awi.get_ufun(partners.index(pid)))
+                (partners.index(pid), p.awi.get_preferences(partners.index(pid)))
                 for pid, p in self._partners.items()
             ]
             pu = sum(unormalize(float(_(agreement)), i) for i, _ in pufuns)
@@ -427,6 +424,11 @@ class NegWorld(NoContractExecutionMixin, World):
             self._received_utility.get(aid, [0])
         ) / self._n_negs_per_copmetitor.get(aid, 0)
 
+    def agreement_rate(self, aid: str):
+        return sum(
+            self._n_agreements_per_cometitor.get(aid, [0])
+        ) / self._n_negs_per_agent.get(aid, 0)
+
     def partner_utility(self, aid: str):
         return sum(
             self._partner_utility.get(aid, [0])
@@ -460,6 +462,9 @@ class NegWorld(NoContractExecutionMixin, World):
             self._stats[f"received_advantage_{aid}"].append(
                 self._received_advantage[aid][-1]
             )
+            self._stats[f"agrement_rate_{aid}"].append(
+                self._n_agreements_per_cometitor[aid][-1]
+            )
             self._stats[f"partner_advantage_{aid}"].append(
                 self._partner_advantage[aid][-1]
             )
@@ -472,10 +477,10 @@ class NegWorld(NoContractExecutionMixin, World):
     ) -> Collection[Contract]:
         return contracts
 
-    def contract_record(self, contract: Contract) -> Dict[str, Any]:
+    def contract_record(self, contract: Contract) -> dict[str, Any]:
         return to_flat_dict(contract, deep=True)
 
-    def breach_record(self, breach: Breach) -> Dict[str, Any]:
+    def breach_record(self, breach: Breach) -> dict[str, Any]:
         return to_flat_dict(breach, deep=True)
 
     def contract_size(self, contract: Contract) -> float:
@@ -486,21 +491,23 @@ class NegWorld(NoContractExecutionMixin, World):
         pass
 
     def execute_action(
-        self, action: Action, agent: "Agent", callback: Callable = None
+        self, action: Action, agent: Agent, callback: Callable | None = None
     ) -> bool:
         """Executes the given action by the given agent"""
+        ...
 
-    def get_private_state(self, agent: "Agent") -> dict:
+    def get_private_state(self, agent: Agent) -> dict:
         """Reads the private state of the given agent"""
+        ...
 
     def executable_contracts(self) -> Collection[Contract]:
         return []
 
-    def start_contract_execution(self, contract: Contract) -> Set[Breach]:
+    def start_contract_execution(self, contract: Contract) -> set[Breach]:
         return set()
 
     def complete_contract_execution(
-        self, contract: Contract, breaches: List[Breach], resolution: Contract
+        self, contract: Contract, breaches: list[Breach], resolution: Contract
     ) -> None:
         pass
 
@@ -508,9 +515,10 @@ class NegWorld(NoContractExecutionMixin, World):
 if __name__ == "__main__":
     from negmas.genius import genius_bridge_is_running
     from negmas.genius.gnegotiators import Atlas3, NiceTitForTat
+    from negmas.outcomes import Issue
+    from negmas.preferences import LinearAdditiveUtilityFunction as U
     from negmas.sao import AspirationNegotiator, NaiveTitForTatNegotiator
     from negmas.situated import save_stats
-    from negmas.utilities import LinearUtilityAggregationFunction as U
 
     issues = [Issue(10, "quantity"), Issue(5, "price")]
     competitors = [AspirationNegotiator, NaiveTitForTatNegotiator]
