@@ -1,10 +1,14 @@
-"""Implements single text negotiation mechanisms"""
+"""
+Implements single text negotiation mechanisms.
+"""
+from __future__ import annotations
+
 import math
 import random
 import time
 from copy import deepcopy
-from dataclasses import dataclass
-from typing import List, Optional, Tuple, Union
+
+from attr import define
 
 from .mechanisms import Mechanism, MechanismRoundResult, MechanismState
 from .outcomes import Outcome
@@ -12,12 +16,12 @@ from .outcomes import Outcome
 __all__ = ["VetoSTMechanism", "HillClimbingSTMechanism"]
 
 
-@dataclass
+@define
 class STState(MechanismState):
     """Defines extra values to keep in the mechanism state. This is accessible to all negotiators"""
 
-    current_offer: Optional["Outcome"] = None
-    new_offer: Optional["Outcome"] = None
+    current_offer: Outcome | None = None
+    new_offer: Outcome | None = None
 
 
 class VetoSTMechanism(Mechanism):
@@ -44,7 +48,7 @@ class VetoSTMechanism(Mechanism):
         *args,
         epsilon: float = 1e-6,
         initial_outcome=None,
-        initial_responses: Tuple[bool] = tuple(),
+        initial_responses: tuple[bool] = tuple(),
         **kwargs,
     ):
         kwargs["state_factory"] = STState
@@ -53,7 +57,7 @@ class VetoSTMechanism(Mechanism):
         self.add_requirements(
             {"compare-binary": True}
         )  # assert that all agents must have compare-binary capability
-        self.current_offer = initial_outcome
+        self._current_state.current_offer = initial_outcome
         """The current offer"""
         self.initial_outcome = deepcopy(initial_outcome)
         """The initial offer"""
@@ -62,16 +66,10 @@ class VetoSTMechanism(Mechanism):
         self.initial_responses = deepcopy(self.last_responses)
         """The initial set of responses. See the remarks of this class to understand its role."""
         self.epsilon = epsilon
-        self.new_offer = initial_outcome
+        self._current_state.new_offer = initial_outcome
         """The new offer generated in this step"""
 
-    def extra_state(self):
-        return dict(
-            current_offer=deepcopy(self.current_offer),
-            new_offer=deepcopy(self.new_offer),
-        )
-
-    def next_outcome(self, outcome: Optional["Outcome"]) -> Optional["Outcome"]:
+    def next_outcome(self, outcome: Outcome | None) -> Outcome | None:
         """Generate the next outcome given some outcome.
 
         Args:
@@ -85,38 +83,37 @@ class VetoSTMechanism(Mechanism):
 
     def round(self) -> MechanismRoundResult:
         """Single round of the protocol"""
-        new_offer = self.next_outcome(self.current_offer)
+        new_offer = self.next_outcome(self._current_state.current_offer)
         responses = []
 
         for neg in self.negotiators:
             strt = time.perf_counter()
             responses.append(
-                neg.is_better(new_offer, self.current_offer, epsilon=self.epsilon)
-                is not False
+                neg.is_better(new_offer, self._current_state.current_offer) is not False
             )
-            if time.perf_counter() - strt > self.ami.step_time_limit:
+            if time.perf_counter() - strt > self.nmi.step_time_limit:
                 return MechanismRoundResult(broken=False, timedout=True, agreement=None)
 
         self.last_responses = responses
-        self.new_offer = new_offer
+        self._current_state.new_offer = new_offer
         if all(responses):
-            self.current_offer = new_offer
+            self._current_state.current_offer = new_offer
 
         return MechanismRoundResult(broken=False, timedout=False, agreement=None)
 
     def on_negotiation_end(self) -> None:
         """Used to pass the final offer for agreement between all negotiators"""
-        if self.current_offer is not None and all(
-            neg.is_acceptable_as_agreement(self.current_offer)
+        if self._current_state.current_offer is not None and all(
+            neg.is_acceptable_as_agreement(self._current_state.current_offer)
             for neg in self.negotiators
         ):
-            self._agreement = self.current_offer
+            self._current_state.agreement = self._current_state.current_offer
 
         super().on_negotiation_end()
 
     def plot(
         self,
-        visible_negotiators: Union[Tuple[int, int], Tuple[str, str]] = (0, 1),
+        visible_negotiators: tuple[int, int] | tuple[str, str] = (0, 1),
         show_all_offers=False,
         **kwargs,
     ):
@@ -150,8 +147,8 @@ class VetoSTMechanism(Mechanism):
                     "current_offer": offer,
                     "relative_time": state.relative_time,
                     "step": state.step,
-                    "u0": visible_negotiators[0].utility_function(offer),
-                    "u1": visible_negotiators[1].utility_function(offer),
+                    "u0": visible_negotiators[0].ufun(offer),
+                    "u1": visible_negotiators[1].ufun(offer),
                 }
             )
         history = pd.DataFrame(data=history)
@@ -159,7 +156,7 @@ class VetoSTMechanism(Mechanism):
         has_front = 1
         # n_negotiators = len(self.negotiators)
         n_agents = len(visible_negotiators)
-        ufuns = self._get_ufuns()
+        ufuns = self._get_preferencess()
         outcomes = self.outcomes
         utils = [tuple(f(o) for f in ufuns) for o in outcomes]
         agent_names = [a.name for a in visible_negotiators]
@@ -259,6 +256,10 @@ class VetoSTMechanism(Mechanism):
 
         fig_util.show()
 
+    @property
+    def current_offer(self):
+        return self.state.current_offer
+
 
 class HillClimbingSTMechanism(VetoSTMechanism):
     """A single text mechanism that use hill climbing
@@ -268,8 +269,37 @@ class HillClimbingSTMechanism(VetoSTMechanism):
         **kwargs: keyword arguments to be passed to the base Mechanism
     """
 
+    def neighbors(self, outcome: Outcome) -> list[Outcome]:
+        """Returns all neighbors
+
+        Neighbor is an outcome that differs any one of the issues from the original outcome.
+        """
+
+        neighbors = []
+        for i, issue in enumerate(self.issues):
+            values = []
+            if isinstance(issue.values, list):
+                values = issue.values
+            if isinstance(issue.values, int):
+                values = [
+                    max(0, outcome[i] - 1),
+                    min(outcome[i] + 1, issue.values),
+                ]
+            if isinstance(issue.values, tuple):
+                delta = random.random() * (issue.values[0] - issue.values[0])
+                values.append(max(issue.values[0], outcome[i] - delta))
+                values.append(min(outcome[i] + delta, issue.values[0]))
+
+            for value in values:
+                neighbor = list(deepcopy(outcome))
+                if neighbor[i] == value:
+                    continue
+                neighbor[i] = value
+                neighbors.append(tuple(neighbor))
+
+        return neighbors
+
     def __init__(self, *args, **kwargs):
-        kwargs["outcome_type"] = dict
         super().__init__(*args, **kwargs)
 
         for issue in self.issues:
@@ -279,10 +309,10 @@ class HillClimbingSTMechanism(VetoSTMechanism):
         if self.initial_outcome is None:
             self.initial_outcome = self.random_outcomes(1)[0]
 
-        self.current_offer = self.initial_outcome
-        self.possible_offers = self.neighbors(self.current_offer)
+        self._current_state.current_offer = self.initial_outcome
+        self.possible_offers = self.neighbors(self._current_state.current_offer)
 
-    def next_outcome(self, outcome: "Outcome") -> Optional["Outcome"]:
+    def next_outcome(self, outcome: Outcome) -> Outcome | None:
         """Generate the next outcome given some outcome.
 
         Args:
@@ -302,56 +332,31 @@ class HillClimbingSTMechanism(VetoSTMechanism):
     def round(self) -> MechanismRoundResult:
         """Single round of the protocol"""
 
-        new_offer = self.next_outcome(self.current_offer)
+        new_offer = self.next_outcome(self._current_state.current_offer)
         if new_offer is None:
             return MechanismRoundResult(
-                broken=False, timedout=False, agreement=self.current_offer
+                broken=False,
+                timedout=False,
+                agreement=self._current_state.current_offer,
             )
 
         responses = []
         for neg in self.negotiators:
             strt = time.perf_counter()
             responses.append(
-                neg.is_better(new_offer, self.current_offer, epsilon=self.epsilon)
-                is not False
+                neg.is_better(new_offer, self._current_state.current_offer) is not False
             )
-            if time.perf_counter() - strt > self.ami.step_time_limit:
+            if time.perf_counter() - strt > self.nmi.step_time_limit:
                 return MechanismRoundResult(broken=False, timedout=True, agreement=None)
 
         self.last_responses = responses
 
         if all(responses):
-            self.current_offer = new_offer
-            self.possible_offers = self.neighbors(self.current_offer)
+            self._current_state.current_offer = new_offer
+            self.possible_offers = self.neighbors(self._current_state.current_offer)
 
         return MechanismRoundResult(broken=False, timedout=False, agreement=None)
 
-    def neighbors(self, outcome: "Outcome") -> List["Outcome"]:
-        """Returns all neighbors
-
-        Neighbor is an outcome that differs any one of the issues from the original outcome.
-        """
-
-        neighbors = []
-        for issue in self.issues:
-            values = []
-            if isinstance(issue.values, List):
-                values = issue.values
-            if isinstance(issue.values, int):
-                values = [
-                    max(0, outcome[issue.name] - 1),
-                    min(outcome[issue.name] + 1, issue.values),
-                ]
-            if isinstance(issue.values, Tuple):
-                delta = random.random(issue.values[0] - issue.values[0])
-                values.append(max(issue.values[0], outcome[issue.name] - delta))
-                values.append(min(outcome[issue.name] + delta, issue.values[0]))
-
-            for value in values:
-                neighbor = deepcopy(outcome)
-                if neighbor[issue.name] == value:
-                    continue
-                neighbor[issue.name] = value
-                neighbors.append(neighbor)
-
-        return neighbors
+    @property
+    def current_offer(self):
+        return self.state.current_offer
