@@ -1,36 +1,43 @@
+from __future__ import annotations
+
 from pytest import fixture
 
-from negmas.outcomes import Issue
+from negmas.outcomes import generate_issues, make_issue
 
 
 @fixture()
 def bissue():
-    return Issue(["be", "not b"], name="binary")
+    return make_issue(["be", "not b"], name="binary")
 
 
 @fixture()
 def hamlet():
-    return Issue([f"val {_}" for _ in range(5)], name="THE problem")
+    return make_issue([f"val {_}" for _ in range(5)], name="THE problem")
 
 
 @fixture()
 def uissue():
-    return Issue([f"val {_}" for _ in range(5)])
+    return make_issue([f"val {_}" for _ in range(5)])
 
 
 @fixture()
 def cissue():
-    return Issue((0.0, 1.0), name="c")
+    return make_issue((0.0, 1.0), name="c")
 
 
 @fixture()
 def dissue():
-    return Issue(10, name="d")
+    return make_issue(10, name="d")
 
 
 @fixture()
 def sissue():
-    return Issue([f"val {_}" for _ in range(5)], name="s")
+    return make_issue([f"val {_}" for _ in range(5)], name="s")
+
+
+@fixture()
+def int_issues():
+    return generate_issues([5], [10])
 
 
 @fixture()
@@ -38,11 +45,6 @@ def issues(cissue, dissue, sissue):
     return [cissue, dissue, sissue]
 
 
-@fixture()
-def int_issues():
-    return Issue.generate([5], [10])
-
-
 @fixture()
 def valid_outcome_dict(issues):
     outcome = {}
