@@ -1,25 +1,29 @@
+from __future__ import annotations
+
 import pathlib
+from math import isclose
 
 import numpy as np
 import pkg_resources
 import pytest
+from numpy.testing import assert_almost_equal
 
 from negmas import (
     GeniusNegotiator,
     genius_bridge_is_running,
-    load_genius_domain,
     load_genius_domain_from_folder,
 )
 from negmas.genius import AgentX, Atlas3, Caduceus, GeniusBridge, YXAgent
+from negmas.genius.gnegotiators import AgentK
 
-dom_folder = pathlib.Path(
+DOMAIN_FOLDER = pathlib.Path(
     pkg_resources.resource_filename(
         "negmas", resource_name="tests/data/scenarios/anac/y2010/Travel"
     )
 )
-dom = dom_folder / "travel_domain.xml"
-util1 = dom_folder / "travel_chox.xml"
-util2 = dom_folder / "travel_fanny.xml"
+DOMAIN_FILE = DOMAIN_FOLDER / "travel_domain.xml"
+UTIL1 = DOMAIN_FOLDER / "travel_chox.xml"
+UTIL2 = DOMAIN_FOLDER / "travel_fanny.xml"
 
 
 SKIP_IF_NO_BRIDGE = True
@@ -35,25 +39,72 @@ def init_genius():
     reason="No Genius Bridge, skipping genius-agent tests",
 )
 def test_genius_agent(init_genius):
-    p, _, issues = load_genius_domain_from_folder(
-        dom_folder,
-        agent_factories=[
-            lambda: GeniusNegotiator(
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    p = domain.make_session(
+        [
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2015.Atlas3.Atlas3",
-                domain_file_name=dom,
-                utility_file_name=util1,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL1,
             ),
-            lambda: GeniusNegotiator(
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2015.AgentX.AgentX",
-                domain_file_name=dom,
-                utility_file_name=util2,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL2,
             ),
         ],
-        keep_issue_names=True,
-        keep_value_names=True,
         time_limit=120,
         n_steps=None,
     )
+    issues = domain.issues
+    assert len(p.negotiators) > 1
+    issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
+    assert issue_list == [
+        "0:Atmosphere: ['Cultural heritage', 'Local traditions', 'Political stability', 'Security (personal)'"
+        ", 'Liveliness', 'Turistic activities', 'Hospitality']",
+        "1:Amusement: ['Nightlife and entertainment', 'Nightclubs', 'Excursion', 'Casinos', 'Zoo', 'Festivals'"
+        ", 'Amusement park']",
+        "2:Culinary: ['Local cuisine', 'Lunch facilities', 'International cuisine', 'Cooking workshops']",
+        "3:Shopping: ['Shopping malls', 'Markets', 'Streets', 'Small boutiques']",
+        "4:Culture: ['Museum', 'Music hall', 'Theater', 'Art gallery', 'Cinema', 'Congress center']",
+        "5:Sport: ['Bike tours', 'Hiking', 'Indoor activities', 'Outdoor activities', 'Adventure']",
+        "6:Environment: ['Parks and Gardens', 'Square', 'Historical places', 'See, river, etc.', 'Monuments'"
+        ", 'Special streets', 'Palace', 'Landscape and nature']",
+    ]
+    first, second = p.negotiators[0], p.negotiators[1]
+    p.run()
+    # print(f'{len(p.history)} bids exchanged')
+    u1 = np.array([float(first.ufun(s.current_offer)) for s in p.history])
+    u2 = np.array([float(second.ufun(s.current_offer)) for s in p.history])
+    u1 + u2
+    # print(
+    #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
+    #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
+    # print(p.state)
+    assert len(u1) > 0
+
+
+@pytest.mark.skipif(
+    condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
+    reason="No Genius Bridge, skipping genius-agent tests",
+)
+def test_genius_agent_top2016_caduceus_first(init_genius):
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    issues = domain.issues
+    p = domain.make_session(
+        [
+            Caduceus(
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL1,
+            ),
+            AgentX(
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL2,
+            ),
+        ],
+        time_limit=20,
+        n_steps=None,
+    )
     assert len(p.negotiators) > 1
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
     assert issue_list == [
@@ -71,57 +122,9 @@ def test_genius_agent(init_genius):
     first, second = p.negotiators[0], p.negotiators[1]
     p.run()
     # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(first.utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(second.utility_function(s.current_offer)) for s in p.history])
-    welfare = u1 + u2
-    # print(
-    #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
-    #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
-    # print(p.state)
-    assert len(u1) > 0
-
-
-@pytest.mark.skipif(
-    condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
-    reason="No Genius Bridge, skipping genius-agent tests",
-)
-def test_genius_agent_top2016_caduceus_first(init_genius):
-    p, _, issues = load_genius_domain_from_folder(
-        dom_folder,
-        agent_factories=[
-            lambda: Caduceus(
-                domain_file_name=dom,
-                utility_file_name=util1,
-            ),
-            lambda: AgentX(
-                domain_file_name=dom,
-                utility_file_name=util2,
-            ),
-        ],
-        keep_issue_names=True,
-        keep_value_names=True,
-        time_limit=20,
-    )
-    assert len(p.negotiators) > 1
-    issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
-    assert issue_list == [
-        "0:Atmosphere: ['Cultural heritage', 'Local traditions', 'Political stability', 'Security (personal)'"
-        ", 'Liveliness', 'Turistic activities', 'Hospitality']",
-        "1:Amusement: ['Nightlife and entertainment', 'Nightclubs', 'Excursion', 'Casinos', 'Zoo', 'Festivals'"
-        ", 'Amusement park']",
-        "2:Culinary: ['Local cuisine', 'Lunch facilities', 'International cuisine', 'Cooking workshops']",
-        "3:Shopping: ['Shopping malls', 'Markets', 'Streets', 'Small boutiques']",
-        "4:Culture: ['Museum', 'Music hall', 'Theater', 'Art gallery', 'Cinema', 'Congress center']",
-        "5:Sport: ['Bike tours', 'Hiking', 'Indoor activities', 'Outdoor activities', 'Adventure']",
-        "6:Environment: ['Parks and Gardens', 'Square', 'Historical places', 'See, river, etc.', 'Monuments'"
-        ", 'Special streets', 'Palace', 'Landscape and nature']",
-    ]
-    first, second = p.negotiators[0], p.negotiators[1]
-    p.run()
-    # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(first.utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(second.utility_function(s.current_offer)) for s in p.history])
-    welfare = u1 + u2
+    u1 = np.array([float(first.ufun(s.current_offer)) for s in p.history])
+    u2 = np.array([float(second.ufun(s.current_offer)) for s in p.history])
+    u1 + u2
     # print(
     #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
     #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
@@ -130,15 +133,15 @@ def test_genius_agent_top2016_caduceus_first(init_genius):
 
 
 def do_run(first_type, second_type):
-    p, _, issues = load_genius_domain_from_folder(
-        dom_folder,
-        agent_factories=[
-            lambda: first_type(domain_file_name=dom, utility_file_name=util1),
-            lambda: second_type(domain_file_name=dom, utility_file_name=util2),
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    issues = domain.issues
+    p = domain.make_session(
+        [
+            first_type(domain_file_name=DOMAIN_FILE, utility_file_name=UTIL1),
+            second_type(domain_file_name=DOMAIN_FILE, utility_file_name=UTIL2),
         ],
-        keep_issue_names=True,
-        keep_value_names=True,
         time_limit=20,
+        n_steps=None,
     )
     assert len(p.negotiators) > 1
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
@@ -157,8 +160,8 @@ def do_run(first_type, second_type):
     first, second = p.negotiators[0], p.negotiators[1]
     p.run()
     # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(first.utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(second.utility_function(s.current_offer)) for s in p.history])
+    u1 = np.array([float(first.ufun(s.current_offer)) for s in p.history])
+    u2 = np.array([float(second.ufun(s.current_offer)) for s in p.history])
     # welfare = u1 + u2
     # print(
     #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
@@ -181,23 +184,23 @@ def test_genius_agent_top2016_yx_second_classes(init_genius, first_type, second_
     reason="No Genius Bridge, skipping genius-agent tests",
 )
 def test_genius_agent_top2016_yx_second(init_genius):
-    p, _, issues = load_genius_domain_from_folder(
-        dom_folder,
-        agent_factories=[
-            lambda: GeniusNegotiator(
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    issues = domain.issues
+    p = domain.make_session(
+        [
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2016.yxagent.YXAgent",
-                domain_file_name=dom,
-                utility_file_name=util1,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL1,
             ),
-            lambda: GeniusNegotiator(
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2015.AgentX.AgentX",
-                domain_file_name=dom,
-                utility_file_name=util2,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL2,
             ),
         ],
-        keep_issue_names=True,
-        keep_value_names=True,
         time_limit=20,
+        n_steps=None,
     )
     assert len(p.negotiators) > 1
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
@@ -216,9 +219,9 @@ def test_genius_agent_top2016_yx_second(init_genius):
     first, second = p.negotiators[0], p.negotiators[1]
     p.run()
     # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(first.utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(second.utility_function(s.current_offer)) for s in p.history])
-    welfare = u1 + u2
+    u1 = np.array([float(first.ufun(s.current_offer)) for s in p.history])
+    u2 = np.array([float(second.ufun(s.current_offer)) for s in p.history])
+    u1 + u2
     # print(
     #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
     #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
@@ -231,24 +234,23 @@ def test_genius_agent_top2016_yx_second(init_genius):
     reason="No Genius Bridge, skipping genius-agent tests",
 )
 def test_genius_agent_step_limit(init_genius):
-    p, _, issues = load_genius_domain_from_folder(
-        dom_folder,
-        agent_factories=[
-            lambda: GeniusNegotiator(
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    issues = domain.issues
+    p = domain.make_session(
+        [
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2015.Atlas3.Atlas3",
-                domain_file_name=dom,
-                utility_file_name=util1,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL1,
             ),
-            lambda: GeniusNegotiator(
+            GeniusNegotiator(
                 java_class_name="agents.anac.y2015.AgentX.AgentX",
-                domain_file_name=dom,
-                utility_file_name=util2,
+                domain_file_name=DOMAIN_FILE,
+                utility_file_name=UTIL2,
             ),
         ],
-        keep_issue_names=True,
-        keep_value_names=True,
-        n_steps=20,
         time_limit=None,
+        n_steps=20,
     )
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
     assert issue_list == [
@@ -263,9 +265,9 @@ def test_genius_agent_step_limit(init_genius):
     atlas3, agentx = p.negotiators[0], p.negotiators[1]
     p.run()
     # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(atlas3.utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(agentx.utility_function(s.current_offer)) for s in p.history])
-    welfare = u1 + u2
+    u1 = np.array([float(atlas3.ufun(s.current_offer)) for s in p.history])
+    u2 = np.array([float(agentx.ufun(s.current_offer)) for s in p.history])
+    u1 + u2
     # print(
     #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
     #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
@@ -279,22 +281,17 @@ def test_genius_agent_step_limit(init_genius):
 def test_genius_agent_step_long_session(init_genius):
     a1 = GeniusNegotiator(
         java_class_name="agents.anac.y2015.Atlas3.Atlas3",
-        domain_file_name=dom,
-        utility_file_name=util1,
+        domain_file_name=DOMAIN_FILE,
+        utility_file_name=UTIL1,
     )
     a2 = GeniusNegotiator(
         java_class_name="agents.anac.y2015.Atlas3.Atlas3",
-        domain_file_name=dom,
-        utility_file_name=util2,
-    )
-    p, _, issues = load_genius_domain(
-        dom,
-        keep_issue_names=True,
-        keep_value_names=True,
-        n_steps=20,
-        time_limit=None,
-        normalize_utilities=True,
+        domain_file_name=DOMAIN_FILE,
+        utility_file_name=UTIL2,
     )
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER).normalize()
+    issues = domain.issues
+    p = domain.make_session([a1, a2], time_limit=None, n_steps=20)
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
     assert issue_list == [
         "0:Atmosphere: ['Cultural heritage', 'Local traditions', 'Political stability', 'Security (personal)', 'Liveliness', 'Turistic activities', 'Hospitality']",
@@ -309,9 +306,9 @@ def test_genius_agent_step_long_session(init_genius):
     p.add(a2)
     p.run()
     # print(f'{len(p.history)} bids exchanged')
-    u1 = np.array([float(a1._utility_function(s.current_offer)) for s in p.history])
-    u2 = np.array([float(a2._utility_function(s.current_offer)) for s in p.history])
-    welfare = u1 + u2
+    u1 = np.array([float(a1._preferences(s.current_offer)) for s in p.history])
+    u2 = np.array([float(a2._preferences(s.current_offer)) for s in p.history])
+    u1 + u2
     # print(
     #     f'Negotiator 1: {u1.mean()}({u1.std()})[{u1.min()}, {u1.max()}]\nNegotiator 2: {u2.mean()}({u2.std()})[{u1.min()}, {u1.max()}]'
     #     f'\nWelfare: {welfare.mean()}({welfare.std()})[{welfare.min()}, {welfare.max()}]')
@@ -323,16 +320,12 @@ def test_genius_agent_step_long_session(init_genius):
     condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
     reason="No Genius Bridge, skipping genius-agent tests",
 )
-def test_genius_agent_same_utility():
+def test_genius_agent_same_utility_with_normalization():
     from negmas import load_genius_domain_from_folder
 
-    p, ufuns, issues = load_genius_domain_from_folder(
-        dom_folder,
-        keep_issue_names=True,
-        keep_value_names=True,
-        time_limit=300000,
-        normalize_utilities=True,
-    )
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER).normalize()
+    issues = domain.issues
+    p = domain.make_session(time_limit=300, n_steps=None)
     assert p is not None, "Could not create a mechanism"
     issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
     assert issue_list == [
@@ -347,13 +340,52 @@ def test_genius_agent_same_utility():
         "6:Environment: ['Parks and Gardens', 'Square', 'Historical places', 'See, river, etc.'"
         ", 'Monuments', 'Special streets', 'Palace', 'Landscape and nature']",
     ]
-    a1 = AgentX(ufun=ufuns[0]["ufun"])
-    a2 = Atlas3(ufun=ufuns[0]["ufun"])
+    a1 = AgentK(preferences=domain.ufuns[0])
+    a2 = Atlas3(preferences=domain.ufuns[0])
     p.add(a1)
     p.add(a2)
     final = p.run()
-    u1 = [float(a1.utility_function(o)) for o in p.offers]
-    u2 = [float(a2.utility_function(o)) for o in p.offers]
+    u1 = [float(a1.preferences(o)) for o in p.offers]
+    u2 = [float(a2.preferences(o)) for o in p.offers]
+    assert len(u1) >= 1 or len(u2) >= 1
+    u1, u2 = a1.ufun(final.agreement), a2.ufun(final.agreement)
+    welfare = u1 + u2
+    assert p.state.agreement is not None
+    assert p.state.broken is False
+    assert welfare > 1
+
+
+@pytest.mark.skipif(
+    condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
+    reason="No Genius Bridge, skipping genius-agent tests",
+)
+def test_genius_agent_same_utility():
+    from negmas import load_genius_domain_from_folder
+
+    domain = load_genius_domain_from_folder(DOMAIN_FOLDER)
+    issues = domain.issues
+    p = domain.make_session(time_limit=300, n_steps=None)
+    assert p is not None, "Could not create a mechanism"
+    issue_list = [f"{k}:{v}" for k, v in enumerate(issues)]
+    assert issue_list == [
+        "0:Atmosphere: ['Cultural heritage', 'Local traditions', 'Political stability', 'Security (personal)'"
+        ", 'Liveliness', 'Turistic activities', 'Hospitality']",
+        "1:Amusement: ['Nightlife and entertainment', 'Nightclubs', 'Excursion', 'Casinos', 'Zoo'"
+        ", 'Festivals', 'Amusement park']",
+        "2:Culinary: ['Local cuisine', 'Lunch facilities', 'International cuisine', 'Cooking workshops']",
+        "3:Shopping: ['Shopping malls', 'Markets', 'Streets', 'Small boutiques']",
+        "4:Culture: ['Museum', 'Music hall', 'Theater', 'Art gallery', 'Cinema', 'Congress center']",
+        "5:Sport: ['Bike tours', 'Hiking', 'Indoor activities', 'Outdoor activities', 'Adventure']",
+        "6:Environment: ['Parks and Gardens', 'Square', 'Historical places', 'See, river, etc.'"
+        ", 'Monuments', 'Special streets', 'Palace', 'Landscape and nature']",
+    ]
+    a1 = AgentK(preferences=domain.ufuns[0])
+    a2 = Atlas3(preferences=domain.ufuns[0])
+    p.add(a1)
+    p.add(a2)
+    final = p.run()
+    u1 = [float(a1.preferences(o)) for o in p.offers]
+    u2 = [float(a2.preferences(o)) for o in p.offers]
     assert len(u1) >= 1 or len(u2) >= 1
     u1, u2 = a1.ufun(final.agreement), a2.ufun(final.agreement)
     welfare = u1 + u2
@@ -367,47 +399,25 @@ class TestGeniusAgentSessions:
         self,
         utils=(0, 0),
         single_issue=True,
-        keep_issue_names=True,
-        keep_value_names=True,
     ):
-        from negmas import convert_genius_domain_from_folder
 
-        src = pkg_resources.resource_filename(
+        base_folder = pkg_resources.resource_filename(
             "negmas", resource_name="tests/data/Laptop"
         )
-        dst = pkg_resources.resource_filename(
-            "negmas", resource_name="tests/data/LaptopConv1D"
-        )
+
+        domain = load_genius_domain_from_folder(base_folder).normalize()
         if single_issue:
-            assert convert_genius_domain_from_folder(
-                src_folder_name=src,
-                dst_folder_name=dst,
-                force_single_issue=True,
-                cache_and_discretize_outcomes=True,
-                n_discretization=10,
-                keep_issue_names=keep_issue_names,
-                keep_value_names=keep_value_names,
-                normalize_utilities=True,
-            )
-            base_folder = dst
-        else:
-            base_folder = src
-        neg, agent_info, issues = load_genius_domain_from_folder(
-            base_folder,
-            normalize_utilities=True,
-            keep_issue_names=keep_issue_names,
-            keep_value_names=keep_value_names,
-        )
-        # atlas = GeniusNegotiator.random_negotiator(
+            domain = domain.to_single_issue(stringify=True)
         atlas = GeniusNegotiator(
             java_class_name="agents.anac.y2015.Atlas3.Atlas3",
-            ufun=agent_info[utils[0]]["ufun"],
+            preferences=domain.ufuns[utils[0]],
         )
         # agentx = GeniusNegotiator.random_negotiator(
         agentx = GeniusNegotiator(
             java_class_name="agents.anac.y2015.AgentX.AgentX",
-            ufun=agent_info[utils[1]]["ufun"],
+            preferences=domain.ufuns[utils[1]],
         )
+        neg = domain.make_session(n_steps=100)
         neg.add(atlas)
         neg.add(agentx)
         return neg
@@ -416,33 +426,35 @@ class TestGeniusAgentSessions:
         condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
         reason="No Genius Bridge, skipping genius-agent tests",
     )
-    def test_genius_agents_can_run_on_converted_single_issue_ufun1(self, init_genius):
+    def test_genius_agents_can_run_on_converted_single_issue_preferences1(
+        self, init_genius
+    ):
         neg = self.prepare(utils=(0, 0), single_issue=True)
-        assert neg.pareto_frontier(sort_by_welfare=True)[0] == [
-            # (30.001554125152623, 30.001554125152623)
-            (1.0, 1.0)
-        ]
-        state = neg.run()
-        # pprint(neg.history)
+        best = neg.pareto_frontier(sort_by_welfare=True)[0][0]
+        assert isclose(best[0], 1.0, abs_tol=0.001) and isclose(
+            best[1], 1.0, abs_tol=0.001
+        )
+        neg.run()
         assert neg.agreement is not None
-        #        assert len(neg.history) <= 3
-        assert neg.agreement == {
-            "Laptop-Harddisk-External Monitor": "HP+60 Gb+19'' LCD"
-        }
+        assert len(neg.history) <= 3
+        assert neg.agreement == ("v18",)
 
     @pytest.mark.skipif(
         condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
         reason="No Genius Bridge, skipping genius-agent tests",
     )
-    def test_genius_agents_can_run_on_converted_single_issue_ufun2(self, init_genius):
+    def test_genius_agents_can_run_on_converted_single_issue_preferences2(
+        self, init_genius
+    ):
         neg = self.prepare(utils=(1, 1), single_issue=True)
-        assert neg.pareto_frontier(sort_by_welfare=True)[0] == [(1.0, 1.0)]
-        state = neg.run()
+        best = neg.pareto_frontier(sort_by_welfare=True)[0][0]
+        assert isclose(best[0], 1.0, abs_tol=0.001) and isclose(
+            best[1], 1.0, abs_tol=0.001
+        )
+        neg.run()
         assert neg.agreement is not None
         assert len(neg.history) <= 3
-        assert neg.agreement == {
-            "Laptop-Harddisk-External Monitor": "Macintosh+80 Gb+19'' LCD"
-        }
+        assert neg.agreement == ("v12",)
 
     @pytest.mark.skipif(
         condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
@@ -450,15 +462,20 @@ class TestGeniusAgentSessions:
     )
     def test_genius_agents_can_run_on_converted_single_issue(self, init_genius):
         neg = self.prepare(utils=(0, 1), single_issue=True)
-        assert neg.pareto_frontier(sort_by_welfare=True)[0] == [
-            (0.7715533992081258, 0.8450562871935449),
-            (0.5775524426410947, 1.0),
-            (1.0, 0.5136317604069089),
-            (0.8059990434329689, 0.6685754732133642),
-        ], neg.pareto_frontier(sort_by_welfare=True)[0]
+        front = neg.pareto_frontier(sort_by_welfare=True)[0]
+        assert_almost_equal(
+            front,
+            [
+                (0.7715, 0.845),
+                (0.577, 1.0),
+                (1.0, 0.5136),
+                (0.805, 0.668),
+            ],
+            decimal=2,
+        )
 
-        state = neg.run()
-        # assert len(neg.history) >= 2
+        neg.run()
+        assert len(neg.history) >= 1
         assert neg.agreement is not None
 
     @pytest.mark.skipif(
@@ -476,11 +493,7 @@ class TestGeniusAgentSessions:
         neg.run()
         assert len(neg.history) < 3
         assert neg.agreement is not None
-        assert neg.agreement == {
-            "Laptop": "HP",
-            "Harddisk": "60 Gb",
-            "External Monitor": "19'' LCD",
-        }
+        assert neg.agreement == ("HP", "60 Gb", "19'' LCD")
 
     # @pytest.mark.skipif(
     #     condition=SKIP_IF_NO_BRIDGE and not genius_bridge_is_running(),
@@ -519,51 +532,27 @@ class TestGeniusAgentSessions:
         agent_name1 = "agents.anac.y2015.Atlas3.Atlas3"
         agent_name2 = "agents.anac.y2015.Atlas3.Atlas3"
         single_issue = False
-        keep_issue_names, keep_value_names = False, False
-        utils = (1, 1)
 
-        from negmas import convert_genius_domain_from_folder
-
-        src = pkg_resources.resource_filename(
+        base_folder = pkg_resources.resource_filename(
             "negmas", resource_name="tests/data/Laptop"
         )
-        dst = pkg_resources.resource_filename(
-            "negmas", resource_name="tests/data/LaptopConv1D"
-        )
+        domain = load_genius_domain_from_folder(base_folder)
         if single_issue:
-            assert convert_genius_domain_from_folder(
-                src_folder_name=src,
-                dst_folder_name=dst,
-                force_single_issue=True,
-                cache_and_discretize_outcomes=True,
-                n_discretization=10,
-            )
-            base_folder = dst
-        else:
-            base_folder = src
-        neg, agent_info, issues = load_genius_domain_from_folder(
-            base_folder,
-            keep_issue_names=keep_issue_names,
-            keep_value_names=keep_value_names,
-        )
+            domain = domain.to_single_issue()
         # atlas = GeniusNegotiator.random_negotiator(
         atlas = GeniusNegotiator(
             java_class_name=agent_name1,
-            domain_file_name=base_folder + "/Laptop-C-domain.xml",
-            utility_file_name=base_folder + f"/Laptop-C-prof{utils[0]}.xml",
-            keep_issue_names=keep_issue_names,
-            keep_value_names=keep_value_names,
+            domain_file_name=domain.agenda.name,
+            utility_file_name=domain.ufuns[0].name,
         )
         agentx = GeniusNegotiator(
             java_class_name=agent_name2,
-            domain_file_name=base_folder + "/Laptop-C-domain.xml",
-            utility_file_name=base_folder + f"/Laptop-C-prof{utils[1]}.xml",
-            keep_issue_names=keep_issue_names,
-            keep_value_names=keep_value_names,
+            domain_file_name=domain.agenda.name,
+            utility_file_name=domain.ufuns[1].name,
         )
+        neg = domain.make_session(n_steps=100)
         neg.add(atlas)
         neg.add(agentx)
-        # print(agent_name1, agent_name2, neg.run(timeout=1))
 
 
 if __name__ == "__main__":
